#!/bin/sh
# the next line restarts using tclsh \
    exec tclsh "$0" "$@"

set scriptdir [file dirname [info script]]
set libDirectory [file join $scriptdir lib]
set auto_path [linsert $auto_path 0 $libDirectory]

package require buildsystem
namespace import buildsystem::*

#parse commandline options

buildsystem::parseConfigSourceArgs $::argv

array set globaloptions [buildsystem::getGlobalOptions]
if { $globaloptions("createPackage") == 1 } {
    set createPackage 1
} else {
    set createPackage 0
}

#read the build.cfg
array set globalbuildcfg [buildsystem::read_servers $globaloptions("buildconfigfile")]

#read componentlist
array set globalcomponentlst [buildsystem::read_components "$scriptdir/component.lst"]

#read rootpath.cfg
buildsystem::read_rootpathcfg "rootpath.cfg"

# we directly read out a dependency file which acts more as a meta package
set cnames [buildsystem::read_componentdeps $globaloptions("projectdependenciesfile") componentdeps 1]
buildsystem::read_cs_deps [array get componentdeps] $cnames

########################################
# generate buildconfig.default and Component.in
buildsystem::ConvertBuildconfigInfoToConfigIn
buildsystem::buildConfigAndComponentAll [array get componentdeps] $scriptdir $createPackage

########################################

########################################
# create staging and release dir
buildsystem::createStagingAndReleaseDir 0

########################################

########################################
# generate component Makefile
buildsystem::generateCompMakeFile 0

########################################

if { $createPackage == 1 } {
	buildsystem::generatePackage [array get componentdeps] $scriptdir
}

