package provide buildsystem 1.0

namespace eval buildsystem {
    variable globaloptions
    variable globalbuildcfg
    variable globalcomponentlst
    variable checkpath
    variable globalrootpathlst
    variable versionpath

    set packagetargetdir "./output/package"
    set package_buildconfig_default "buildconfig.default.m4"


    set globaloptions("buildconfigfile") "$::env(HOME)/.server.cfg"

  namespace export parse_cs_commandline read_cs_deps generatePackage
}

# parse configsource commandline options
# options are returned in commandlineoptions as an associative list
proc buildsystem::parseConfigSourceArgs {argv} {
    variable globaloptions
    set options {
        {p.arg     "set to projectdependenciesfile"}
        {h.arg     "set to hardcofile"}
        {s.arg     "set to configurationfile"}
        {o.arg     "set to outputdirectory"}
        {b.arg     "set to buildconfig.default file"}
        {v.arg     "set to version_info file"}
        {l.arg     "use specific release directory "}
        {g.arg     "use specific staging directory "}
        {r         "get the rootpath from the version control system"}
        {t         "enable tracing"}
        {c	   "use ccache"}
        {y     "use legacy releasedir "}
        {z         "create package"}
    }

    set globaloptions("gendefaultbuildconfig") 1
    set globaloptions("ccache") 0
    set globaloptions("differentComponentReleaseDir") 0
    set globaloptions("createPackage") 0
    set globaloptions("version_info") "versioninfo.txt"

    if {[llength $argv]} {
	while {[llength $argv]} {

	    set err [::cmdline::getopt argv {s.arg p.arg h.arg o.arg b.arg v.arg l.arg g.arg r t c d y z} optVar optValue]

	    if {$err == 1} {
		switch $optVar {
		    "p" {
			set globaloptions("projectdependenciesfile") $optValue
		    }
		    "h" {
			set globaloptions("hardcofile") $optValue
		    }
		    "s" {
			set globaloptions("buildconfigfile") $optValue
		    }
		    "o" {
			set globaloptions("outputdirectory") $optValue
		    }
		    "b" {
		        set globaloptions("buildconfigdefault") $optValue
		    }
		    "v" {
		        set globaloptions("version_info") $optValue
		    }
		    "r" {
			set globaloptions("rootpathfromvcs") "1"
		    }
		    "t" {
			log::lvSuppress debug 0
		    }
		    "l" {
            set globaloptions("releasedir") $optValue
            }
            "g" {
            set globaloptions("stagingdir") $optValue
            }
		    "c" {
			set globaloptions("ccache") 1
		}
		    "y" {
                        set globaloptions("legacyreleasedir") 1
		    }
		    "z" {
    			set globaloptions("createPackage") 1
		    }
		}
	    } elseif {$err < 0 } {
		set usage [::cmdline::usage $options]
		puts $usage
		exit 1
	    } else {
		exit 1
	    }
	}
    } else {
	set usage [::cmdline::usage $options]
	puts $usage
	exit 1
    }
}

#read all dependencies recursively
proc buildsystem::read_cs_deps {proccomponentdeps componentnames {indent ""}} {
    variable globaloptions
    variable globalbuildcfg
    variable globalcomponentlst
    variable checkpath
    variable globalrootpathlst
    variable versionpath
    variable package_info

    array set componentdeps $proccomponentdeps

    foreach name $componentnames {
	if { [info exists globalcomponentlst($name)] } {
	    array set servercomponent $globalcomponentlst($name)
	    array set server $globalbuildcfg($servercomponent(servername))
	    lassign [split $server(serveraddress) "/"] proto workspace

	    ### we get our rootpath ###
	    if {[info exists globaloptions("rootpathfromvcs")] == 1} {
		set rootpath [get_perforce_rootpath $servercomponent(servername)]
	    } else {
		set rootpath $globalrootpathlst($servercomponent(servername))
	    }
	    ######################################

	    foreach componentlist $componentdeps($name) {
                array set componentstruct $componentlist
                set version $componentstruct("version")
                set assignment $componentstruct("assignment")
                set depfile $componentstruct("depfile")
                set update_package_info 0
                array set printmap {"EQUALS" "=" "LESSER" "<=" "GREATER" ">="}
                set ilast [expr {"$name" == "[lindex $componentnames end]"}]
                array set iprint {0 "|- " 1 "`- "}
                array set isub   {0 "|  " 1 "   "}
                puts "${indent}$iprint($ilast)$name $printmap($assignment) $version"

		switch $assignment {
		    "EQUALS" {
			# When the assignment is EQUAL
			set path $servercomponent(serverpath)/$version

			if {[info exists versionpath($name)] && [string compare $versionpath($name) $version]} {
				puts "Warning: override $name version $versionpath($name) with $version (forced by $depfile)"
			}

                        set checkpath($name) "$rootpath$path"
			set versionpath($name) $version
			set update_package_info 1

			if {[array exists compdeps]} {
			    array unset compdeps
			}

			set cnames [read_componentdeps "$rootpath$path/Component.dep" compdeps 0]
			if { [array size compdeps] } {
				read_cs_deps [array get compdeps] $cnames "$indent$isub($ilast)"
		        }
		    }
		    "LESSER" {
			set path $servercomponent(serverpath)/$version

			if { [info exists checkpath($name)] } {
			    if { [string compare $rootpath$path $checkpath($name)] <= 0 } {
				set checkpath($name) $rootpath$path
				set versionpath($name) $version
				set update_package_info 1
		            }
			} else {
			    set checkpath($name) "$rootpath$path"
			    set versionpath($name) $version
			    set update_package_info 1
			}

			if {[array exists compdeps]} {
			    array unset compdeps
			}

			set cnames [read_componentdeps "$rootpath$path/Component.dep" compdeps 0]

			if { [array size compdeps] } {
			    read_cs_deps [array get compdeps] $cnames "$indent$isub($ilast)"
			}
		    }
		    "GREATER" {

			set path $servercomponent(serverpath)/$version

			if { [info exists checkpath($name)] } {
			    if { [string compare $rootpath$path $checkpath($name)] >= 0 } {
				set checkpath($name) $rootpath$path
				set versionpath($name) $version
				set update_package_info 1
			    }
			} else {
			    set checkpath($name) "$rootpath$path"
			    set versionpath($name) $version
			    set update_package_info 1
			}

			if {[array exists compdeps]} {
			    array unset compdeps
			}

			set cnames [read_componentdeps "$rootpath$path/Component.dep" compdeps 0]

			if { [array size compdeps] } {
			    read_cs_deps [array get compdeps] $cnames "$indent$isub($ilast)"
			}
		    }
		}
		if { $update_package_info == 1} {
		  if [info exists package_info($name)] {
		    array set component_package_info [lindex $package_info($name) 0]
		  } else {
		    set component_package_info(release_as_binary) 1
		    set component_package_info(enable_debug) 0
		  }

		  set component_package_info(comp_location) $checkpath($name)
		  set component_package_info(relative_package_location) $servercomponent(serverpath)/$version
		  set package_info($name) [list [array get component_package_info]]
		}
	    }
	}
    }
}

proc buildsystem::getGlobalOptions {} {
        variable globaloptions
        return [array get globaloptions]
}

# Create buildconfig.default and Component.all
proc buildsystem::buildConfigAndComponentAll {proccomponentdeps scriptdir generateForPackageCreation} {
        variable checkpath
        variable globaloptions
        variable versionpath
        variable globalrootpathlst
        variable globalcomponentlst
        variable package_info
        variable package_buildconfig_default

        if { [ info exists globaloptions("stagingdir") ] } {
             set stagingdir [ string trim "$globaloptions("stagingdir")" ]
        } else {
             set stagingdir [string trim "$globaloptions("outputdirectory")/staging"]
        }

        array set componentdeps $proccomponentdeps
        foreach name [array names componentdeps] {
            array set componentstruct [lindex $componentdeps($name) 0]

            if { [string compare $versionpath($name) $componentstruct("version")] != 0} {
                puts "WARNING: replacing $checkpath($name) with $componentstruct("version") forced by $componentstruct("depfile")"
                set checkpath($name) "[string range $checkpath($name) 0 [expr [string length $checkpath($name)] - [string length $versionpath($name)] - 1]]$componentstruct("version")"
                set versionpath($name) $componentstruct("version")

                if {$generateForPackageCreation == 1} {
                    array set servercomponent $globalcomponentlst($name)
                    if [info exists package_info($name)] {
                        array set component_package_info [lindex $package_info($name) 0]
                    }

                    set component_package_info(comp_location) $checkpath($name)
                    set component_package_info(relative_package_location) $servercomponent(serverpath)/$componentstruct("version")
                    set package_info($name) [list [array get component_package_info]]
            }
        }
        }

        ########################################
        # generate buildconfig.default and Component.all


        if {$generateForPackageCreation == 1} {
            set buildconfig_package [open "$package_buildconfig_default" "w"]
        }

        set buildconfig [open $globaloptions("buildconfigdefault") "w"]
        set version_info [open $globaloptions("version_info") "w"]
        set componentall [open "$scriptdir/Component.all" "w"]
        puts $componentall "source \"$stagingdir\/buildconfig.in\""
        puts $componentall "source \"Global.in\""

        set levelOne ""
        set levelTwo ""

        set lastpath []

        foreach name [lsort [array names checkpath]] {
            puts $buildconfig "[string toupper $name]=$checkpath($name)"
            puts $version_info "$name=$versionpath($name)"
           if {$generateForPackageCreation == 1} {
                array set comp_package_info [lindex $package_info($name) 0]
                puts $buildconfig_package "[string toupper $name]=__CWD__$comp_package_info(relative_package_location)"
            }
             if {[file exists $checkpath($name)/Component.in]} {
                    array set servercomponent $globalcomponentlst($name)
                    set paths [split $name "_"]

                    set commonCount 0
                    for {set i 0} {$i<[llength $paths] && $i<[llength $lastpath]} {incr i} {
                            if {[lindex $paths $i] == [lindex $lastpath $i]} {
                                    incr commonCount
                            } else {
                                    break
                            }
                            }

                    # Close old menus
                    for {set i $commonCount} {$i<[llength $lastpath]} {incr i} {
                                    puts $componentall "endmenu"
                            }

                    # Open new menus
                    for {set i $commonCount} {$i<[llength $paths]} {incr i} {
                            puts $componentall "menu \"[lindex $paths $i]\""
                    }

	            # write entry in Component.all
	            puts $componentall "source \"$checkpath($name)/Component.in\""

                    set lastpath $paths
            } else {
                    puts "Warning: $name: no Component.in at $checkpath($name)!"
            }
        }

        # Close menus for the last entry
        for {set i 0} {$i<[llength $lastpath]} {incr i} {
                puts $componentall "endmenu"
        }

        close $componentall
        close $buildconfig
        close $version_info

        if {$generateForPackageCreation == 1} {
            close $buildconfig_package
        }
}

# Create staging dir, release dir and Makefile.global
proc buildsystem::createStagingAndReleaseDir {generateForPackageCreation} {
        variable globaloptions
        variable checkpath
        variable package_info
        variable packagetargetdir

        if { [info exists globaloptions("projectdependenciesfile")] } {
		if { $generateForPackageCreation == 1 } {
			array set build_comp_package_info [lindex $package_info(build) 0]
			set package_build_dir "$packagetargetdir/$build_comp_package_info(relative_package_location)"
			set stagingdir "__CWD__/output/staging"
			set releasedir "__CWD__/output/release"
		} else {
                if { [ info exists globaloptions("stagingdir") ] } {
                    set stagingdir [ string trim "$globaloptions("stagingdir")" ]
                } else {
                    set stagingdir [string trim "$globaloptions("outputdirectory")/staging"]
                }
                if { [ info exists globaloptions("releasedir") ] } {
                    set releasedir [ string trim "$globaloptions("releasedir")" ]
                } else {
                    set releasedir [string trim "$globaloptions("outputdirectory")/release"]
                }
		}

		if { $globaloptions("ccache") } {
			set ccache "ccache ";
		} else {
			set ccache "";
		}

                if {$generateForPackageCreation == 1} {
                    file mkdir "$package_build_dir/output/release"
                    set global_makefile [open "$package_build_dir/Makefile.global.m4" "w"]
                } else {
                file mkdir $stagingdir
                file mkdir $releasedir

                puts "using releasedir $releasedir"
                set global_makefile [open "Makefile.global" "w"]
                }

                puts $global_makefile "# Makefile.global"
                puts $global_makefile ""

                if { $generateForPackageCreation == 1 } {
                    puts $global_makefile "export GLOBALMAKEFILE=__CWD__/Makefile.global"
                } else {
                puts $global_makefile "export GLOBALMAKEFILE=$::env(PWD)/Makefile.global"
                }
                puts $global_makefile "export STAGINGDIR=$stagingdir"
                puts $global_makefile "export REAL_RELEASEDIR=$releasedir"
                if { [ info exists globaloptions("legacyreleasedir") ] } {
                    puts $global_makefile "export RELEASEDIR=$releasedir/root_fs"
                } else {
                puts $global_makefile "export RELEASEDIR=$releasedir"
                }
                puts $global_makefile "export ROOT_FS_DIR=$releasedir/root_fs"
                puts $global_makefile "export PKG_CONFIG_LIBDIR=$stagingdir/build/pkg-config"
                puts $global_makefile "export PKG_CONFIG_TOP_BUILD_DIR=$stagingdir"
                if {$generateForPackageCreation == 1} {
                    puts $global_makefile "export OUTPUTDIR=__CWD__/output"
                } else {
                puts $global_makefile "export OUTPUTDIR=$globaloptions("outputdirectory")"
                }
		if { $globaloptions("ccache") } {
			# we can only set one BASEDIR, so set it to the first
			# rootpath and cross our fingers
        		variable globalrootpathlst
		        set myrootpath [lindex [array get globalrootpathlst] 1]
	                puts $global_makefile "export CCACHE_BASEDIR=$myrootpath"
		}
                puts $global_makefile ""
                puts $global_makefile "quote := \""
                puts $global_makefile "empty :="
                puts $global_makefile ""

                puts $global_makefile "export CROSS_COMPILE=\$(subst \$(quote),\$(empty),\$(CONFIG_CROSS_COMPILE))"
                puts $global_makefile "export AS            = \$(CROSS_COMPILE)as"
                puts $global_makefile "export LD            = \$(CROSS_COMPILE)ld"
                puts $global_makefile "export CC            = $ccache\$(CROSS_COMPILE)gcc"
                puts $global_makefile "export CXX           = $ccache\$(CROSS_COMPILE)g++"
                puts $global_makefile "export C++           = $ccache\$(CROSS_COMPILE)c++"
                puts $global_makefile "export G++           = $ccache\$(CROSS_COMPILE)g++"
                puts $global_makefile "export NASM          = \$(CROSS_COMPILE)nasm"
                puts $global_makefile "export AR            = \$(CROSS_COMPILE)ar"
                puts $global_makefile "export NM            = \$(CROSS_COMPILE)nm"
                puts $global_makefile "export STRIP         = \$(CROSS_COMPILE)strip"
                puts $global_makefile "export OBJCOPY       = \$(CROSS_COMPILE)objcopy"
                puts $global_makefile "export OBJDUMP       = \$(CROSS_COMPILE)objdump"
                puts $global_makefile "export RANLIB        = \$(CROSS_COMPILE)ranlib"
                puts $global_makefile "export CC_NOCCACHE   = \$(CROSS_COMPILE)gcc"
                puts $global_makefile "export CXX_NOCCACHE  = \$(CROSS_COMPILE)g++"
                puts $global_makefile "export INSTALL       = /usr/bin/install"
                puts $global_makefile "export VFSL_INSTALL  = \$(OUTPUTDIR)/../scripts/install/vfsl_install.sh"
                if {$generateForPackageCreation == 1} {
                     puts $global_makefile "export BINARY_PACKAGE_CREATION = y"
                }
                puts $global_makefile ""

                set relativeBuildLocation [buildsystem::getRelativeBuildLocation]

                if { [info exists globaloptions("hardcofile")] } {
                        if { [file exists $globaloptions("hardcofile")] } {
                            read_hardcocfg $globaloptions("hardcofile") hardcoconfig

                            if { [info exists hardcoconfig(hardco_hal)] && [string length $hardcoconfig(hardco_hal)] } {
                                if { $generateForPackageCreation == 1 } {
                                    array set hal_comp_package_info [lindex $package_info(sah_hal) 0]
                                    set hardco_hal_dir "__CWD__/$relativeBuildLocation$hal_comp_package_info(relative_package_location)"
                                } else {
                                set hardco_hal_dir $checkpath($hardcoconfig(hardco_hal))
                                }
                            } else {
                                set hardco_hal_dir ""
                            }

                            if { [info exists hardcoconfig(hardco_kernel)] && [string length $hardcoconfig(hardco_kernel)] } {
                                if { $generateForPackageCreation == 1 } {
                                    array set kernel_comp_package_info [lindex $package_info(sah_kernel) 0]
                                    set hardco_kernel_dir "__CWD__/$relativeBuildLocation$kernel_comp_package_info(relative_package_location)"
                                } else {
                                set hardco_kernel_dir $checkpath($hardcoconfig(hardco_kernel))
                                }
                            } else {
                                set hardco_kernel_dir ""
                            }

                            if { [info exists hardcoconfig(hardco_toolchain)] } {
                                if { $generateForPackageCreation == 1 } {
                                    array set toolchain_comp_package_info [lindex $package_info(sah_toolchain) 0]
                                    set hardco_toolchain_dir "__CWD__/$relativeBuildLocation$toolchain_comp_package_info(relative_package_location)"
                                } else {
                            set hardco_toolchain_dir $checkpath($hardcoconfig(hardco_toolchain))
                                }
                            } else {
                                set hardco_toolchain_dir ""
                            }

                            puts $global_makefile "export HARDCO_HAL_DIR=$hardco_hal_dir"
                            puts $global_makefile "export HARDCO_KERNEL_DIR=$hardco_kernel_dir"
                            puts $global_makefile "export HARDCO_TOOLCHAIN_DIR=$hardco_toolchain_dir"

                            if { [info exists hardcoconfig(hardco_hal)] && [string length $hardcoconfig(hardco_hal)] } {
                                puts $global_makefile "include \$(HARDCO_HAL_DIR)/Makefile.hal"
                            }
                        }
                }

                puts $global_makefile ""
                puts $global_makefile "include \$(STAGINGDIR)/components.config"

                close $global_makefile
        }
}

# determine the relative position from the build dir compared to the other directories
proc buildsystem::getRelativeBuildLocation {} {
        variable package_info

        array set build_comp_package_info [lindex $package_info(build) 0]
        set relativePosCount [expr [llength [split $build_comp_package_info(relative_package_location) /]] - 1]
        set relativeBuildLocation ""
        for { set i 0 } { $i < $relativePosCount} {incr i} {
            if [string length $relativeBuildLocation ] {
                set relativeBuildLocation "$relativeBuildLocation/"
            }
                set relativeBuildLocation "$relativeBuildLocation.."
        }

        return $relativeBuildLocation
}

# generate component Makefile
proc buildsystem::generateCompMakeFile {generateForPackageCreation} {
        variable globaloptions
        variable checkpath
        variable versionpath
        variable package_info
        variable packagetargetdir

        if {$generateForPackageCreation == 1} {
           array set build_comp_package_info [lindex $package_info(build) 0]
           set component_makefile [open "$packagetargetdir/$build_comp_package_info(relative_package_location)/Makefile.components" "w"]
           set relativeBuildLocation [buildsystem::getRelativeBuildLocation]
        } else {
        set component_makefile [open "Makefile.components" "w"]
        }

        puts $component_makefile "# Makefile.components"
        puts $component_makefile ""
        puts $component_makefile "include Makefile.global"
        puts $component_makefile ""
        puts $component_makefile ""
        puts $component_makefile "all: components"
        puts $component_makefile ""

        # ################################################

         # collect the paths of the workspaces
        variable globalrootpathlst
        set pathrootlst ""
        foreach path [array names globalrootpathlst] {
          lappend pathrootlst $globalrootpathlst($path)
        }

        foreach name [array names checkpath] {
            array set component_package_info [lindex $package_info($name) 0]
            set enable_debug 0
            set release_as_binary 1
            set relative_package_location "$component_package_info(relative_package_location)"
            set comp_location "$component_package_info(comp_location)"

            set line_sah_debug ""
            if [info exists component_package_info(enable_debug)]  {
                set enable_debug $component_package_info(enable_debug)
                if { $enable_debug == 1 } {
		            set line_sah_debug $component_package_info(debug_options)
                }
            }
            if [info exists component_package_info(release_as_binary)]  {
                set release_as_binary $component_package_info(release_as_binary)
            }

            if {$generateForPackageCreation == 1} {
                set component_location "$relativeBuildLocation$relative_package_location"
            } else {
                set component_location "$checkpath($name)"
            }

            # binary_package_dir is the private root_fs directory for a component
            set binary_package_dir {COMP_PACKAGE_DIR=$(STAGINGDIR)}
            set binary_package_dir "$binary_package_dir/binary_packages/$name"
            set binary_package_dir_location {$(STAGINGDIR)}
            set binary_package_dir_location "$binary_package_dir_location/binary_packages/$name"

            set source_package_dir {COMP_SOURCE_DIR=$(STAGINGDIR)}
            set source_package_dir "$source_package_dir/../package$relative_package_location"

            ### Generate build commands
            puts $component_makefile "$name: [get_componentdeps_string_with_suffix "$checkpath($name)/Component.dep" ""] ${name}_component"
            puts $component_makefile  ""

            puts $component_makefile "${name}_component:"
            if { $generateForPackageCreation == 1 && $release_as_binary == 1 } {
                puts $component_makefile "\t@echo Binary component version, nothing to done for $name\n"
            } else {
            set line {ifeq ($(CONFIG_}
            set line "$line[string toupper $name]),y)"
            puts $component_makefile $line
            set line {}
            puts $component_makefile "ifeq \(\$\(A\),1\)"
            set line "\tinstall -d \"\$\(OUTPUTDIR\)/analysis/$name\""
            puts $component_makefile $line
                set CC_PREFIX "scan-build -analyze-headers --use-cc \"\$\(CC_NOCCACHE\)\" --use-c++ \"\$\(CXX_NOCCACHE\)\" -o \"\$\(OUTPUTDIR\)/analysis/$name\" --keep-going"
                set line "\t$CC_PREFIX make -C $component_location -f Component.mk"
            puts $component_makefile $line
                set line "\t$CC_PREFIX make $component_location -f Component.mk install"
            puts $component_makefile $line
            puts $component_makefile "else"
                set line "\tmake -C $component_location -f Component.mk $line_sah_debug"
            puts $component_makefile $line
                set line "\tmake -C $component_location -f Component.mk install $line_sah_debug"
            puts $component_makefile $line
            puts $component_makefile "endif"
            puts $component_makefile "endif"
            puts $component_makefile ""
            }

            ### generate release commands
            set strip_cmd {STRIP=$(STRIP)}
            puts $component_makefile "${name}_release: [get_componentdeps_string_with_suffix "$checkpath($name)/Component.dep" "_release"] ${name}_component_release"
            puts $component_makefile  ""

            puts $component_makefile "${name}_component_release:"
            set line {ifeq ($(CONFIG_}
            set line "$line[string toupper $name]),y)"
            puts $component_makefile $line

            if { $generateForPackageCreation == 1 && $release_as_binary == 1 } {
                set line "ifneq (\"\$(shell cat $component_location/Component.mk | grep '^\[^\\#\[:space:\]\].*:' | grep \"release_package:\")\",\"\")"
            puts $component_makefile $line
                set line "\tmake -C $component_location -f Component.mk release_package $strip_cmd $line_sah_debug $binary_package_dir"
                puts $component_makefile $line
                puts $component_makefile "else"
                set line "\tmake -C $component_location -f Component.mk release $strip_cmd $line_sah_debug"
                puts $component_makefile $line
	    puts $component_makefile "endif"
            } else {
                set line "\tmake -C $component_location -f Component.mk release $strip_cmd $line_sah_debug"
                puts $component_makefile $line

                if { $globaloptions("createPackage") == 1 } {
                    set line "ifneq (\"\$(shell cat $component_location/Component.mk | grep '^\[^\\#\[:space:\]\].*:' | grep \"build_package:\")\",\"\")"
                    puts $component_makefile $line
                    puts $component_makefile "\trm -rf $binary_package_dir_location"
                    set line "\tmake -C $component_location -f Component.mk build_package $strip_cmd $line_sah_debug $binary_package_dir $source_package_dir"
                    puts $component_makefile $line
	    puts $component_makefile "endif"
                }
            }

      puts $component_makefile "endif"
	    puts $component_makefile ""

      #generate make clean commands
      puts $component_makefile "${name}_clean: [get_componentdeps_string_with_suffix "$checkpath($name)/Component.dep" "_clean"] ${name}_component_clean"
      puts $component_makefile  ""

      puts $component_makefile "${name}_component_clean:"
      if { $generateForPackageCreation == 1 && $release_as_binary == 1 } {
         puts $component_makefile "\t@echo Binary component version, nothing to be done for $name\n"
      } else {
      set line {ifeq ($(CONFIG_}
      set line "$line[string toupper $name]),y)"
      puts $component_makefile $line
        set line "\tmake -C $component_location -f Component.mk clean $line_sah_debug"
      puts $component_makefile $line
      puts $component_makefile "endif"
      puts $component_makefile ""
      }

      #entry for counting lines of code per component
      puts $component_makefile "${name}_countlines: [get_componentdeps_string_with_suffix "$checkpath($name)/Component.dep" "_countlines"] ${name}_component_countlines"
      puts $component_makefile  ""

      puts $component_makefile "${name}_component_countlines:"
      puts $component_makefile "\tperl ./scripts/cloc/cloc-1.08.pl --quiet --progress-rate=0 $component_location"
      puts $component_makefile ""

      #entry for gathering license files
      puts $component_makefile "${name}_getlicenses: [get_componentdeps_string_with_suffix "$checkpath($name)/Component.dep" "_getlicenses"] ${name}_component_getlicenses"
      puts $component_makefile  ""

      puts $component_makefile "${name}_component_getlicenses:"
      puts $component_makefile "\tperl ./scripts/getlicenses.pl ${name} $component_location"
      puts $component_makefile ""

      puts $component_makefile "${name}_getsourcecode:"
      puts $component_makefile  "\tperl ./scripts/getsourcecode.pl TARGET=COLLECT COMPONENT=$name PATH=$component_location CODE_SOURCE=\$\{CODE_SOURCE\} LICENSE=\$\{LICENSE\} STAGINGDIR=\$\(STAGINGDIR\) ROOT_LIST=[join $pathrootlst ","]"
      puts $component_makefile ""

      #generate make doc commands
      # do not add dependencies... otherwise components that have no doc target will cause documentation not to be build
      puts $component_makefile "${name}_doc: ${name}_component_doc"
      puts $component_makefile  ""

      puts $component_makefile "${name}_component_doc:"
      set line {ifeq ($(CONFIG_}
      set line "$line[string toupper $name]),y)"
      puts $component_makefile $line
      set docdir "\$\(OUTPUTDIR\)/doc"
      set docsourcedir "$docdir/source/$name"
      set docnosourcedir "$docdir/nosource/$name"

      set line "\t@echo \"********************** building ${name} documentation \""
      puts $component_makefile $line
      set line "ifneq (\"\$(shell cat $component_location/Component.mk | grep '^\[^\\#\[:space:\]\].*:' | grep \"doc:\")\",\"\")"
      puts $component_makefile $line
      set line "\tinstall -d $docsourcedir"
      puts $component_makefile $line
      set line "\tinstall -d $docnosourcedir"
      puts $component_makefile $line
      set line "\tmake -k -C $component_location -f Component.mk doc COMPONENTNAME=\"$name\" DOCDIR=\"$docsourcedir\" SOURCE_BROWSER=\"YES\""
      puts $component_makefile $line
      set line "\tmake -k -C $component_location -f Component.mk doc COMPONENTNAME=\"$name\" DOCDIR=\"$docnosourcedir\" SOURCE_BROWSER=\"NO\""
      puts $component_makefile $line
      puts $component_makefile "else"
      puts $component_makefile "\t@echo doc target not supported"
      puts $component_makefile "endif"
      puts $component_makefile "endif"
      puts $component_makefile ""


      #generate make test commands
      # do not add dependencies... otherwise components that have no tests target will cause tests not to be build
      puts $component_makefile "${name}_test: ${name}_component_test"
      puts $component_makefile  ""


      puts $component_makefile "${name}_component_test:"
      set line {ifeq ($(CONFIG_}
      set line "$line[string toupper $name]),y)"
      puts $component_makefile $line

      set line "\t@echo \"********************** building ${name} test \""
      puts $component_makefile $line
      set line "ifeq (\"\$(shell cat $component_location/Component.mk | grep '^\[^\\#\[:space:\]\].*:' | grep \"^test:\" | sed \'s/\\(:\\).*/\\1/\')\",\"test:\")"
      puts $component_makefile $line
      set localtestdir "\$(OUTPUTDIR)/test/$name"
      set line "\tinstall -d $localtestdir"
      puts $component_makefile $line
      set line "\techo export COMPONENT_NAME=$name > $localtestdir/component.name"
      puts $component_makefile $line
      set line "\techo export COMPONENT_LOCATION=$component_location > $localtestdir/component.location"
      puts $component_makefile $line
      set line "\techo export COMPONENT_VERSION=$versionpath($name) > $localtestdir/component.version"
      puts $component_makefile $line

      set testtype "remote"
      if {[file exists "output/test/test.config"] == 1} {
        #  Slurp up the data file
        set fp [open "output/test/test.config" r]
        set file_data [read $fp]
        close $fp
        #  Process data file
        set data [split $file_data "\n"]
        foreach line $data {
          if {[string length $line] > 1 } {
            set x [checkconfig "TESTTYPE" $line]
            if {$x == 1} { set testtype [setconfig $line] }
          }
        }
      }


      #gather configuration information needed for xml
      set spin [exec cat output/staging/buildconfig.info | grep BUILD_SPIN_NUMBER= | sed -e s/BUILD_SPIN_NUMBER=//]
      set configlocation [exec cat output/staging/buildconfig.info |  grep BUILD_DEF | sed -e s/BUILD_DEF=//]
      set configindex [expr [ string first "/config/" $configlocation ] + 8 ]
      set strippedlocation [ string range $configlocation $configindex [ string length $configlocation ] ]
      set dirparts [split $strippedlocation "/"]
      set version ""
      foreach dirpart $dirparts {
	if [string length $dirpart] {
	  if ![string compare $dirpart [string toupper $dirpart] ] {
         	set version "$version/$dirpart"
	  }
	}
      }
      set testconfig [string range $strippedlocation [string length $version] [string length $strippedlocation]]
      set testconfigversion [string range $version 1 [string length $version]]

      set line "\tmake -k -C $component_location -f Component.mk test COMPONENTNAME=\"$name\" LOCALTESTDIR=\"$localtestdir\" "
      puts $component_makefile $line

      set line "\techo \"<?xml version=\\\"1.0\\\"?>\" > \$(OUTPUTDIR)/test/$name/component.testlist.xml"
      puts $component_makefile $line

      set line "\techo \"<testrun config=\\\"$testconfig\\\" configversion=\\\"$testconfigversion\\\" spinnumber=\\\"$spin\\\">\" >> \$(OUTPUTDIR)/test/$name/component.testlist.xml"
      puts $component_makefile $line

      #<component id="nmc" version="REL/2011-01-12_V1.0.0">
      set line "\techo \"<component id=\\\"$name\\\" version=\\\"$versionpath($name)\\\">\" >> \$(OUTPUTDIR)/test/$name/component.testlist.xml"
      puts $component_makefile $line

      #	for plan in $(OUTPUTDIR)/test/$name/*_testplan.sh; do if [ "$($plan --type)" = "local_standalone" ]; $plan --list >> $(OUTPUTDIR)/test/$name/component.testlist.xml; fi done
      set line "\t@for plan in \$(OUTPUTDIR)/test/$name/*_testplan.sh; do ptype=`\$\$plan --type`; echo \$\$ptype; if \[ \"\$\$ptype\" = $testtype \]; then \$\$plan --list; fi done"
      puts $component_makefile $line

      set line "\techo \"</component>\" >> \$(OUTPUTDIR)/test/$name/component.testlist.xml"
      puts $component_makefile $line

      set line "\techo \"</testrun>\" >> \$(OUTPUTDIR)/test/$name/component.testlist.xml"
      puts $component_makefile $line

      puts $component_makefile "else"
      puts $component_makefile "\t@echo test target not supported"
      puts $component_makefile "endif"
      puts $component_makefile "endif"
      puts $component_makefile ""



        #############################
        }

        ######################################################

        puts $component_makefile "components: [get_componentdeps_string_with_suffix $globaloptions("projectdependenciesfile") ""]"
        puts $component_makefile ""

        puts $component_makefile "releasecomp: [get_componentdeps_string_with_suffix $globaloptions("projectdependenciesfile") "_release"]"
        puts $component_makefile ""

        puts $component_makefile "release: releasecomp"
        puts $component_makefile ""
        puts $component_makefile ""

        puts $component_makefile "cleancomp: [get_componentdeps_string_with_suffix $globaloptions("projectdependenciesfile") "_clean"]"
        puts $component_makefile ""
        puts $component_makefile "clean: cleancomp"
        puts $component_makefile ""

        set line "doccomp: "
        foreach name [array names checkpath] {
          set line "$line ${name}_doc"
        }
        puts $component_makefile $line
        puts $component_makefile ""
        puts $component_makefile "doc: doccomp"

        puts $component_makefile ""

        set line "testcomp: "
        foreach name [array names checkpath] {
          set line "$line ${name}_test"
        }
        puts $component_makefile $line

        #gather configuration information needed for xml
        set spin [exec cat output/staging/buildconfig.info | grep BUILD_SPIN_NUMBER= | sed -e s/BUILD_SPIN_NUMBER=//]
        set configlocation [exec cat output/staging/buildconfig.info |  grep BUILD_DEF | sed -e s/BUILD_DEF=//]
        set configindex [expr [ string first "/config/" $configlocation ] + 8 ]
        set strippedlocation [ string range $configlocation $configindex [ string length $configlocation ] ]
        set dirparts [split $strippedlocation "/"]
        set version ""
        foreach dirpart $dirparts {
	  if [string length $dirpart] {
	    if ![string compare $dirpart [string toupper $dirpart] ] {

		set version "$version/$dirpart"
	    }
	  }
        }
        set testconfig [string range $strippedlocation [string length $version] [string length $strippedlocation]]
        set testconfigversion [string range $version 1 [string length $version]]

        #add the global header
        puts $component_makefile "\techo \"<?xml version=\\\"1.0\\\"?>\" > \$(OUTPUTDIR)/test/testlist.xml"
        puts $component_makefile "\techo \"<testrun config=\\\"$testconfig\\\" configversion=\\\"$testconfigversion\\\" spinnumber=\\\"$spin\\\">\" >> \$(OUTPUTDIR)/test/testlist.xml"

        foreach name [array names checkpath] {
        puts $component_makefile "\t@if \[ -f \$(OUTPUTDIR)/test/${name}/component.testlist.xml \] && \[ `cat \$(OUTPUTDIR)/test/${name}/component.testlist.xml | wc -l` -gt 5 \]; then cat \$(OUTPUTDIR)/test/${name}/component.testlist.xml | grep -v \"<?xml version=\" | grep -v \"<testrun\" | grep -v \"testrun>\" >> \$(OUTPUTDIR)/test/testlist.xml; fi"
          set line "$line ${name}_test"
        }

        #add the global xml footer
        puts $component_makefile "\techo \"</testrun>\" >> \$(OUTPUTDIR)/test/testlist.xml"

        exec install -d output/test
        exec echo $spin > output/test/test.spin
        exec echo $testconfig > output/test/test.configlocation
        exec echo $testconfigversion > output/test/test.configversion

        puts $component_makefile ""
        puts $component_makefile "tests: testcomp"

        puts $component_makefile ""

        puts $component_makefile "countlines: [get_componentdeps_string_with_suffix $globaloptions("projectdependenciesfile") "_countlines"]"
        puts $component_makefile ""

        puts $component_makefile "getlicenses: [get_componentdeps_string_with_suffix $globaloptions("projectdependenciesfile") "_getlicenses"]"
        puts $component_makefile ""

         ################ begin get_source_code
        puts $component_makefile "getsourcecode_release: getsourcecode"
        puts $component_makefile  "\tperl ./scripts/getsourcecode.pl TARGET=RELEASE RELEASEDIR=\$\(RELEASEDIR\) STAGINGDIR=\$\(STAGINGDIR\)"
        puts $component_makefile ""

        set souce_code_deps ""
        foreach name [array names checkpath] {
          set souce_code_deps "${souce_code_deps} ${name}_getsourcecode"
        }

        puts $component_makefile "getsourcecode: getsourcecode_clean $souce_code_deps"
        puts $component_makefile ""

        puts $component_makefile "getsourcecode_clean:"
        puts $component_makefile  "\tperl ./scripts/getsourcecode.pl TARGET=CLEAN STAGINGDIR=\$\(STAGINGDIR\)"
        puts $component_makefile ""
        ################ end get_source_code


        puts $component_makefile ".PHONY: all components release"
        close $component_makefile

}

# converts the buildconfig.info to a config.in so it can be used in menuconfigs
proc buildsystem::ConvertBuildconfigInfoToConfigIn {} {
     variable globaloptions
     variable checkpath

     # this list needs to be hardcoded because otherwise we don't know which ones need to be used as a boolean as well!
     set boollist "BUILD_HARDWARE BUILD_CONFIGTYPE BUILD_PROJECT BUILD_CUSTOMER"
     set skiplist "BUILD_VER BUILD_DEF BUILD_START BUILD_HOST BUILD_USER"

     if { [ info exists globaloptions("stagingdir") ] } {
         set stagingdir [ string trim "$globaloptions("stagingdir")" ]
     } else {
        set stagingdir [string trim "$globaloptions("outputdirectory")/staging"]
     }

     set filename "$stagingdir/buildconfig.info"
     set buildconfig [open "$stagingdir/buildconfig.in" "w"]
     if { [ catch { set fp [ open "$filename" "r" ] } ] } {
         puts "error occured while reading $filename"
         exit
     }
     #set line "menu \"build settings\""
     #puts $buildconfig $line
     set line ""
     fconfigure $fp -buffering line
     gets $fp data
     while {$data != ""} {
     	set skip 0
     	#puts $data
          set record [ split $data "="]
          # create string config
		  foreach l $skiplist {
		  	#puts "$l = [lindex $record 0 ]"
		  	#after 1000
			if { [ string equal $l [ lindex $record 0 ] ] } {
				set skip 1
				#puts "break";
				break;
			}
		  }
		  if {$skip} {
		  	gets $fp data;
		  	continue
		  }
          set line "config [lindex $record 0 ]\n"
          set line "$line \tstring\n"
          set line "$line \tdefault \"[lindex $record 1]\"\n"
          set line "$line \thelp\n\t\t[lindex $record 0] parsed from buildconfig.info"
          puts $buildconfig $line
          foreach index $boollist {
               if { [string equal $index [lindex $record 0]] } {
                    # create bool config
                    set line "config [string toupper [lindex $record 1] ]\n"
                    set line "$line \tbool\n\t"
                    set line "$line default y\n"
                    set line "$line \thelp\n\t\t[lindex $record 1] parsed from buildconfig.info"
                    puts $buildconfig $line
               }
          }
          gets $fp data
     }
     #set line "endmenu"
     puts $buildconfig $line
     close $fp
     close $buildconfig
}

proc buildsystem::generatePackage {proccomponentdeps scriptdir} {
        variable globaloptions
        variable checkpath
        variable versionpath
        variable package_info
        variable packagetargetdir
        variable package_buildconfig_default

        array set componentdeps $proccomponentdeps

        puts "\n\n---- GeneratePackage ----\n"

        set fp [open "createPackageCopyStagingDir.sh" "w"]
        puts $fp "#!/bin/bash\n"

        # clear the package directory
        exec rm -rf "$packagetargetdir"

        # collect the paths of the workspaces
        variable globalrootpathlst
        set pathrootlst ""
        foreach path [array names globalrootpathlst] {
          lappend pathrootlst $globalrootpathlst($path)
        }

        # loop through all the components
        foreach name [array names checkpath] {
           set enable_debug 0
           set release_as_binary 1
           set comp_location ""
           set relative_package_location ""

           if [info exists package_info($name)] {
              array set component_package_info [lindex $package_info($name) 0]
              if [info exists component_package_info(enable_debug)]  {
                set enable_debug $component_package_info(enable_debug)
              }
              if [info exists component_package_info(release_as_binary)]  {
                set release_as_binary $component_package_info(release_as_binary)
              }

              set comp_location $component_package_info(comp_location)
              set relative_package_location $component_package_info(relative_package_location)

           }

           exec mkdir -p $packagetargetdir$relative_package_location

           if {$release_as_binary == 1} {
               exec cp -f $comp_location/Component.mk $packagetargetdir$relative_package_location
               if [file exists $comp_location/Component.lic] {
                   exec cp -f $comp_location/Component.lic $packagetargetdir$relative_package_location
               }
           } else {
               exec cp -rf $comp_location $packagetargetdir$relative_package_location/..
           }

          if {$release_as_binary == 0 || $enable_debug == 1} {
              puts "--> $name: release_as_binary($release_as_binary) enable_debug($enable_debug)"
          }
        }

        puts "\n"

        buildsystem::createStagingAndReleaseDir 1
        buildsystem::generateCompMakeFile 1

        # -> at this point we have all the necessary information, thus create a file to be
        # executed when the staging_dir is ready to be copied to the package directory
        if { [ info exists globaloptions("stagingdir") ] } {
            set stagingdir [ string trim "$globaloptions("stagingdir")" ]
        } else {
            set stagingdir [string trim "$globaloptions("outputdirectory")/staging"]
        }
        array set build_comp_package_info [lindex $package_info(build) 0]
        set package_build_dir "$packagetargetdir$build_comp_package_info(relative_package_location)"
        puts $fp "cp -rf $stagingdir $package_build_dir/output"
        puts $fp "cp -f output/staging/buildconfig.info $package_build_dir/output/staging/buildconfig.info.m4"
        puts $fp "rm -rf $package_build_dir/build.sh"
        puts $fp "cp create_package.sh $package_build_dir/build.sh"
        puts $fp "rm -rf $package_build_dir/Makefile"
        puts $fp "cp Makefile $package_build_dir/Makefile"
        puts $fp "cp -f $package_buildconfig_default $package_build_dir/$package_buildconfig_default\n"
        close $fp
        exec chmod +x createPackageCopyStagingDir.sh
}
