package provide buildsystem 1.0

namespace eval buildsystem {
      variable globalbuildcfg
      variable globalcomponentlst
      variable checkpath
      variable rootpaths
      variable versionpath

  namespace export parse_gs_commandline read_gs_deps
}

# parse getsource commandline options
# options are returned in commandlineoptions as an associative list
proc buildsystem::parseGetSourceArgs {argv} {
    variable globaloptions

    set options {
        {p.arg     "set to projectdependenciesfile"}
	{s.arg     "set to configurationfile"}
        {t         "enable tracing"}
	{n         "do not track in workspace"}
	{nf        "do not force sync operations"}
        {z         "create package"}
    }

    set globaloptions("regendefconfig") 0
    set globaloptions("notrackws") 0
    set globaloptions("noforcesync") 0
    set globaloptions("gendefaultbuildconfig") 1
    set globaloptions("createPackage") 0

    if {[llength $argv]} {
	while {[llength $argv]} {

	    set err [::cmdline::getopt argv {s.arg p.arg t n nf r z} optVar optValue]

	    if {$err == 1} {
		switch $optVar {
		    "r" {
			set globaloptions("regendefconfig") $optValue
			
		    }
		    "p" {
			set globaloptions("projectdependenciesfile") $optValue
			
		    }
		    "s" {
			set globaloptions("buildconfigfile") $optValue			
		    }
		    "n" {
			set globaloptions("notrackws") 1
		    }
		    "nf" {
			set globaloptions("noforcesync") 1
		    }
		    "t" {
			log::lvSuppress debug 0
		    }
		    "z" {
			    set globaloptions("createPackage") 1
		}
		}
	    } elseif {$err < 0 } {
		set usage [::cmdline::usage $options]
		puts $usage
		exit 1
	    } else {
		exit 1
	    }
	} 
    } else {
	set usage [::cmdline::usage $options]
	puts $usage
	exit 1
    }

}

#read all dependencies recursively
#fill in checkpath
proc buildsystem::read_gs_deps {proccomponentdeps componentnames {indent ""}} {
    variable globalbuildcfg
    variable globalcomponentlst
    variable checkpath
    variable rootpaths
    variable versionpath

    array set componentdeps $proccomponentdeps

    foreach name $componentnames {

	if { [info exists globalcomponentlst($name)] } {
	    array set servercomponent $globalcomponentlst($name)
	    array set server $globalbuildcfg($servercomponent(servername))
	    lassign [split $server(serveraddress) "/"] proto workspace

	    ### we get our rootpath ###
	    set rootpath [buildsystem::get_perforce_rootpath $servercomponent(servername)]
	    set rootpaths($servercomponent(servername)) "$rootpath"

	    foreach componentlist $componentdeps($name) {
                array set componentstruct $componentlist
                set version $componentstruct("version")
                set assignment $componentstruct("assignment")
                array set printmap {"EQUALS" "=" "LESSER" "<=" "GREATER" ">="}
                set ilast [expr {"$name" == "[lindex $componentnames end]"}]
                array set iprint {0 "|- " 1 "`- "}
                array set isub   {0 "|  " 1 "   "}
                puts "${indent}$iprint($ilast)$name $printmap($assignment) $version"

                set path $servercomponent(serverpath)/$version
                set componentpath //$workspace/[string range $path 1 [string length $path]]

                set updated 0
                if { [info exists checkpath($name)] } {
                    switch $assignment {
                        "EQUALS" {
                            set cmpres [string compare $componentpath $checkpath($name)]
                            if { $cmpres < 0 } {
                                puts "WARNING: replacing $checkpath($name) with $componentstruct("version")"
                            }
                            if { $cmpres != 0 } {
                                set updated 1
                            }
                        }
                        "LESSER" {
                            if { [string compare $componentpath $checkpath($name)] <= 0 } {
				puts "Error: Incompatible dependencies: $checkpath($name) $componentdeps($name)"
				exit 1
			    }
                        }
                        "GREATER" {
			    if { [string compare $componentpath $checkpath($name)] >= 0 } {
				set updated 1
			    }
			}
		    }
                } else {
                    set updated 1
		}

		if { $updated != 1 } {
                    continue
		}

		set checkpath($name) $componentpath
		set versionpath($name) $version

		log::log debug "Executing perforce command: p4 -p $proto -c $workspace print $componentpath/Component.dep"
		set tempfile fileutil::tempfile
                if { [catch { exec p4 -p $proto -c $workspace print $componentpath/Component.dep > $tempfile} msg] } {
                    #ignore error
                }
                
		if {[array exists compdeps]} {
		    array unset compdeps
		}
                
		set cnames [read_componentdeps $tempfile compdeps 0]
                
		exec rm -f $tempfile
                
		if { [array size compdeps] } {
		    read_gs_deps [array get compdeps] $cnames "$indent$isub($ilast)"
		}
	    }
	}
    }
}

proc buildsystem::getP4Source {proccomponentdeps} {
        variable versionpath
        variable checkpath
        variable globalcomponentlst
        variable globalbuildcfg
        variable globaloptions
        
        array set componentdeps $proccomponentdeps
        
        foreach name [array names componentdeps] {
                array set componentstruct [lindex $componentdeps($name) 0]

                if { [string compare $versionpath($name) $componentstruct("version")] != 0} {
                        puts "WARNING: replacing $checkpath($name) with $componentstruct("version")"
                        set checkpath($name) "[string range $checkpath($name) 0 [expr [string length $checkpath($name)] - [string length $versionpath($name)] - 1]]$componentstruct("version")"
                }
        }
        foreach name [array names checkpath] {
                array set servercomponent $globalcomponentlst($name)
                array set server $globalbuildcfg($servercomponent(servername))

                lassign [split $server(serveraddress) "/"] proto workspace

                if { $globaloptions("notrackws") == "1" } {
                	set syncopt "-p"
                } elseif { $globaloptions("noforcesync") == "1" } {
                	# -q is needed so that p4 sync does not show "file(s) up-to-date" on stderr, but it might hide some real failures
                	set syncopt "-q"
		} else {
                	set syncopt "-f"
		}

                log::log debug "Executing perforce command: p4 -p $proto -c $workspace sync $syncopt $checkpath($name)/..."
                puts  "Syncing $checkpath($name)"
                if { $globaloptions("regendefconfig") == "1" } {
                    catch {exec p4 -p $proto -c $workspace sync $syncopt $checkpath($name)/Makefile }
                    catch {exec p4 -p $proto -c $workspace sync $syncopt $checkpath($name)/Makefile.hal }
                    catch {set result [exec p4 -p $proto -c $workspace sync $syncopt $checkpath($name)/Component.*] }
                } else {
                    catch {set result [exec p4 -p $proto -c $workspace sync $syncopt $checkpath($name)/...] }
                }
        }
}

proc buildsystem::writeRootPathFile {filename} {
    variable rootpaths
    set rootpathfile [open $filename w]
    foreach name [array names rootpaths] {
        puts $rootpathfile "$name=$rootpaths($name)"
    }
    close $rootpathfile
}

proc buildsystem::getCheckPath {} {
    variable checkpath
    return [array get checkpath]
}


