#!/usr/bin/perl

use strict;
#use warnings;

# global variables
my $g_target_clean = 0;
my $g_target_release = 0;
my $g_target_collect = 0;
my $g_statging_dir = "";
my $g_licensefile = "Component.lic";
my $g_licensefile_name;
my $g_output_dir = "sources";
my $g_staging_output_dir_name;

my $g_component;
my $g_sourcedir;
my $g_stagingdir;
my $g_releasedir;
my $req_licenses;
my @req_licenses_ar;
my @req_code_source_ar;
my $req_code_source;
my @g_globalrootpathlst;
my @g_dir_array;

# first input parameter: a comma separated list
# second input parameter: the value to look for
# returns: 0 -> not found, > 0 -> number of occurances in the array
sub array_contains
{
  my @list = split(',', $_[0]);
  my $value = $_[1];

  if (length($value))
  {
    # we need an exact match -> gpl != gplv2
    my $val;
    foreach $val (@list)
    {
      if ("$val" eq "$value")
      {
        return 1;
      }
    }
  }

  return 0;
}

# input parameter: a comma separated list containing the specified code sources
# returns: 0 -> not found, > 0 -> number of occurances in the array
sub code_source_matches_requested
{
  my @spec_list = split(',', $_[0]);
  my $reqvalue;
  my $specvalue;

  foreach $reqvalue (@req_code_source_ar)
  {
    foreach $specvalue (@spec_list)
    {
      if ("$specvalue" eq "$reqvalue")
      {
        return 1;
      }
    }
  }

  return 0;
}

# input parameter: a comma separated list containing the specified licenses
# returns: 0 -> not found, > 0 -> number of occurances in the array
sub license_matches_requested
{
  my @spec_list = split(',', $_[0]);
  my $reqvalue;
  my $specvalue;

  foreach $reqvalue (@req_licenses_ar)
  {
    foreach $specvalue (@spec_list)
    {
      if ("$specvalue" eq "$reqvalue")
      {
        return 1;
      }
    }
  }

  return 0;
}

# function: copy the content of the directories keeping in mind the requested code_source and license types
sub copy_sources
{
  my $href;
  for $href ( @g_dir_array )
  {
    my $path = $href->{ dir };                 # the current dir
    my $code_source = $href->{ code_source };  # the code_source belonging to the current dir
    my $license = $href->{ license };          # the license belonging to the current dir

    # * can be used as wildcard
    if (("$req_code_source" eq "*" || code_source_matches_requested($code_source)) &&
        ("$req_licenses" eq "*" || license_matches_requested($license)))
    {
      # we need to copy the content of the current directory
      my $target_path = "$path";
      my $rootpath;
      foreach $rootpath (@g_globalrootpathlst)
      {
        $target_path =~ s/$rootpath//;
      }
      $target_path="$g_staging_output_dir_name$target_path";

      if (!-d "$path")
      {
        die ">>>>>>> ERROR: source directory \"$path\" does not exist";
      }

      # create the target dir
      system("install -d \"$target_path\"");

      my $files = "";
      my $file;

      for $file (`find \"$path\" -maxdepth 1 -type f`)
      {
         # remove newline and carriage return
        $file =~ s/\n//;
        $file =~ s/\r//;

        # make sure character $ is not interpreted
        $file =~ s/\$/\\\$/;
         $files="$files \"${file}\"";
      }

      if (length($files))
      {
        print "Copying path=$path code_source=$code_source license=$license\n";
        system("cd \"$path\" && cp -f -t \"$target_path\" $files");
      }

      my $links = "";
      my $link;
      # recreate links
      for $link (`find \"$path\" -maxdepth 1 -type l`)
      {
         # remove newline and carriage return
        $link =~ s/\n//;
        $link =~ s/\r//;

        # make sure character $ is not interpreted
        $link =~ s/\$/\\\$/;
        $links="$links \"${link}\"";
      }

       if (length($links))
       {
         system("cd \"$path\" && cp -f -d -t \"$target_path\" $links");
      }
    }
  }
}


# function: compares the def_path_new (the new code_source/license def in the .lic file)
#           and the def_path_prev (the previous path that defines the code_source/ license for the specified path)
# returns 1: the def_path_new is more precise than def_path_prev
# returns 0: ignore the settings for def_path_new
sub is_more_specific
{
  my $specified_path = $_[0];         # the current dir we are looking at
  my $def_path_prev = $_[1];          # the previous path definition form the .lic file
  my $def_path_new = $_[2];           # the new path definition form the .lic file

  my $rc = 0;

  # first check whether this path_definition is valid for the current dir
  my $temp = "$specified_path";
  $temp =~ s/^$def_path_new//;

  my $strlen_temp = length($temp);
  my $strlen_specified_path = length($specified_path);
  if ($strlen_temp != $strlen_specified_path)  # we were able to substite the beginning of the path, thus this is a valid def
  {
    if (!length($def_path_prev))
    {
      # nothing defined yet, thus this must be more specific
      $rc =1;
    }
    else
    {
      # compare the current path def with the new one
      my $temp_prev = "$specified_path";
      $temp_prev =~ s/^$def_path_prev//;
      my $strlen_temp_prev = length($temp_prev);

      if ($strlen_temp < $strlen_temp_prev)
      {
        $rc = 1;
      }
    }
  }

  return $rc;
}

# function: handles a line of a license file, fills the directory array with the correct information
# first input parameter: the line
# second input parameter: the linenumber
sub handle_license_file_line
{
  my $actual_line =$_[0];
  my $line_nmbr =$_[1];

  # remove newline and carriage return
  $actual_line =~ s/\n//;
  $actual_line =~ s/\r//;

  my @list = split(':', $actual_line);
  my $path = $list[0];
  my $code_source = $list[1];
  my $license = $list[2];

  if (length($path))
  {
    if (!length($code_source) || !length($license))
    {
      print ">>>>>>> ERROR $g_component: Invalid syntax found in license file: \"$g_licensefile_name\", line $line_nmbr -> $actual_line \n"
    }
    else
    {
      if ( "$path" eq "/")
      {
        $path = "";
      }

      my $actual_path = "$g_sourcedir$path";

      if (!-d $actual_path)
      {
         print ">>>>>>> ERROR $g_component: License file: \"$g_licensefile_name\", line $line_nmbr -> $actual_line - directory \"$actual_path\" does not exist\n"
      }

      my $href;
      for $href ( @g_dir_array ) {

        if (is_more_specific($href->{ dir }, $href->{ defined_dir } , "$actual_path"))
        {
          $href->{ defined_dir } = "$actual_path";
          $href->{ code_source } = "$code_source";
          $href->{ license } = "$license";
        }
      }
    }
  }
}


########################
# main loop starts here
########################

# retrieve the commandline options
my $argnum;
foreach $argnum (0 .. $#ARGV)
{
  my $option = $ARGV[$argnum];

  if ($option =~ m/TARGET=CLEAN/)
  {
    $g_target_clean = 1;
  }
  elsif ($option =~ m/TARGET=RELEASE/)
  {
    $g_target_release = 1;
  }
  elsif ($option =~ m/TARGET=COLLECT/)
  {
    $g_target_collect = 1;
  }
  elsif ($option =~ m/COMPONENT=/)
  {
    $g_component = $option;
    $g_component =~ s/COMPONENT=//;
  }
  elsif ($option =~ m/PATH=/)
  {
    $g_sourcedir = $option;
    $g_sourcedir =~ s/PATH=//;
  }
  elsif ($option =~ m/LICENSE=/)
  {
    $req_licenses = $option;
    $req_licenses =~ s/LICENSE=//;
    @req_licenses_ar = split(',', $req_licenses);
  }
  elsif ($option =~ m/CODE_SOURCE=/)
  {
    $req_code_source = $option;
    $req_code_source =~ s/CODE_SOURCE=//;
    @req_code_source_ar = split(',', $req_code_source);
  }
  elsif ($option =~ m/STAGINGDIR=/)
  {
    $g_stagingdir = $option;
    $g_stagingdir =~ s/STAGINGDIR=//;
  }
  elsif ($option =~ m/RELEASEDIR=/)
  {
    $g_releasedir = $option;
    $g_releasedir =~ s/RELEASEDIR=//;
  }
  elsif ($option =~ m/ROOT_LIST=/)
  {
    my $temp = $option;
    $temp =~ s/ROOT_LIST=//;
    @g_globalrootpathlst = split(',', $temp);
  }
  else
  {
    print ">>>>>>> ERROR: unknown option \"$option\"\n";
    exit 1;
  }
}

#print "g_target_clean=$g_target_clean\n";
#print "g_target_release=$g_target_release\n";
#print "g_target_collect=$g_target_collect\n";
#print "g_component=$g_component\n";
#print "g_sourcedir=$g_sourcedir\n";
#print "g_stagingdir=$g_stagingdir\n";
#print "g_releasedir=$g_releasedir\n";
#print "req_licenses_ar=@req_licenses_ar\n";
#print "req_code_source_ar=@req_code_source_ar\n";
#print "g_globalrootpathlst=@g_globalrootpathlst\n";

# construct the output directory name
$g_staging_output_dir_name = "$g_stagingdir/$g_output_dir";
# construct the license file name
$g_licensefile_name = "$g_sourcedir/$g_licensefile";


if ($g_target_clean)
{
  if (length($g_stagingdir))
  {
    if (-d "$g_staging_output_dir_name")
    {
      system("rm -rf $g_staging_output_dir_name");
      print "Directory \"$g_staging_output_dir_name\" removed\n";
    }

    exit 0;
  }
  else
  {
  print ">>>>>>> ERROR: Please specify the staging directory\n";
  exit 1;
  }
}
elsif ($g_target_release)
{
  if (!length($g_stagingdir))
  {
    print ">>>>>>> ERROR: Please specify the STAGINGDIR\n";
    exit 1;
  }
  elsif (!length($g_releasedir))
  {
    print ">>>>>>> ERROR: Please specify the RELEASEDIR\n";
    exit 1;
  }

  if (!-d "$g_staging_output_dir_name")
  {
    print ">>>>>>> ERROR: nothing to release, directory \"$g_staging_output_dir_name\" does not exist\n";
    exit 1;
  }

  if (!-d "$g_releasedir")
  {
    print ">>>>>>> ERROR: release, directory \"$g_releasedir\" does not exist\n";
    exit 1;
  }

  my $tar_file = "$g_releasedir/$g_output_dir.tar.bz2";
  if (-e "$tar_file")
  {
    system("rm $tar_file");
  }

  system("pushd $g_staging_output_dir_name && echo \"Creating tar...\" && tar cjvf $tar_file .");

  if (-e "$tar_file")
  {
    print "--> File \"$tar_file\" created!!\n";
  }
  else
  {
    print ">>>>>>> ERROR: Unable to create file \"$tar_file\"\n";
    exit 1;
  }
}
elsif ($g_target_collect)
{
  if (!@req_licenses_ar)
  {
    print ">>>>>>> ERROR: Please specify the requested licenses (LICENSE=)\n";
    exit 1;
  }
  elsif (!@req_code_source_ar)
  {
    print ">>>>>>> ERROR: Please specify the requested code sources (CODE_SOURCE=)\n";
    exit 1;
  }
  elsif (!length($g_sourcedir))
  {
    print ">>>>>>> ERROR: Please specify the PATH\n";
    exit 1;
  }
  elsif (!length($g_stagingdir))
  {
    print ">>>>>>> ERROR: Please specify the STAGINGDIR\n";
    exit 1;
  }
  elsif (!@g_globalrootpathlst)
  {
    print ">>>>>>> ERROR: Please specify the ROOT_LIST\n";
    exit 1;
  }

  # create output dir if non-existing
  if (!-d "$g_staging_output_dir_name")
  {
    system("mkdir -p $g_staging_output_dir_name");
  }

  # collect all directories in the specified path
  my $dir;
  foreach $dir (`find $g_sourcedir -type d`)
  {
    # remove newline and carriage return
    $dir =~ s/\n//;
    $dir =~ s/\r//;

    push @g_dir_array, { dir => "$dir", defined_dir => "", code_source => "", license => "" };
  }

  if (-e "$g_licensefile_name")
  {
    open (LIC_FILE, "$g_licensefile_name");
    my $lineNmbr = 1;
    while (my $line= <LIC_FILE>)
    {
      # fill in the code_source and license information in every directory
      handle_license_file_line($line, $lineNmbr++);
    }
    close (LIC_FILE);

    # FOR DEBUGGING
    #my $href;
    #my $role;
    #for $href ( @g_dir_array ) {
    #    print "{ ";
    #    for $role ( keys %$href ) {
    #        print "$role=$href->{$role} ";
    #    }
    #    print "}\n";
    #}

    copy_sources();
  }
  else
  {
    print STDOUT ">>>>>>> ERROR: licensefile not found for component $g_component\n\n";
    #exit 1;
  }
}
else
{
  print ">>>>>>> ERROR: Please specify the target type (CLEAN,COLLECT,RELEASE)\n";
  exit 1;
}


