#!/bin/bash

MY_DIR=$(dirname $(readlink -f ${BASH_SOURCE[0]}))

# We resolve mount points relative to the RELEASE_DIR because as far as we know
# The build system does not store any files meant for the image outside of the RELEASE_DIR
# Note: When adding new mountpounts make sure they are ordered by length
# FIXME: This can't resolve mountpoints in mounted directories
resolve_mountpoints() {
  INPUT_PATH=$( echo $1 | ${MY_DIR}/../file_tracing/reduce_path.awk )
  case $INPUT_PATH in
    /mnt/jffs2/jffs2_3/system/*)
        echo "/root_fs/fs/rw/system/${INPUT_PATH##/mnt/jffs2/jffs2_3/system/}"
    ;;
    /mnt/jffs2/jffs2_3/system)
        echo "/root_fs/fs/rw/system/"
    ;;
    /mnt/jffs2/jffs2_3/common/*)
        echo "/root_fs/fs/rw/common/${INPUT_PATH##/mnt/jffs2/jffs2_3/common/}"
    ;;
    /mnt/jffs2/jffs2_3/common)
        echo "/root_fs/fs/rw/common/"
    ;;
    /var/*)
        echo "/root_fs/fs/tmp/${INPUT_PATH##/var/}"
    ;;
    /var)
        echo "/root_fs/fs/tmp/"
    ;;
    /*)
        echo /root_fs/${INPUT_PATH##/}
    ;;
    /)
        echo /root_fs/
    ;;
#    *)
#        echo ${INPUT_PATH}
#    ;;
  esac
}

MAX_ITERATIONS=500

# This function reads a symbolic link target relative to RELEASE_DIR
getSystemPath() {
         IN_PATH=$1
   
         CYCLIC_LINK_PROTECTOR=$(($CYCLIC_LINK_PROTECTOR+1))
         if [ $CYCLIC_LINK_PROTECTOR -eq $MAX_ITERATIONS ]; then
             echo "ERROR: $0: $CYCLIC_LINK_PROTECTOR iterations have passed. There is a good chance you are stuck in symlink loop. If not increase $MAX_ITERATIONS" >&2
             exit 1;
         fi

#	 echo "IN_PATH=\"${IN_PATH}\"" >&2
         if [ "${IN_PATH}" == "" ] || [ "${IN_PATH}" == "/" ] ; then
             RV_systemPath="/"
         else
             dir=${IN_PATH%/*}
             getSystemPath $dir
             IN_PATH=$1
             file=${IN_PATH##*/}
	     if [ ${RV_systemPath} == "/" ]; then
                 VIRTUAL_PATH=${RV_systemPath}${file}
             else 
                 VIRTUAL_PATH=${RV_systemPath}/${file}
	     fi
#	     echo "VIRTUAL_PATH=$VIRTUAL_PATH" >&2
             FULL_REAL_PATH=${RELEASE_DIR}$( resolve_mountpoints ${VIRTUAL_PATH} )
#	     echo FULL_REAL_PATH= ${FULL_REAL_PATH} >&2
             if [ -L ${FULL_REAL_PATH} ]; then    
                 SYMLINK_TARGET=$(readlink ${FULL_REAL_PATH})
#	         echo "SYMLINK_TARGET= ${SYMLINK_TARGET}" >&2
                 # We need to translate the mount point here
                 if [ "${SYMLINK_TARGET:0:1}" == "/" ]; then
                        # We need to get the real path of the link as well
		        getSystemPath ${SYMLINK_TARGET}
                        #RV_systemPath=${SYMLINK_TARGET}
                 else 
                        getSystemPath "${RV_systemPath}/${SYMLINK_TARGET}"
                 fi;
             else 
                 RV_systemPath=${VIRTUAL_PATH}
             fi;
         fi;
#	 echo "RV_systemPath=$RV_systemPath" >&2
}

verifyInput() {
  IN_PATH=$1
  if [ "${IN_PATH:0:1}" != "/" ]; then
    echo "ERROR: $0: parameter ${IN_PATH} is not an absolute path" >&2
    exit 2
  fi;
}


resolve_stream() {
(while read LINE; do
    verifyInput $LINE
    CYCLIC_LINK_PROTECTOR=0
    REDUCED_IN_PATH=$( echo ${LINE} | ${MY_DIR}/../file_tracing/reduce_path.awk )
    getSystemPath "$REDUCED_IN_PATH"
    RV_systemPath=$( resolve_mountpoints ${RV_systemPath} )
#   echo ${RELEASE_DIR}${RV_systemPath}
    echo "${RELEASE_DIR}${RV_systemPath}"
  done) <&0
}


RELEASE_DIR=$( cd $MY_DIR/../../output/release; pwd )
# if no arguments
if [ $# -lt 1 ]; then
  resolve_stream <&0
else
  (for ARG in $@; do echo $ARG; done) | resolve_stream
fi
