#!/usr/bin/perl

# File::Find is just not good enough, there's no way to
# get subdirectories before the directory contents itself.
# This is important for us as we can only delta subdirectories
# if the parent still has the original files.
# Do a manual traversal instead.

use strict;
use File::Spec;
use Cwd;

my $startdir = cwd();
print "Root: $startdir\n";

recursive_patch();

sub recursive_patch {
	my @content = `ls --group-directories-first -1`;
	
	foreach my $thing (@content) {
		chomp $thing;
		if (-f $thing) {			
			# it's a file, patch it if possib;e
			# get our name, parent name, pos relative to root
			my $fname = $thing;
			my $parent   = File::Spec->updir();
			my $abs_path = Cwd::abs_path($fname);
			my $rel_path = File::Spec->abs2rel($abs_path, $startdir);
			my $me = $rel_path;
		
			# do we have a daddy?
			# and are we below the starting directory?
			if (-e "$parent/$fname" && cwd() ne $startdir) {
				# dont patch patch files
				if (!($fname =~ /.*\.patch/)) {
					patchify($fname, "$parent/$fname");
					print $me . " -> $me.patch\n";
				}		
			}
		} elsif (-d $thing) {
			my $mydir = cwd();
			chdir $thing;
			recursive_patch();
			chdir $mydir;
		}
	}
};

sub patchify {
	my $one = shift;
	my $two = shift;
#	if (-e "$one.patch") {
#		if (!(-w "$one.patch")) {
#			system("p4 edit $one.patch");
#		}
#	}
	system("diff -d -u $two $one > $one.patch");
#	system("rm -f $one");
};
