/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

#ifndef __BSP_BOARD_H__
#define __BSP_BOARD_H__

/**
 * \file bsp_board.h Bsp Board description.
 *
 * This file allow to describe Bsp Board.
*/

/*
 * Typedefs
 * ~~~~~~~~
 */

/**
 * \name Enum typedefs.
 */
/*@{*/
/** Typedef on enum bsp_board_type_enu. */
typedef enum bsp_board_type_enu		bsp_board_type_e;
/** Typedef on enum bsp_board_info_enu. */
typedef enum bsp_board_info_enu		bsp_board_info_e;
/** Typedef on enum bsp_board_vox_type_enu. */
typedef enum bsp_board_vox_type_enu		bsp_board_vox_type_e;
/*@}*/

/*
 * Enums
 * ~~~~~
 */

/** Board info. */
enum bsp_board_info_enu {
	/** Unknown.
	 * 	\note Don't touch this place.
	 */
	BSP_Board_info_unknown,

	/** Mini. */
	BSP_Board_info_lb_mini,

	/** Ftth. */
	BSP_Board_info_lb_ftth_v2,

	/** 3504. */
	BSP_Board_info_fast_3504,

	/** LVB2 STD. */
	BSP_Board_info_lb_standard,

	/** LVB2 V2_1. */
	BSP_Board_info_lb_v2_1,

	/** LVB2 V2_2. */
	BSP_Board_info_lb_v2_2,

	/** 3764 vox180. */
	BSP_Board_info_3764,

	/** TDC vox180 */
	BSP_Board_info_TDC_180,

	/** BBOX */
	BSP_Board_info_Bbox,

	/** Livebox ProV3 vox185 */
	BSP_Board_info_lbv3,

	/** TDC HGW3 vox183 */
	BSP_Board_info_TDC_183,

	/** Livebox ProV3 vox185 v2 */
	BSP_Board_info_lbv3_v2,

	/** Livebox 2.8 hardware "v0" (aka "P1" at FT's) */
	BSP_Board_info_lbv28_v0,

	/** Livebox 2.8 hardware "v1" (aka "P2" at FT's) */
	BSP_Board_info_lbv28_v1,

	/** Max. */
	BSP_Board_info_max,
};

/** Type Board type. */
enum bsp_board_type_enu {
	/** Mini. */
	BSP_Board_type_lb_mini,

	/** Ftth. */
	BSP_Board_type_lb_ftth_v2,

	/** 3504. */
	BSP_Board_type_fast_3504,

	/** LVB2 STD. */
	BSP_Board_type_lb_standard,

	/** LVB2 V2. */
	BSP_Board_type_lb_v2,

	/** 3764 */
	BSP_Board_type_3764,

	/** TDC vox180 */
	BSP_Board_type_TDC_180,

	/** BBOX */
	BSP_Board_type_Bbox,

	/** Livebox ProV3 vox185 */
	BSP_Board_type_lbv3,

	/** TDC HGW3 vox183 */
	BSP_Board_type_TDC_183,

	/** Livebox ProV3 vox185 v2 */
	BSP_Board_type_lbv3_v2,

	/** Livebox 2.8 hardware "v0" (aka "P1" at FT's) */
	BSP_Board_type_lbv28_v0,

	/** Livebox 2.8 hardware "v1" (aka "P2" at FT's) */
	BSP_Board_type_lbv28_v1,

	/** Max. */
	BSP_Board_type_max,

	/** Unknown. */
	BSP_Board_type_unknown	= 0xff,
};


/** Board info. */
enum bsp_board_vox_type_enu {
	/** Board with Vox 160 or 180 */
	BSP_Board_vox_type_160_180,

	/** Board with Vox 183 or 185 */
	BSP_Board_vox_type_183_185,

};

#ifdef __KERNEL__

/*
 * Functions
 * ~~~~~~~~~
 */

/** Get BoardType. */
bsp_board_type_e bsp_get_boardType(	void);

/** Get BoardInfo. */
bsp_board_info_e bsp_get_boardInfo(	void);

#endif
#endif

// vim:ts=5:
