/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

#ifndef __BSP_DEV_H__
#define __BSP_DEV_H__

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \defgroup api_bsp API -> Kernel module Bsp by /dev/bsp.
 */
/*@{*/

/**
 * \file bsp_dev.h Device description
 *
 * \author Sagem SA
 *
 * This file allow to describe device interface (/dev/bsp).
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include <linux/ioctl.h>

#include "bsp_permParam.h"

/*
 * Defines
 * ~~~~~~~
 */

/** \name Various Defines
 */
/*@{*/
/** Debug Dev Major. */
#define	BSP_DEV_MAJOR									225
/** Debug Dev Perm Param Name. */
//#define	BSP_DEV_NAME									"bsp_name"
#define	BSP_DEV_NAME									"/dev/bsp_name"
/*@}*/

/** \name Debug Defines
 */
/*@{*/
/** Debug Dev. */
#define	BSP_FLAGS_DEBUG_DEV								(1 << 0)
/** Debug Gpio. */
#define	BSP_FLAGS_DEBUG_GPIO							(1 << 1)
/** Debug Led. */
#define	BSP_FLAGS_DEBUG_LED								(1 << 2)
/** Debug But. */
#define	BSP_FLAGS_DEBUG_BUT								(1 << 3)
/** Debug Relay. */
#define	BSP_FLAGS_DEBUG_RELAY							(1 << 4)
/** Debug Max. */
#define	BSP_FLAGS_DEBUG_MAX								(1 << 7)
/*@}*/

/** \name Ioctl Defines
 */
/*@{*/
/** Debug ioctl base. */
#define	BSP_IOCTL_BASE									0xFA

/** Get All. */
#define	BSP_IOCTL_GET_PERMPARAM_ALL						_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_all, \
														BSP_TYPE_PERMPARAM_ALL)

/** Get Version. */
#define	BSP_IOCTL_GET_PERMPARAM_VERSION					_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_version, \
														BSP_TYPE_PERMPARAM_VERSION)

/** Get Serial Number. */
#define	BSP_IOCTL_GET_PERMPARAM_SERIAL_NUMBER				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_serial_number, \
														BSP_TYPE_PERMPARAM_SERIAL_NUMBER)

/** Get Sagem Product Id. */
#define	BSP_IOCTL_GET_PERMPARAM_SAGEM_PRODUCT_ID			_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_sagem_product_id, \
														BSP_TYPE_PERMPARAM_SAGEM_PRODUCT_ID)

/** Get Hardware Version. */
#define	BSP_IOCTL_GET_PERMPARAM_SAGEM_HARDWARE_VERSION		_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_sagem_hardware_version, \
														BSP_TYPE_PERMPARAM_SAGEM_HARDWARE_VERSION)
/** Get Hardware Revision. */
#define	BSP_IOCTL_GET_PERMPARAM_SAGEM_HARDWARE_REVISION		_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_sagem_hardware_revision, \
														BSP_TYPE_PERMPARAM_SAGEM_HARDWARE_REVISION)

/** Get Customer Product Id. */
#define	BSP_IOCTL_GET_PERMPARAM_CUSTOMER_PRODUCT_ID			_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_customer_product_id, \
														BSP_TYPE_PERMPARAM_CUSTOMER_PRODUCT_ID)

/** Get Customer Hardware Version. */
#define	BSP_IOCTL_GET_PERMPARAM_CUSTOMER_HARDWARE_VERSION		_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_customer_hardware_version, \
														BSP_TYPE_PERMPARAM_CUSTOMER_HARDWARE_VERSION)

/** Get Customer Hardware Revision. */
#define	BSP_IOCTL_GET_PERMPARAM_CUSTOMER_HARDWARE_REVISION	_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_customer_hardware_revision, \
														BSP_TYPE_PERMPARAM_CUSTOMER_HARDWARE_REVISION)

/** Get Ram Size. */
#define	BSP_IOCTL_GET_PERMPARAM_RAM_SIZE					_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_ram_size, \
														BSP_TYPE_PERMPARAM_RAM_SIZE)

/** Get Flash Size. */
#define	BSP_IOCTL_GET_PERMPARAM_FLASH_SIZE					_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_flash_size, \
														BSP_TYPE_PERMPARAM_FLASH_SIZE)

/** Get Firmware Version. */
#define	BSP_IOCTL_GET_PERMPARAM_FIRMWARE_VERSION			_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_firmware_version, \
														BSP_TYPE_PERMPARAM_FIRMWARE_VERSION)

/** Get Lan Base Mac Address. */
#define	BSP_IOCTL_GET_PERMPARAM_LAN_BASE_MAC_ADDRESS			_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_lan_base_mac_address, \
														BSP_TYPE_PERMPARAM_LAN_BASE_MAC_ADDRESS)

/** Get Lan Mac Address Range. */
#define	BSP_IOCTL_GET_PERMPARAM_LAN_MAC_ADDRESS_RANGE		_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_lan_mac_address_range, \
														BSP_TYPE_PERMPARAM_LAN_MAC_ADDRESS_RANGE)

/** Get Wan Base Mac Address. */
#define	BSP_IOCTL_GET_PERMPARAM_WAN_BASE_MAC_ADDRESS			_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_wan_base_mac_address, \
														BSP_TYPE_PERMPARAM_WAN_BASE_MAC_ADDRESS)

/** Get Lan Mac Address Range. */
#define	BSP_IOCTL_GET_PERMPARAM_WAN_MAC_ADDRESS_RANGE		_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_wan_mac_address_range, \
														BSP_TYPE_PERMPARAM_WAN_MAC_ADDRESS_RANGE)

/** Get Usb Mac. */
#define	BSP_IOCTL_GET_PERMPARAM_USB_MAC					_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_usb_mac, \
														BSP_TYPE_PERMPARAM_USB_MAC)

/** Get Wifi Ssid 1. */
#define	BSP_IOCTL_GET_PERMPARAM_WIFI_SSID_1				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_wifi_ssid_1, \
														BSP_TYPE_PERMPARAM_WIFI_SSID_1)

/** Get Wifi Mac Address 1. */
#define	BSP_IOCTL_GET_PERMPARAM_WIFI_MAC_ADDRESS_1			_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_wifi_mac_address_1, \
														BSP_TYPE_PERMPARAM_WIFI_MAC_ADDRESS_1)

/** Get Wifi Ssid 2. */
#define	BSP_IOCTL_GET_PERMPARAM_WIFI_SSID_2				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_wifi_ssid_2, \
														BSP_TYPE_PERMPARAM_WIFI_SSID_2)

/** Get Wifi Mac Address 1. */
#define	BSP_IOCTL_GET_PERMPARAM_WIFI_MAC_ADDRESS_2			_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_wifi_mac_address_2, \
														BSP_TYPE_PERMPARAM_WIFI_MAC_ADDRESS_2)

/* Manufacturer. */
#define	BSP_IOCTL_GET_PERMPARAM_MANUFACTURER				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_manufacturer, \
														BSP_TYPE_PERMPARAM_MANUFACTURER)

/* Manufacturer oui. */
#define	BSP_IOCTL_GET_PERMPARAM_MANUFACTURER_OUI			_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_manufacturer_oui, \
														BSP_TYPE_PERMPARAM_MANUFACTURER_OUI)

/* Manufacturer url. */
#define	BSP_IOCTL_GET_PERMPARAM_MANUFACTURER_URL			_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_manufacturer_url, \
														BSP_TYPE_PERMPARAM_MANUFACTURER_URL)

/* Country Code. */
#define	BSP_IOCTL_GET_PERMPARAM_COUNTRY_CODE				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_country_code, \
														BSP_TYPE_PERMPARAM_COUNTRY_CODE)

/* Wifi Key 1. */
#define	BSP_IOCTL_GET_PERMPARAM_WIFI_KEY_1					_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_wifi_key_1, \
														BSP_TYPE_PERMPARAM_WIFI_KEY_1)

/* Wifi Key 2. */
#define	BSP_IOCTL_GET_PERMPARAM_WIFI_KEY_2					_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_wifi_key_2, \
														BSP_TYPE_PERMPARAM_WIFI_KEY_2)

/* H235 Key. */
#define	BSP_IOCTL_GET_PERMPARAM_H235_KEY					_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_h235_key, \
														BSP_TYPE_PERMPARAM_H235_KEY)

/* Public Dsa Syst. */
#define	BSP_IOCTL_GET_PERMPARAM_PUBLIC_DSA_SYST				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_public_dsa_syst, \
														BSP_TYPE_PERMPARAM_PUBLIC_DSA_SYST)

/* Public Dsa Resc. */
#define	BSP_IOCTL_GET_PERMPARAM_PUBLIC_DSA_RESC				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_public_dsa_resc, \
														BSP_TYPE_PERMPARAM_PUBLIC_DSA_RESC)

/* Private Key. */
#define	BSP_IOCTL_GET_PERMPARAM_PRIVATE_KEY				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_private_key, \
														BSP_TYPE_PERMPARAM_PRIVATE_KEY)

/* Certificate. */
#define	BSP_IOCTL_GET_PERMPARAM_CERTIFICATE				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_certificate, \
														BSP_TYPE_PERMPARAM_CERTIFICATE)

/* Dsa. */
#define	BSP_IOCTL_GET_PERMPARAM_DSA						_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_dsa, \
														BSP_TYPE_PERMPARAM_DSA)

/* Vendor Info 1. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_1				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_1, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_1)

/* Vendor Info 2. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_2				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_2, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_2)

/* Vendor Info 3. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_3				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_3, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_3)

/* Vendor Info 4. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_4				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_4, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_4)

/* Vendor Info 5. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_5				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_5, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_5)

/* Vendor Info 6. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_6				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_6, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_6)

/* Vendor Info 7. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_7				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_7, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_7)

/* Vendor Info 8. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_8				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_8, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_8)

/* Vendor Info 9. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_9				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_9, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_9)

/* Vendor Info 10. */
#define	BSP_IOCTL_GET_PERMPARAM_VENDOR_INFO_10				_IOR(BSP_IOCTL_BASE, \
														BSP_Get_permParam_vendor_info_10, \
														BSP_TYPE_PERMPARAM_VENDOR_INFO_10)

/*@}*/

/*
 * Enums
 * ~~~~~
 */

/** Bsp_dev_minor. */
enum bsp_dev_minor_enu {
	/** PermParam. */
	BSP_Dev_minor_pp,

	/** End. */
	BSP_Dev_minor_max,
};

/*
 * Typedefs
 * ~~~~~~~~
 */

/*
 * Struct
 * ~~~~~~~~
 */

/* Bsp_permParam_str.
 *
 *	This struct manage Permanent Parameters.
 */
struct bsp_permParam_str {
	/** Magic. */
	__u32			magic;

	/* Permanent parameters version. */
	char				pp_version[32];

	/** Serial number. */
	char				serial_number[64];

	/** Sagem product identifier. */
	char				s_product_id[64];

	/** Sagem product version number. */
	char				s_hardware_version[64];

	/** Sagem product revision number. */
	char				s_hardware_revision[64];

	/** Client product identifier. */
	char				c_product_id[64];

	/** Sagem product hardware version. */
	char 			c_hardware_version[64];

	/** Sagem product revision number. */
	char 			c_hardware_revision[64];

	/** DDRAM-SDRAM size. */
	__u32			ram_size;

	/** Flash size. */
	__u32			flash_size;

	/** Firmware version. */
	char				fmw_version[32];

	/** MAC address on Lan side. */
	__u8				lan_base_mac_address[6];

	/** MAC address number reserved on lan side. */
	__u16			lan_mac_address_range;

	/** MAC address on Wan side. */
	__u8				wan_base_mac_address[6];

	/** MAC address number reserved on wan side. */
	__u16			wan_mac_address_range;

	/** USB MAC address. */
	__u8				usb_mac[6];

	/** WIFI MAC address n°1 .*/
	__u8				wifi_mac1[6];

	/** WIFI SSID n°1 .*/
	char				wifi_ssid1[32];

	/* WIFI MAC address n°2 */
	__u8				wifi_mac2[6];

	/** WIFI SSID n°2 .*/
	char				wifi_ssid2[32];

	/* Manufacturer : "SAGEM" */
	char				manufacturer[64];

	/** 3 first bytes of base_mac_address. */
	__u8				manufacturer_oui[3];

	/** URL : "http://www.sagem.com". */
	char				manufacturer_url[128];

	/** Commercialization country. */
	char				country_code[32];

	/** Default key (WEP or WPA). */
	char				wifi_key1[64];

	/* Default key (WEP or WPA) */
	char				wifi_key2[64];

	/** H235 key. */
	char				h235_key[64];

	/** DSA key used to sign APP2. */
	__u8				public_dsa_syst[404];

	/** DSA key used to sign APP1. */
	__u8				public_dsa_resc[404];

	/** Gateway private DSA key. */
	__u8				private_key[2048];

	/** Certifacate associated to private key. */
	__u8				certificate[2048];

	/** DSA check on/off. */
	__u32			dsa;

	char				vendor_info1[64];

	char				vendor_info2[64];

	char				vendor_info3[64];

	char				vendor_info4[64];

	char				vendor_info5[64];

	char				vendor_info6[64];

	char				vendor_info7[64];

	char				vendor_info8[64];

	char				vendor_info9[64];

	char				vendor_info10[64];
};

/** \name Enums.
 */
/*@{*/
/** \link bsp_dev_minor_enu bsp_dev_minor enum \endlink typedef. */
typedef enum bsp_dev_minor_enu							bsp_dev_minor_e;
/*@}*/

/*
 * Functions
 * ~~~~~~~~~
 */

#ifdef __KERNEL__

/** Open device. */
int bsp_devOpen(			struct inode					*_pInode,
						struct file					*_pFile);

/** Release device. */
int bsp_devRelease(			struct inode					*_pInode,
						struct file					*_pFile);

/** Ioctl device. */
int bsp_devIoctl(			struct inode					*_pInode,
						struct file					*_pFile,
						unsigned int					_cmd,
						unsigned long					_arg);
#endif

/*@}*/

#ifdef __cplusplus
}
#endif

#endif

// vim:ts=5:
