/*
 * ============================================================================
 *
 * Copyright : (c) 2009 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

#ifndef __BSP_GPIO_H__
#define __BSP_GPIO_H__

/**
 * \file bsp_gpio.h Bsp description for manage GPIO.
 *
 * This file allow to describe Bsp Public.
*/

#ifdef __KERNEL__

/*
 * Defines
 * ~~~~~~~~
 */

/** \name Gpio Various.
 */
/*@{*/
#define	BSP_GPIO_MAX				32
/*@}*/

/** \name Gpio alias.
 */
/*@{*/
/** Led Off. */
#define	BSP_LED_OFF				BSP_GPIO_STATSU_RESET
/** Led On */
#define	BSP_LED_ON				BSP_GPIO_STATSU_SET
/** But Released. */
#define	BSP_BUT_RELEASED			BSP_GPIO_STATUS_RESET
/** But Pushed. */
#define	BSP_BUT_PUSHED				BSP_GPIO_STATUS_GPIO_SET
/** Relay On. */
#define	BSP_RELAY_ON				BSP_GPIO_STATUS_SET
/** Relay Off. */
#define	BSP_RELAY_OFF				BSP_GPIO_STATUS_RESET
/*@}*/

/*
 * Typedefs
 * ~~~~~~~~
 */

/** \name Enums.
 */
/*@{*/
/** \link bsp_gpio_name_enu bsp_gpio_name enum \endlink typedef. */
typedef enum bsp_gpio_name_enu		bsp_gpio_name_e;
/** \link bsp_gpio_state_enu bsp_gpio_state enum \endlink typedef. */
typedef enum bsp_gpio_state_enu		bsp_gpio_state_e;
#if (defined HALBSP_PERGPIO_PROCENTRY)
/** \link bsp_proc_name_enu bsp_procname enum \endlink typedef. */
typedef enum bsp_proc_name_enu bsp_proc_name_e;
#endif /* HALBSP_PERGPIO_PROCENTRY */
/*@}*/

/*
 * Enums
 * ~~~~~
 */

/** Bsp_gpio_state. */
enum bsp_gpio_state_enu {
	/** Unset. */
	BSP_Gpio_state_unset,
	/** Set. */
	BSP_Gpio_state_set,
	/** Failure. */
	BSP_Gpio_state_failure,
	/** Max. */
	BSP_Gpio_state_max
};

#if (defined HALBSP_PERGPIO_PROCENTRY)

/** Bsp_proc_name. */
enum bsp_proc_name_enu {

	BSP_Proc_name_reserved,

  /* Akka(S.VIE), 2011-12-01: Per-LED proc entries.
   */
	BSP_Proc_name_led_wireless,
	BSP_Proc_name_led_screen_green,
	BSP_Proc_name_led_screen_red,

	BSP_Proc_name_max
};

#endif /* HALBSP_PERGPIO_PROCENTRY */

/** Bsp_gpio_name. */
enum bsp_gpio_name_enu {
	/** Reserved. */
	BSP_Gpio_name_reserved,
	/** */

	/** Led part. */
	BSP_Gpio_name_led_internet_synchroline,
	BSP_Gpio_name_led_internet_data,
	BSP_Gpio_name_led_wireless,
	BSP_Gpio_name_led_wireless_pairing,
	BSP_Gpio_name_led_power,
	BSP_Gpio_name_led_lan,
	BSP_Gpio_name_led_usb,
	BSP_Gpio_name_led_upgrade,
	BSP_Gpio_name_led_phone_1,
	BSP_Gpio_name_led_phone_2,
	BSP_Gpio_name_led_screen_green,
	BSP_Gpio_name_led_screen_red,

	/** Button part. */
	BSP_Gpio_name_button_reset,
	BSP_Gpio_name_button_reboot,
	BSP_Gpio_name_button_pairing,
	BSP_Gpio_name_buttonUp,
	BSP_Gpio_name_buttonDown,
	BSP_Gpio_name_buttonOk,
	BSP_Gpio_name_button_wifi_on,
	BSP_Gpio_name_button_wakeup_display,

	/** Relay part. */
	BSP_Gpio_name_relay_fxs_all,
	BSP_Gpio_name_relay_fxs_1,
	BSP_Gpio_name_relay_fxs_2,
	BSP_Gpio_name_relay_fxo,

	/** Spi part. */
	BSP_Gpio_name_spi_cs_fxs_all_and_fxo,
	BSP_Gpio_name_spi_cs_fxs_all,
	BSP_Gpio_name_spi_cs_fxo,
	BSP_Gpio_name_spi_cs_dect,
	BSP_Gpio_name_spi_cs_zigbee,
	BSP_Gpio_name_spi_cs_oled,

	/** OLED Screen part */
	BSP_Gpio_name_rst_oled,
	BSP_Gpio_name_oled_data_cmd,

	/** Voice part. */
	BSP_Gpio_name_voice_rst_fxs_all_and_fxo,
	BSP_Gpio_name_voice_rst_fxs_1,
	BSP_Gpio_name_voice_rst_fxs_2,
	BSP_Gpio_name_voice_rst_fxs_2_and_fxo,
	BSP_Gpio_name_voice_rst_dect,
	BSP_Gpio_name_voice_rst_zigbee,
	BSP_Gpio_name_voice_int_fxs_1,
	BSP_Gpio_name_voice_int_fxs_2,
	BSP_Gpio_name_voice_int_fxs_2_and_fxo,
	BSP_Gpio_name_voice_int_dect,
	BSP_Gpio_name_voice_int_zigbee,
	BSP_Gpio_name_voice_det_fxs1,
	BSP_Gpio_name_voice_det_fxs2,

	/** Config input part. */
	BSP_Gpio_name_config_1,
	BSP_Gpio_name_config_2,
	BSP_Gpio_name_config_3,
	BSP_Gpio_name_led_card,

	/** Usb part. */
	BSP_Gpio_name_usb_pwr_en_master_all,
	BSP_Gpio_name_usb_pwr_en_master_1,
	BSP_Gpio_name_usb_pwr_en_master_2,
	BSP_Gpio_name_usb_pwr_en_master_3,
	BSP_Gpio_name_usb_overcurrent_master_all,
	BSP_Gpio_name_usb_overcurrent_master_1,
	BSP_Gpio_name_usb_overcurrent_master_2,
	BSP_Gpio_name_usb_overcurrent_master_3,
	BSP_Gpio_name_usb_pwr_en_slave,
	BSP_Gpio_name_usb_pwr_detect_slave,
	BSP_Gpio_name_usb_rst_hub,

	/** LBV2 Mini specific LED of Ethernet mgmt. */
	BSP_Gpio_name_mini_led_act_eth_all,
	BSP_Gpio_name_mini_led_act_eth_0,
	BSP_Gpio_name_mini_led_act_eth_1,
	BSP_Gpio_name_mini_pin_crs_eth_0,
	BSP_Gpio_name_mini_pin_crs_eth_1,

	/** Adls part. */
	BSP_Gpio_name_adsl_rst_afe,

	/** Dying Gasp part. */
	BSP_Gpio_name_dying_gasp,

	/** Lan part. */
	BSP_Gpio_name_switch_rst,
	BSP_Gpio_name_switch_int,
	BSP_Gpio_name_phy_rst_all,
	BSP_Gpio_name_phy_rst_1,
	BSP_Gpio_name_phy_rst_2,
	BSP_Gpio_name_lan_wakeup,

	/** Wifi part. */
	BSP_Gpio_name_wifi_rst,
	BSP_Gpio_name_wifi_rst_5G,
	BSP_Gpio_name_wifi_rst_2G4,

	/** Nl part. */
	BSP_Gpio_name_cmd_nl,
	
	/** green reset. */
	BSP_Gpio_name_green_rst,
	BSP_Gpio_name_green_boot0,
	BSP_Gpio_name_green_boot1,

	/** Max. */
	BSP_Gpio_name_max
};

#define BSP_Gpio_name_led_voip				BSP_Gpio_name_led_phone_1
#define BSP_Gpio_name_led_adsl				BSP_Gpio_name_led_internet_red
#define BSP_Gpio_name_led_ip				BSP_Gpio_name_led_internet_green
#define BSP_Gpio_name_led_data				BSP_Gpio_name_led_lan
#define BSP_Gpio_name_led_wifi				BSP_Gpio_name_led_wireless
#define BSP_Gpio_name_led_dect				BSP_Gpio_name_led_phone_2
#define BSP_Gpio_name_led_internet_green	BSP_Gpio_name_led_internet_synchroline
#define BSP_Gpio_name_led_internet_red		BSP_Gpio_name_led_internet_data

/*
 * Functions
 * ~~~~~~~~~
 */

/** Set GPIO state */

void bsp_gpio_set_state(				bsp_gpio_name_e	_name,
								bsp_gpio_state_e	_state);

void  bsp_gpio_set_state_ex(

	bsp_gpio_name_e	_name,
	bsp_gpio_state_e	_state,

	/* Akka(S.VIE), 2011-12-12: New argument "bEnablePolarityInversion".
	 * Setting that argument to 1 will apply the reverted value of argument "_state"
	 * if the underlying GPIO was defined with flag "active_low".
	 */
	int bEnablePolarityInversion);

/** Get GPIO state */

bsp_gpio_state_e bsp_gpio_get_state(	bsp_gpio_name_e	_name);

bsp_gpio_state_e bsp_gpio_get_state_ex(

	bsp_gpio_name_e _name,

	/* Akka(S.VIE), 2011-12-12: New argument "bEnablePolarityInversion".
	 * Setting that argument to 1 will make the function return a reverted value
	 * if the underlying GPIO was defined with flag "active_low".
	 */
	int bEnablePolarityInversion);

#endif

#endif

// vim:ts=5:
