#ifndef __AUTO_SYSCTL_H
#define __AUTO_SYSCTL_H

#include <linux/sysctl.h>

/*
 * Deal with the sysctl handler api changing.
 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,8)
#define SYSCTL_DECL(f, ctl, write, filp, buffer, lenp, ppos) \
    f(ctl_table *ctl, int write, struct file *filp, void *buffer, \
        size_t *lenp)
#define SYSCTL_PROC_DOINTVEC(ctl, write, filp, buffer, lenp, ppos) \
    proc_dointvec(ctl, write, filp, buffer, lenp)
#define SYSCTL_PROC_DOSTRING(ctl, write, filp, buffer, lenp, ppos) \
    proc_dostring(ctl, write, filp, buffer, lenp)
#define SYSCTL_PROC_ARGS \
			ctl_table	*_pTable, \
			int			_write, \
			struct file	*_pFile, \
			void __user	*_pBuffer, \
			size_t		*_pLen, \
			loff_t		*_pPpos
#else /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,8) */

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,32)
#define SYSCTL_DECL(f, ctl, write, filp, buffer, lenp, ppos) \
    f(ctl_table *ctl, int write, struct file *filp, void *buffer,\
        size_t *lenp, loff_t *ppos)
#define SYSCTL_PROC_DOINTVEC(ctl, write, filp, buffer, lenp, ppos) \
    proc_dointvec(ctl, write, filp, buffer, lenp, ppos)
#define SYSCTL_PROC_DOINTVEC_MINMAX(ctl, write, filp, buffer, lenp, ppos) \
    proc_dointvec_minmax(ctl, write, filp, buffer, lenp, ppos)
#define SYSCTL_PROC_DOSTRING(ctl, write, filp, buffer, lenp, ppos) \
    proc_dostring(ctl, write, filp, buffer, lenp, ppos)
#define SYSCTL_PROC_ARGS \
			ctl_table	*_pTable, \
			int			_write, \
			struct file	*_pFile, \
			void __user	*_pBuffer, \
			size_t		*_pLen, \
			loff_t		*_pPpos
#else
#define SYSCTL_DECL(f, ctl, write, filp, buffer, lenp, ppos) \
    f(ctl_table *ctl, int write, struct file *filp, void *buffer,\
        size_t *lenp, loff_t *ppos)
#define SYSCTL_PROC_DOINTVEC(ctl, write, filp, buffer, lenp, ppos) \
    proc_dointvec(ctl, write, buffer, lenp, ppos)
#define SYSCTL_PROC_DOINTVEC_MINMAX(ctl, write, filp, buffer, lenp, ppos) \
    proc_dointvec_minmax(ctl, write, buffer, lenp, ppos)
#define SYSCTL_PROC_DOSTRING(ctl, write, filp, buffer, lenp, ppos) \
    proc_dostring(ctl, write, buffer, lenp, ppos)
#define SYSCTL_PROC_ARGS \
			ctl_table	*_pTable, \
			int			_write, \
			void __user	*_pBuffer, \
			size_t		*_pLen, \
			loff_t		*_pPpos
#endif

#endif

#endif
