/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_recovery.c Bsp recovery functions
 *
 * This file contain all Bsp Recovery functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

/*
 * Functions
 * ~~~~~~~~~
 */

/** Get  boot recovery force. */
bsp_boot_recovery_force_e bsp_get_boot_recovery_force(	void)
{
	/* Ok */
	return(bsp_base.Env.Boot.Recovery.force);
}
EXPORT_SYMBOL(bsp_get_boot_recovery_force);

/** Recovery force sysctl proc_do int vec. */
int bsp_boot_recovery_force_proc_dointvec_minmax(	SYSCTL_PROC_ARGS)
{
	int _res;

	/* Write */
	if (_write) {
		/* Take write sem */
		down_write(&(bsp_base.Env.Boot.Recovery.sem));

		/* Call proc_dointvec */
		_res = (SYSCTL_PROC_DOINTVEC_MINMAX(	_pTable,
								_write,
								_pFile,
								_pBuffer,
								_pLen,
								_pPpos));

		/* If Ok and write. */
		if (_res == 0) {
			switch (*(int *)_pTable->data) {
				case BSP_Boot_recovery_force_normal:
				{
					/* Cpy */
					memcpy(	(char *)BSP_BOOT_RECOVERY_FORCE_ADDR,
							BSP_BOOT_RECOVERY_FORCE_NORMAL_VALUE,
							sizeof(BSP_BOOT_RECOVERY_FORCE_NORMAL_VALUE));
				}
				break;
	
				case BSP_Boot_recovery_force_rescue:
				{
					/* Cpy */
					memcpy(	(char *)BSP_BOOT_RECOVERY_FORCE_ADDR,
							BSP_BOOT_RECOVERY_FORCE_RESCUE_VALUE,
							sizeof(BSP_BOOT_RECOVERY_FORCE_RESCUE_VALUE));
				}
				break;

				case BSP_Boot_recovery_force_bootp:
				{
					/* Cpy */
					memcpy(	(char *)BSP_BOOT_RECOVERY_FORCE_ADDR,
							BSP_BOOT_RECOVERY_FORCE_NETWORK_VALUE,
							sizeof(BSP_BOOT_RECOVERY_FORCE_NETWORK_VALUE));
				}
				break;
	
				default:
				{
					HALBSP_ERROR(	"Unknown force(%u)",
							*(int *)_pBuffer);

					/* Release write sem */
					up_write(&(bsp_base.Env.Boot.Recovery.sem));

					/* Error */
					return(-EFAULT);
				}
				break;
			}
		}

		/* Release write sem */
		up_write(&(bsp_base.Env.Boot.Recovery.sem));
	}
	/* Read */
	else {
		/* Take read sem */
		down_read(&(bsp_base.Env.Boot.Recovery.sem));

		/* Call proc_dointvec */
		_res = (SYSCTL_PROC_DOINTVEC_MINMAX(	_pTable,
								_write,
								_pFile,
								_pBuffer,
								_pLen,
								_pPpos));
		/* Release read sem */
		up_read(&(bsp_base.Env.Boot.Recovery.sem));
	}

	/* Ok */
	return(_res);
}

// vim:ts=5:
