/* ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_but.c Bsp Button functions
 *
 * This file contain all Bsp Button functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"


/*
 * Functions
 * ~~~~~~~~~
 */

/** Get BUTTON state. */
BOARD_BUTTON_STATE kSysGetButtonState(BOARD_BUTTON_NAME _nameUser)
{
	BOARD_BUTTON_STATE	_stateUser;

	/* Check */
	if ( (_nameUser < 0) || (_nameUser >= kButtonEnd) ){
		HALBSP_INFO(	"WARNING -> Button >= Max(%u)",
				kButtonEnd);

		return(-1);
	}

	/* Retrieve state */
	{
		bsp_gpio_state_e	_stateGpio;
		bsp_gpio_name_e	_nameGpio;

		/* Retrieve nameGpio */
		_nameGpio = bsp_base.Index.But.Name.user2gpio[_nameUser];

		/* Retrieve stateGpio */
		if ((_stateGpio = bsp_gpio_get_state(_nameGpio)) == BSP_Gpio_state_failure) {
			/* Error */
			return(-1);
		}

		/* Retrieve userState */
		_stateUser = bsp_base.Index.But.State.gpio2user[_stateGpio];

		/* If bsp */
		HALBSP_DEBUG(	"But -> nameUser(%u) nameGpio(%u) stateUser(%u) stateGpio(%u)",
				_nameUser,
				_nameGpio,
				_stateUser,
				_stateGpio);
	}

	return ( _stateUser ? kButtonReleased : kButtonPushed );
}
EXPORT_SYMBOL(kSysGetButtonState);

/** sysctl bsp_but_maps_proc.
 *
 *   \par Description
 *	<b>This function allow to set connexion tampon.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int bsp_but_maps_proc(	SYSCTL_PROC_ARGS)
{
	char				*_pTampon		= (char *)_pTable->data;
	BOARD_BUTTON_STATE	_userState;

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Retrieve tampon */
	if ((_pTampon = (char *)_pTable->data) != (char *)0) {
		sprintf(	_pTampon,
				"| Reboot | Reset | Pairing |   Up    |   Down  |   OK    |\n");

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Adsl part */
		{
			/* Retrieve but state */
			_userState = kSysGetButtonState(kButtonReboot);
	
			switch (_userState) {
				case kButtonPushed:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   1    |");
				}
				break;
	
				case kButtonReleased:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   0    |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet green part */
		{
			/* Retrieve but state */
			_userState = kSysGetButtonState(kButtonReset);
	
			switch (_userState) {
				case kButtonPushed:
				{
					/* Print */
					sprintf(	_pTampon,
							"   1   |");
				}
				break;
	
				case kButtonReleased:
				{
					/* Print */
					sprintf(	_pTampon,
							"   0   |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"   -   |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet red part */
		{
			/* Retrieve but state */
			_userState = kSysGetButtonState(kButtonPairing);
	
			switch (_userState) {
				case kButtonPushed:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1    |");
				}
				break;
	
				case kButtonReleased:
				{
					/* Print */
					sprintf(	_pTampon,
							"    0    |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet red part */
		{
			/* Retrieve but state */
			_userState = kSysGetButtonState(kButtonUp);

			switch (_userState) {
				case kButtonPushed:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1    |");
				}
				break;

				case kButtonReleased:
				{
					/* Print */
					sprintf(	_pTampon,
							"    0    |");
				}
				break;

				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet red part */
		{
			/* Retrieve but state */
			_userState = kSysGetButtonState(kButtonDown);

			switch (_userState) {
				case kButtonPushed:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1    |");
				}
				break;

				case kButtonReleased:
				{
					/* Print */
					sprintf(	_pTampon,
							"    0    |");
				}
				break;

				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet red part */
		{
			/* Retrieve but state */
			_userState = kSysGetButtonState(kButtonOk);

			switch (_userState) {
				case kButtonPushed:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1    |");
				}
				break;

				case kButtonReleased:
				{
					/* Print */
					sprintf(	_pTampon,
							"    0    |");
				}
				break;

				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);
	}
	else {
		/* Print */
		sprintf(	_pTampon,
				"Process Error\n");

		/* Next */
		_pTampon += strlen(_pTampon);
	}

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Call proc_dostring */
	return(SYSCTL_PROC_DOSTRING(	_pTable,
						_write,
						_pFile,
						_pBuffer,
						_pLen,
						_pPpos));
}



/** register callback for interrupt. */
void kSysButtonRegisterCb (BOARD_BUTTON_NAME _nameUser, void *ptr)
	{
	bsp_gpio_name_e	_nameGpio;
	int		_numGpio;
	//unsigned long     flags;

	/* Retrieve nameGpio */
	_nameGpio = bsp_base.Index.But.Name.user2gpio[_nameUser];

	/* and his num */
	_numGpio = bsp_gpio_get_num(_nameGpio);

	if (_numGpio != -1) {
	   /* save pointer */
	   bsp_base.interrupt_cb_p[_numGpio] = ptr;
   }
}
EXPORT_SYMBOL(kSysButtonRegisterCb);

/** deregister callback for interrupt. */
void kSysButtonUnRegisterCb (BOARD_BUTTON_NAME _nameUser)
	{
	bsp_gpio_name_e	_nameGpio;
	int		_numGpio;

	/* Retrieve nameGpio */
	_nameGpio = bsp_base.Index.But.Name.user2gpio[_nameUser];

	/* and his num */
	_numGpio = bsp_gpio_get_num(_nameGpio);

	if (_numGpio != -1) {
		/* save pointer */
		bsp_base.interrupt_cb_p[_numGpio] = 0;
	}
}
EXPORT_SYMBOL(kSysButtonUnRegisterCb);

// vim:ts=5:
