/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_glob.c Bsp Global Parameters
 *
 * This file contain all Bsp Global parameters.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

/*
 * Globals
 * ~~~~~~~
 */

/* Base struct. */
bsp_base_s		bsp_base = {
	.debug										= 0,
	.flags										= 0,
	/* Sysctl. */
	.SysCtl.head									= (struct ctl_table_header *)0,
	/** Sagem. */
	.SysCtl.sagem[0]								= {
		.ctl_name								= BSP_SysCtl_sagem,
		.procname								= "sagem",
		.mode									= 0555,
		.child									= bsp_base.SysCtl.sagem_list,
	},
	.SysCtl.sagem[1]								= {
		.ctl_name								= 0,
	},
	/** Sagem List. */
	.SysCtl.sagem_list[0]								= {
		.ctl_name								= BSP_SysCtl_sagem_list_type_firmware,
		.procname								= "firmwareType",
		.data									= &(bsp_base.Env.Krnopt.typeFirmware),
		.maxlen									= sizeof(bsp_base.Env.Krnopt.typeFirmware),
		.mode									= 0644,
		.proc_handler								= &proc_dointvec,
	},
	.SysCtl.sagem_list[1]								= {
		.ctl_name								= BSP_SysCtl_sagem_list_type_board,
		.procname								= "boardType",
		.data									= &(bsp_base.Board.Cur.boardType),
		.maxlen									= sizeof(bsp_base.Board.Cur.boardType),
		.mode									= 0444,
		.proc_handler								= &proc_dointvec,
	},
	.SysCtl.sagem_list[2]								= {
		.ctl_name								= BSP_SysCtl_sagem_list_boot,
		.procname								= "boot",
		.mode									= 0555,
		.child									= bsp_base.SysCtl.sagem_list_boot,
	},
	.SysCtl.sagem_list[3]								= {
		.ctl_name								= BSP_SysCtl_sagem_list_relay,
		.procname								= "relay",
		.mode									= 0555,
		.child									= bsp_base.SysCtl.sagem_list_relay,
	},
	.SysCtl.sagem_list[4]								= {
		.ctl_name								= BSP_SysCtl_sagem_list_led,
		.procname								= "led",
		.mode									= 0555,
		.child									= bsp_base.SysCtl.sagem_list_led,
	},
	.SysCtl.sagem_list[5]								= {
		.ctl_name								= BSP_SysCtl_sagem_list_but,
		.procname								= "but",
		.mode									= 0555,
		.child									= bsp_base.SysCtl.sagem_list_but,
	},
	.SysCtl.sagem_list[6]								= {
		.ctl_name								= BSP_SysCtl_sagem_list_gpio,
		.procname								= "gpio",
		.mode									= 0555,
		.child									= bsp_base.SysCtl.sagem_list_gpio,
	},
	.SysCtl.sagem_list[7]								= {
		.ctl_name								= 0,
	},
	/** Sagem List Boot. */
	.SysCtl.sagem_list_boot[0]							= {
		.ctl_name								= BSP_SysCtl_sagem_list_boot_recovery_force,
		.procname								= "recoveryForce",
		.data									= &(bsp_base.Env.Boot.Recovery.force),
		.maxlen									= sizeof(bsp_base.Env.Boot.Recovery.force),
		.mode									= 0644,
		.proc_handler								= &bsp_boot_recovery_force_proc_dointvec_minmax,
		.strategy								= &sysctl_intvec,
		.extra1									= &bsp_base.Env.Boot.Recovery.min,
		.extra2									= &bsp_base.Env.Boot.Recovery.max,
	},
	.SysCtl.sagem_list_boot[1]							= {
		.ctl_name								= 0,
	},
 	/** Sagem List led. */
	.SysCtl.sagem_list_led[0]							= {
		.ctl_name								= BSP_SysCtl_sagem_list_led_maps,
		.procname								= "maps",
		.data									= bsp_base.SysCtl.tampon,
		.maxlen									= BSP_SYSCTL_TAMPON_MAX_SIZE,
		.mode									= 0444,
		.proc_handler								= &bsp_led_maps_proc,
	},
	.SysCtl.sagem_list_led[1]							= {
		.ctl_name								= 0,
	},
	/** Sagem List but. */
	.SysCtl.sagem_list_but[0]							= {
		.ctl_name								= BSP_SysCtl_sagem_list_but_maps,
		.procname								= "maps",
		.data									= bsp_base.SysCtl.tampon,
		.maxlen									= BSP_SYSCTL_TAMPON_MAX_SIZE,
		.mode									= 0444,
		.proc_handler								= &bsp_but_maps_proc,
	},
	.SysCtl.sagem_list_but[1]							= {
		.ctl_name								= 0,
	},
	/** Sagem List gpio maps. */
	.SysCtl.sagem_list_gpio[0]							= {
		.ctl_name								= BSP_SysCtl_sagem_list_gpio_maps,
		.procname								= "maps",
		.data									= bsp_base.SysCtl.tampon,
		.maxlen									= BSP_SYSCTL_TAMPON_MAX_SIZE,
		.mode									= 0444,
		.proc_handler								= &bsp_gpio_maps_proc,
	},
	.SysCtl.sagem_list_gpio[1]							= {
		.ctl_name								= 0,
	},
	/** Dev Perm Param. */
	.Dev[BSP_Dev_minor_pp]								= {
		.name									= "bsp_dev",
		.fops.open								= bsp_devOpen,
		.fops.release								= bsp_devRelease,
		.fops.ioctl								= bsp_devIoctl,
	},
	/** Perm Parm Address. */
	.Pp.pFlashAddr									= (pBsp_permParam_s)(BSP_FLASH_ADDR_PERMPARM),
	/** Perm Parm callback. */
	.Pp.getPpCb[BSP_Get_permParam_all] 						= (bsp_getPermParamCb_f)bsp_get_permParam_all,
	.Pp.getPpCb[BSP_Get_permParam_version] 						= (bsp_getPermParamCb_f)bsp_get_permParam_version,
	.Pp.getPpCb[BSP_Get_permParam_serial_number]					= (bsp_getPermParamCb_f)bsp_get_permParam_serialNumber,
	.Pp.getPpCb[BSP_Get_permParam_sagem_product_id]					= (bsp_getPermParamCb_f)bsp_get_permParam_sagemProductId,
	.Pp.getPpCb[BSP_Get_permParam_sagem_hardware_version] 				= (bsp_getPermParamCb_f)bsp_get_permParam_sagemHardwareVersion,
	.Pp.getPpCb[BSP_Get_permParam_sagem_hardware_revision]				= (bsp_getPermParamCb_f)bsp_get_permParam_sagemHardwareRevision,
	.Pp.getPpCb[BSP_Get_permParam_customer_product_id] 				= (bsp_getPermParamCb_f)bsp_get_permParam_customerProductId,
	.Pp.getPpCb[BSP_Get_permParam_customer_hardware_version] 			= (bsp_getPermParamCb_f)bsp_get_permParam_customerHardwareVersion,
	.Pp.getPpCb[BSP_Get_permParam_customer_hardware_revision]			= (bsp_getPermParamCb_f)bsp_get_permParam_customerHardwareRevision,
	.Pp.getPpCb[BSP_Get_permParam_ram_size] 					= (bsp_getPermParamCb_f)bsp_get_permParam_ramSize,
	.Pp.getPpCb[BSP_Get_permParam_flash_size] 					= (bsp_getPermParamCb_f)bsp_get_permParam_flashSize,
	.Pp.getPpCb[BSP_Get_permParam_firmware_version] 				= (bsp_getPermParamCb_f)bsp_get_permParam_firmVersion,
	.Pp.getPpCb[BSP_Get_permParam_lan_base_mac_address]				= (bsp_getPermParamCb_f)bsp_get_permParam_lanBaseMacAddr,
	.Pp.getPpCb[BSP_Get_permParam_lan_mac_address_range]				= (bsp_getPermParamCb_f)bsp_get_permParam_lanMacAddrRange,
	.Pp.getPpCb[BSP_Get_permParam_wan_base_mac_address] 				= (bsp_getPermParamCb_f)bsp_get_permParam_wanBaseMacAddr,
	.Pp.getPpCb[BSP_Get_permParam_wan_mac_address_range]				= (bsp_getPermParamCb_f)bsp_get_permParam_wanMacAddrRange,
	.Pp.getPpCb[BSP_Get_permParam_usb_mac] 						= (bsp_getPermParamCb_f)bsp_get_permParam_usbMacAddr,
	.Pp.getPpCb[BSP_Get_permParam_wifi_mac_address_1]				= (bsp_getPermParamCb_f)bsp_get_permParam_wifiMac1Addr,
	.Pp.getPpCb[BSP_Get_permParam_wifi_ssid_1] 					= (bsp_getPermParamCb_f)bsp_get_permParam_wifiSsid1,
	.Pp.getPpCb[BSP_Get_permParam_wifi_mac_address_2] 				= (bsp_getPermParamCb_f)bsp_get_permParam_wifiMac2Addr,
	.Pp.getPpCb[BSP_Get_permParam_wifi_ssid_2] 					= (bsp_getPermParamCb_f)bsp_get_permParam_wifiSsid2,
	.Pp.getPpCb[BSP_Get_permParam_manufacturer] 					= (bsp_getPermParamCb_f)bsp_get_permParam_manufacturer,
	.Pp.getPpCb[BSP_Get_permParam_manufacturer_oui]					= (bsp_getPermParamCb_f)bsp_get_permParam_manufacturerBaseMacAddr,
	.Pp.getPpCb[BSP_Get_permParam_manufacturer_url] 				= (bsp_getPermParamCb_f)bsp_get_permParam_manufacturerUrl,
	.Pp.getPpCb[BSP_Get_permParam_country_code] 					= (bsp_getPermParamCb_f)bsp_get_permParam_comCountry,
	.Pp.getPpCb[BSP_Get_permParam_wifi_key_1] 					= (bsp_getPermParamCb_f)bsp_get_permParam_wifiKey1,
	.Pp.getPpCb[BSP_Get_permParam_wifi_key_2] 					= (bsp_getPermParamCb_f)bsp_get_permParam_wifiKey2,
	.Pp.getPpCb[BSP_Get_permParam_h235_key] 					= (bsp_getPermParamCb_f)bsp_get_permParam_h235Key,
	.Pp.getPpCb[BSP_Get_permParam_public_dsa_syst] 					= (bsp_getPermParamCb_f)bsp_get_permParam_publicDsaSyst,
	.Pp.getPpCb[BSP_Get_permParam_public_dsa_resc] 					= (bsp_getPermParamCb_f)bsp_get_permParam_publicDsaResc,
	.Pp.getPpCb[BSP_Get_permParam_private_key] 					= (bsp_getPermParamCb_f)bsp_get_permParam_privateDsaKey,
	.Pp.getPpCb[BSP_Get_permParam_certificate] 					= (bsp_getPermParamCb_f)bsp_get_permParam_certificate,
	.Pp.getPpCb[BSP_Get_permParam_dsa] 						= (bsp_getPermParamCb_f)bsp_get_permParam_dsa,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_1] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo1,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_2] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo2,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_3] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo3,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_4] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo4,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_5] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo5,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_6] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo6,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_7] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo7,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_8] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo8,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_9] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo9,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_10] 					= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo10,
	/** Permparam To Board Unknown. */
	.Pp.pp2board[BSP_Pp2board_unknown]						= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_version]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_revision]				= (char *)0,
		.boardType								= BSP_Board_type_unknown,
		.boardInfo								= BSP_Board_info_unknown,
		.voxType												= BSP_Board_vox_type_160_180,
	},
	/** Permparam To Livebox ProV3 vox185. */
	.Pp.pp2board[BSP_Pp2board_lbv3]								= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= "SagemFast3965",
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= "0.1",
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= "1.0",
		.pInfo[BSP_Pp_info_customer_product_id]					= "LBPV3",
		.pInfo[BSP_Pp_info_customer_hardware_version]			= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType												= BSP_Board_type_lbv3,
		.boardInfo												= BSP_Board_info_lbv3,
		.voxType												= BSP_Board_vox_type_183_185,
	},
	/** Permparam To Livebox ProV3 vox185. version2 */
	.Pp.pp2board[BSP_Pp2board_lbv3_v2]								= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= "SagemFast3965",
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= "0.2",
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= "1.0",
		.pInfo[BSP_Pp_info_customer_product_id]					= "LBPV3",
		.pInfo[BSP_Pp_info_customer_hardware_version]			= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType												= BSP_Board_type_lbv3_v2,
		.boardInfo												= BSP_Board_info_lbv3_v2,
		.voxType												= BSP_Board_vox_type_183_185,
	},
	/** Permparam To Livebox 2.8 harware "v0" (aka "P1" at FT's) */                                                                                                                                             
	.Pp.pp2board[BSP_Pp2board_lbv28_v0] = {
		.pInfo[BSP_Pp_info_sagem_product_id] = "SagemcomFast3965_LB2.8",
		.pInfo[BSP_Pp_info_sagem_hardware_version] = "0.1",
		.pInfo[BSP_Pp_info_sagem_hardware_revision] = "1.0",
		.pInfo[BSP_Pp_info_customer_product_id] = "LB2.8",
		.pInfo[BSP_Pp_info_customer_hardware_version] = (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_revision] = (char *)0,
		.boardType = BSP_Board_type_lbv28_v0,
		.boardInfo = BSP_Board_info_lbv28_v0,
		.voxType = BSP_Board_vox_type_183_185,
	},  
	/** Permparam To Livebox 2.8 harware "v1" (aka "P2" at FT's) */                                                                                                                                             
	.Pp.pp2board[BSP_Pp2board_lbv28_v1] = {
		.pInfo[BSP_Pp_info_sagem_product_id] = "SagemcomFast3965_LB2.8",
		.pInfo[BSP_Pp_info_sagem_hardware_version] = "0.2",
		.pInfo[BSP_Pp_info_sagem_hardware_revision] = "1.0",
		.pInfo[BSP_Pp_info_customer_product_id] = "LB2.8",
		.pInfo[BSP_Pp_info_customer_hardware_version] = (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_revision] = (char *)0,
		.boardType = BSP_Board_type_lbv28_v1,
		.boardInfo = BSP_Board_info_lbv28_v1,
		.voxType = BSP_Board_vox_type_183_185,
	},  

/*------------------------------------------------------------------------------------------------------------*/

	/** Index. */
	.Index										= {
		/** Button. */
		.But.Name.user2gpio[kButtonReset]					= BSP_Gpio_name_button_reset,
		.But.Name.user2gpio[kButtonReboot]					= BSP_Gpio_name_button_reboot,
		.But.Name.user2gpio[kButtonPairing]					= BSP_Gpio_name_button_pairing,
		.But.Name.user2gpio[kButtonWifiOn] 					= BSP_Gpio_name_button_wifi_on,
		.But.Name.user2gpio[kButtonWakeupDisplay]				= BSP_Gpio_name_button_wakeup_display,
		.But.Name.user2gpio[kButtonUp]						= BSP_Gpio_name_buttonUp,
		.But.Name.user2gpio[kButtonDown]					= BSP_Gpio_name_buttonDown,
		.But.Name.user2gpio[kButtonOk]						= BSP_Gpio_name_buttonOk,

		.But.Name.gpio2user[BSP_Gpio_name_button_reset]		= kButtonReset,
		.But.Name.gpio2user[BSP_Gpio_name_button_reboot]	= kButtonReboot,
		.But.Name.gpio2user[BSP_Gpio_name_button_pairing]	= kButtonPairing,
		.But.Name.gpio2user[BSP_Gpio_name_button_wifi_on]		= kButtonWifiOn,
		.But.Name.gpio2user[BSP_Gpio_name_button_wakeup_display]	= kButtonWakeupDisplay,
		.But.Name.gpio2user[BSP_Gpio_name_buttonUp]			= kButtonUp,
		.But.Name.gpio2user[BSP_Gpio_name_buttonDown]		= kButtonDown,
		.But.Name.gpio2user[BSP_Gpio_name_buttonOk]			= kButtonOk,

		.But.State.user2gpio[kButtonReleased]				= BSP_Gpio_state_unset,
		.But.State.user2gpio[kButtonPushed]					= BSP_Gpio_state_set,
		.But.State.gpio2user[BSP_Gpio_state_unset]			= kButtonReleased,
		.But.State.gpio2user[BSP_Gpio_state_set]			= kButtonPushed,

		/** Led. */
		.Led.Name.user2gpio[kLedPower]					= BSP_Gpio_name_led_power,
		.Led.Name.user2gpio[kLedInternet_SynchroLine]		= BSP_Gpio_name_led_internet_red,
		.Led.Name.user2gpio[kLedInternet_DataConnectivity]	= BSP_Gpio_name_led_internet_green,
		.Led.Name.user2gpio[kLedVoip1]					= BSP_Gpio_name_led_phone_1,
//		.Led.Name.user2gpio[kLedVoip2]					= BSP_Gpio_name_led_phone_2,
		.Led.Name.user2gpio[kLedLan]						= BSP_Gpio_name_led_lan,
		.Led.Name.user2gpio[kLedWireless]					= BSP_Gpio_name_led_wireless,
		.Led.Name.user2gpio[kLedUpgrade]					= BSP_Gpio_name_led_upgrade,
		.Led.Name.user2gpio[kLedScreen_Green]                  = BSP_Gpio_name_led_screen_green,
		.Led.Name.user2gpio[kLedScreen_Red]		          = BSP_Gpio_name_led_screen_red,

#if (defined HALBSP_PERGPIO_PROCENTRY)

		/* Akka(S.VIE), 2011-12-01: Per-LED proc entries.
		 */
		.Led.Name.user2proc[kLedWireless]                      = BSP_Proc_name_led_wireless,
		.Led.Name.user2proc[kLedScreen_Red]                    = BSP_Proc_name_led_screen_red,
		.Led.Name.user2proc[kLedScreen_Green]                  = BSP_Proc_name_led_screen_green,

#endif /* HALBSP_PERGPIO_PROCENTRY */

		.Led.State.user2gpio[kLedStateOff]					= BSP_Gpio_state_unset,
		.Led.State.user2gpio[kLedStateOn]					= BSP_Gpio_state_set,
		.Led.State.gpio2user[BSP_Gpio_state_unset]			= kLedStateOff,
		.Led.State.gpio2user[BSP_Gpio_state_set]			= kLedStateOn,
	},
	/** Env. */
	.Env										= {
		/** Boot. */
		.Boot.Recovery.force						= BSP_Boot_recovery_force_unknown,
		.Boot.Recovery.force						= BSP_Boot_recovery_force_unknown,
		.Boot.Recovery.min							= 0,
		.Boot.Recovery.max							= BSP_Boot_recovery_force_max - 1,
		/** Kernel Options. */
		.Krnopt.typeFirmware						= BSP_Firmware_type_unknown,
		.Krnopt.myfs_start							= 0,
		.Krnopt.myfs_size							= 0,
		.Krnopt.operational_start					= 0,
		.Krnopt.rescue_start						= 0,
	},
	/** Board. */
	.Board										= {
		/** Gpio Intr. */
		.GpioIntr.low								= BSP_Gpio_intr_0_to_15,
		.GpioIntr.high								= BSP_Gpio_intr_16_to_31,
		/** Current Type. */
		.Cur.pp2board								= BSP_Pp2board_unknown,
		.Cur.boardType								= BSP_Board_type_unknown,
		.Cur.boardInfo								= BSP_Board_info_unknown,

		/** Gpio Name. */

		.pGpioNameTxt[BSP_Gpio_name_reserved]					= "        Reserved        ",
		.pGpioNameTxt[BSP_Gpio_name_led_internet_green]			= "    Led Synchro line    ",
		.pGpioNameTxt[BSP_Gpio_name_led_internet_red]			= "   Led internet data    ",
		.pGpioNameTxt[BSP_Gpio_name_led_wireless]				= "      Led wireless      ",
		.pGpioNameTxt[BSP_Gpio_name_led_power]					= "        Led power       ",
		.pGpioNameTxt[BSP_Gpio_name_led_lan]					= "        Led lan         ",
		.pGpioNameTxt[BSP_Gpio_name_led_upgrade]				= "      Led upgrade       ",
		.pGpioNameTxt[BSP_Gpio_name_led_phone_1]				= "      Led phone 1       ",
		.pGpioNameTxt[BSP_Gpio_name_led_phone_2]				= "      Led phone 2       ",
		.pGpioNameTxt[BSP_Gpio_name_led_screen_green]			= "    Led Screen green    ",
		.pGpioNameTxt[BSP_Gpio_name_led_screen_red]  			= "     Led Screen red     ",

		.pGpioNameTxt[BSP_Gpio_name_led_usb]					= "        Led usb         ",
		.pGpioNameTxt[BSP_Gpio_name_lan_wakeup]			     	= "      Led LAN wakeup    ",

		.pGpioNameTxt[BSP_Gpio_name_button_reset]				= "       But reset        ",
		.pGpioNameTxt[BSP_Gpio_name_button_reboot]				= "       But reboot       ",
		.pGpioNameTxt[BSP_Gpio_name_button_pairing]				= "       But pairing      ",
		.pGpioNameTxt[BSP_Gpio_name_buttonUp]					= "        Button UP       ",
		.pGpioNameTxt[BSP_Gpio_name_buttonDown]					= "       Button DOWN      ",
		.pGpioNameTxt[BSP_Gpio_name_buttonOk]					= "        Button OK       ",
		.pGpioNameTxt[BSP_Gpio_name_button_wifi_on]	          	= "   Button Wifi ON/OFF   ",
		.pGpioNameTxt[BSP_Gpio_name_button_wakeup_display]		= " Button Wake-up Display ",

		.pGpioNameTxt[BSP_Gpio_name_relay_fxs_all]				= "      Relay Fxs all     ",
		.pGpioNameTxt[BSP_Gpio_name_relay_fxs_1]				= "     Relay Fxs 1        ",
		.pGpioNameTxt[BSP_Gpio_name_relay_fxs_2]				= "    Relay Fxs 2         ",
		.pGpioNameTxt[BSP_Gpio_name_relay_fxo]					= "    Relay Fxo           ",
		.pGpioNameTxt[BSP_Gpio_name_spi_cs_fxs_all_and_fxo]		= "  Spi Cs Fxs all & Fxo  ",
		.pGpioNameTxt[BSP_Gpio_name_spi_cs_fxs_all]				= "     Spi Cs Fxs all     ",
		.pGpioNameTxt[BSP_Gpio_name_spi_cs_fxo]					= "       Spi Cs Fxo       ",
		.pGpioNameTxt[BSP_Gpio_name_spi_cs_dect]				= "      Spi Cs DECT       ",
		.pGpioNameTxt[BSP_Gpio_name_spi_cs_zigbee]				= "     Spi Cs Zigbee      ",
		.pGpioNameTxt[BSP_Gpio_name_spi_cs_oled]				= "      Spi Cs Oled       ",

		.pGpioNameTxt[BSP_Gpio_name_voice_rst_fxs_all_and_fxo]		= "   Reset Fxs all & Fxo  ",
		.pGpioNameTxt[BSP_Gpio_name_voice_rst_fxs_1]				= "     Reset Fxs 1        ",
		.pGpioNameTxt[BSP_Gpio_name_voice_rst_fxs_2]				= "     Reset Fxs 2        ",
		.pGpioNameTxt[BSP_Gpio_name_voice_rst_fxs_2_and_fxo]		= "     Reset Fxs 2 & Fxo  ",
		.pGpioNameTxt[BSP_Gpio_name_voice_rst_dect]				= "      Reset DECT        ",
		.pGpioNameTxt[BSP_Gpio_name_voice_rst_zigbee]			= "      Reset Zigbee      ",
		.pGpioNameTxt[BSP_Gpio_name_rst_oled]				    = "       Oled Reset       ",
		.pGpioNameTxt[BSP_Gpio_name_oled_data_cmd]			    = "      Oled DataCom      ",

		.pGpioNameTxt[BSP_Gpio_name_voice_int_fxs_1]				= "       Int Fxs 1        ",
		.pGpioNameTxt[BSP_Gpio_name_voice_int_fxs_2]				= "       Int Fxs 2        ",
		.pGpioNameTxt[BSP_Gpio_name_voice_int_fxs_2_and_fxo]		= "      Int Fxs 2 & Fxo   ",
		.pGpioNameTxt[BSP_Gpio_name_voice_int_dect]				= "        Int DECT        ",
		.pGpioNameTxt[BSP_Gpio_name_voice_int_zigbee]			= "       Int Zigbee       ",

		.pGpioNameTxt[BSP_Gpio_name_config_1]					= "        Config 1        ",
		.pGpioNameTxt[BSP_Gpio_name_config_2]					= "        Config 2        ",
		.pGpioNameTxt[BSP_Gpio_name_config_3]					= "        Config 3        ",
		.pGpioNameTxt[BSP_Gpio_name_led_card]					= "   Presence LED card    ",

		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_en_master_all]		= "  Usb pwr en master all ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_en_master_1]			= "  Usb pwr en master 1   ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_en_master_2]			= "  Usb pwr en master 2   ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_en_master_3]			= "  Usb pwr en master 3   ",
		.pGpioNameTxt[BSP_Gpio_name_usb_overcurrent_master_all]	= "  Usb overcur master all",
		.pGpioNameTxt[BSP_Gpio_name_usb_overcurrent_master_1]		= "  Usb overcur master 1  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_overcurrent_master_2]		= "  Usb overcur master 2  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_overcurrent_master_3]		= "  Usb overcur master 3  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_en_slave]			= "  Usb pwr en slave      ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_detect_slave]			= "  Usb pwr detect slave  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_rst_hub]				= "        Usb rst hub     ",
		.pGpioNameTxt[BSP_Gpio_name_mini_led_act_eth_all]		= "  Mini led act eth all  ",
		.pGpioNameTxt[BSP_Gpio_name_mini_led_act_eth_0]			= "   Mini led act eth 0   ",
		.pGpioNameTxt[BSP_Gpio_name_mini_led_act_eth_1]			= "   Mini led act eth 1   ",
		.pGpioNameTxt[BSP_Gpio_name_mini_pin_crs_eth_0]			= "   Mini pin crs eth 0   ",
		.pGpioNameTxt[BSP_Gpio_name_mini_pin_crs_eth_1]			= "   Mini pin crs eth 1   ",
		.pGpioNameTxt[BSP_Gpio_name_adsl_rst_afe]				= "       Adsl rst afe     ",
		.pGpioNameTxt[BSP_Gpio_name_dying_gasp]					= "        Dying gasp      ",
		.pGpioNameTxt[BSP_Gpio_name_switch_rst]					= "        Switch rst      ",
		.pGpioNameTxt[BSP_Gpio_name_switch_int]					= "        Switch int      ",
		.pGpioNameTxt[BSP_Gpio_name_phy_rst_all]				= "       Phy rst all      ",
		.pGpioNameTxt[BSP_Gpio_name_phy_rst_1]					= "        Phy rst 1       ",
		.pGpioNameTxt[BSP_Gpio_name_phy_rst_2]					= "        Phy rst 2       ",
		.pGpioNameTxt[BSP_Gpio_name_wifi_rst]					= "        Wifi rst        ",
		.pGpioNameTxt[BSP_Gpio_name_wifi_rst_5G]				= "    Wifi rst  5 Giga    ",
		.pGpioNameTxt[BSP_Gpio_name_wifi_rst_2G4]				= "   Wifi rst  2.4 Giga   ",
		
		.pGpioNameTxt[BSP_Gpio_name_cmd_nl]						= "         Cmd Nl         ",
		.pGpioNameTxt[BSP_Gpio_name_green_rst]					= "       Green  rst       ",
		.pGpioNameTxt[BSP_Gpio_name_green_boot0]				= "       Green  boot0     ",
		.pGpioNameTxt[BSP_Gpio_name_green_boot1]				= "       Green  boot1     ",
		.pGpioNameTxt[BSP_Gpio_name_voice_det_fxs1]				= "     detection FXS1     ",
		.pGpioNameTxt[BSP_Gpio_name_voice_det_fxs2]				= "     detection FXS2     ",

#if (defined HALBSP_PERGPIO_PROCENTRY)

		/** Proc Names. */

		.pProcNameTxt[BSP_Proc_name_led_wireless]				     = "led-wireless",
		.pProcNameTxt[BSP_Proc_name_led_screen_green]		     = "led-screen-green",
		.pProcNameTxt[BSP_Proc_name_led_screen_red]			     = "led-screen-red",

#endif /* HALBSP_PERGPIO_PROCENTRY */

	},

	/*----------------------------------------------------------------------------------------------------------------*/
	.interrupt_cb_p[0]	= 0,
	.interrupt_cb_p[1]	= 0,
	.interrupt_cb_p[2]	= 0,
	.interrupt_cb_p[3]	= 0,
	.interrupt_cb_p[4]	= 0,
	.interrupt_cb_p[5]	= 0,
	.interrupt_cb_p[6]	= 0,
	.interrupt_cb_p[7]	= 0,
	.interrupt_cb_p[8]	= 0,
	.interrupt_cb_p[9]	= 0,
	.interrupt_cb_p[10]	= 0,
	.interrupt_cb_p[11]	= 0,
	.interrupt_cb_p[12]	= 0,
	.interrupt_cb_p[13]	= 0,
	.interrupt_cb_p[14]	= 0,
	.interrupt_cb_p[15]	= 0,
	.interrupt_cb_p[16]	= 0,
	.interrupt_cb_p[17]	= 0,
	.interrupt_cb_p[18]	= 0,
	.interrupt_cb_p[19]	= 0,
	.interrupt_cb_p[20]	= 0,
	.interrupt_cb_p[21]	= 0,
	.interrupt_cb_p[22]	= 0,
	.interrupt_cb_p[23]	= 0,
	.interrupt_cb_p[24]	= 0,
	.interrupt_cb_p[25]	= 0,
	.interrupt_cb_p[26]	= 0,
	.interrupt_cb_p[27]	= 0,
	.interrupt_cb_p[28]	= 0,
	.interrupt_cb_p[29]	= 0,
	.interrupt_cb_p[30]	= 0,
	.interrupt_cb_p[31]	= 0,
	.int_mask	= 0,
	.int_current_state = 0,
	/*----------------------------------------------------------------------------------------------------------------*/
	/** Board All Unknwon. */
	.Board.All[BSP_Board_info_unknown]		= {
		.Gpio[0].name				= BSP_Gpio_name_reserved,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[1].name				= BSP_Gpio_name_reserved,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[2].name				= BSP_Gpio_name_reserved,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[3].name				= BSP_Gpio_name_reserved,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[4].name				= BSP_Gpio_name_reserved,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[5].name				= BSP_Gpio_name_reserved,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[6].name				= BSP_Gpio_name_reserved,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[7].name				= BSP_Gpio_name_reserved,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[8].name				= BSP_Gpio_name_reserved,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[9].name				= BSP_Gpio_name_reserved,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[10].name				= BSP_Gpio_name_reserved,
		.Gpio[10].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[11].name				= BSP_Gpio_name_reserved,
		.Gpio[11].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[12].name				= BSP_Gpio_name_reserved,
		.Gpio[12].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[13].name				= BSP_Gpio_name_reserved,
		.Gpio[13].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[14].name				= BSP_Gpio_name_reserved,
		.Gpio[14].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[15].name				= BSP_Gpio_name_reserved,
		.Gpio[15].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[16].name				= BSP_Gpio_name_reserved,
		.Gpio[16].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[17].name				= BSP_Gpio_name_reserved,
		.Gpio[17].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[18].name				= BSP_Gpio_name_reserved,
		.Gpio[18].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[19].name				= BSP_Gpio_name_reserved,
		.Gpio[19].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[20].name				= BSP_Gpio_name_reserved,
		.Gpio[20].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[21].name				= BSP_Gpio_name_reserved,
		.Gpio[21].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[22].name				= BSP_Gpio_name_reserved,
		.Gpio[22].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[23].name				= BSP_Gpio_name_reserved,
		.Gpio[23].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[24].name				= BSP_Gpio_name_reserved,
		.Gpio[24].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[25].name				= BSP_Gpio_name_reserved,
		.Gpio[25].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[26].name				= BSP_Gpio_name_reserved,
		.Gpio[26].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[27].name				= BSP_Gpio_name_reserved,
		.Gpio[27].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[28].name				= BSP_Gpio_name_reserved,
		.Gpio[28].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[29].name				= BSP_Gpio_name_reserved,
		.Gpio[29].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[30].name				= BSP_Gpio_name_reserved,
		.Gpio[30].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[31].name				= BSP_Gpio_name_reserved,
		.Gpio[31].flags				= (BSP_Gpio_flags_dir_out),
	},

	/*----------------------------------------------------------------------------------------------------------------*/
	/** Board  lb ProV3. */
	.Board.All[BSP_Board_info_lbv3]		= {
		.Gpio[0].name				= BSP_Gpio_name_switch_rst,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[1].name				= BSP_Gpio_name_led_power,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[2].name				= BSP_Gpio_name_voice_rst_dect,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[3].name				= BSP_Gpio_name_voice_rst_fxs_1,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[4].name				= BSP_Gpio_name_voice_rst_fxs_2_and_fxo,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[5].name				= BSP_Gpio_name_spi_cs_fxs_all,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		
		.Gpio[6].name				= BSP_Gpio_name_spi_cs_fxs_all_and_fxo,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		
		.Gpio[7].name				= BSP_Gpio_name_spi_cs_dect,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		
		.Gpio[8].name				= BSP_Gpio_name_spi_cs_zigbee,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
				
		.Gpio[9].name				= BSP_Gpio_name_buttonUp,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[10].name				= BSP_Gpio_name_buttonOk,
		.Gpio[10].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[11].name				= BSP_Gpio_name_reserved,		// UART2_TXD (Green)
		.Gpio[11].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[12].name				= BSP_Gpio_name_reserved,		// UART2_RXD (Green)
		.Gpio[12].flags				= (BSP_Gpio_flags_dir_out),		// input

		.Gpio[13].name				= BSP_Gpio_name_green_rst,		// GREEN_RST_N
		.Gpio[13].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[14].name				= BSP_Gpio_name_green_boot1,		// GREEN_BOOT1
		.Gpio[14].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[15].name				= BSP_Gpio_name_config_1,
		.Gpio[15].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),

		.Gpio[16].name				= BSP_Gpio_name_config_2,
		.Gpio[16].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),

		.Gpio[17].name				= BSP_Gpio_name_green_boot0,		// GREEN_BOOT0
		.Gpio[17].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[18].name				= BSP_Gpio_name_usb_pwr_en_master_1,
		.Gpio[18].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[19].name				= BSP_Gpio_name_wifi_rst_5G,
		.Gpio[19].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[20].name				= BSP_Gpio_name_led_wireless,
		.Gpio[20].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[21].name				= BSP_Gpio_name_voice_rst_zigbee,
		.Gpio[21].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[22].name				= BSP_Gpio_name_wifi_rst_2G4,
		.Gpio[22].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[23].name				= BSP_Gpio_name_buttonDown,
		.Gpio[23].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[24].name				= BSP_Gpio_name_switch_int,
		.Gpio[24].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[25].name				= BSP_Gpio_name_voice_int_fxs_1,
		.Gpio[25].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[26].name				= BSP_Gpio_name_voice_int_fxs_2,
		.Gpio[26].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[27].name				= BSP_Gpio_name_voice_int_zigbee,
		.Gpio[27].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[28].name				= BSP_Gpio_name_button_pairing,
		.Gpio[28].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_soft_intr ),

		.Gpio[29].name				= BSP_Gpio_name_voice_int_dect,
		.Gpio[29].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[30].name				= BSP_Gpio_name_dying_gasp,
		.Gpio[30].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[31].name				= BSP_Gpio_name_button_reset,
		.Gpio[31].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),
	},
	/*----------------------------------------------------------------------------------------------------------------*/
	/** Board  lb ProV3 V2. */
	.Board.All[BSP_Board_info_lbv3_v2]		= {
		.Gpio[0].name				= BSP_Gpio_name_switch_rst,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[1].name				= BSP_Gpio_name_led_power,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[2].name				= BSP_Gpio_name_voice_rst_dect,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[3].name				= BSP_Gpio_name_voice_rst_fxs_1,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[4].name				= BSP_Gpio_name_voice_rst_fxs_2_and_fxo,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[5].name				= BSP_Gpio_name_spi_cs_fxs_all,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		.Gpio[6].name				= BSP_Gpio_name_spi_cs_fxs_all_and_fxo,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		.Gpio[7].name				= BSP_Gpio_name_spi_cs_dect,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		.Gpio[8].name				= BSP_Gpio_name_spi_cs_zigbee,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		.Gpio[9].name				= BSP_Gpio_name_buttonUp,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[10].name				= BSP_Gpio_name_buttonOk,
		.Gpio[10].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[11].name				= BSP_Gpio_name_reserved,		// UART2_TXD (Green)
		.Gpio[11].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[12].name				= BSP_Gpio_name_reserved,		// UART2_RXD (Green)
		.Gpio[12].flags				= (BSP_Gpio_flags_dir_out),		// input

		.Gpio[13].name				= BSP_Gpio_name_green_rst,		// GREEN_RST_N
		.Gpio[13].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[14].name				= BSP_Gpio_name_green_boot1,		// GREEN_BOOT1
		.Gpio[14].flags				= (BSP_Gpio_flags_dir_out_opendrain | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[15].name				= BSP_Gpio_name_voice_det_fxs1,
		.Gpio[15].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),

		.Gpio[16].name				= BSP_Gpio_name_voice_det_fxs2,
		.Gpio[16].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),

		.Gpio[17].name				= BSP_Gpio_name_green_boot0,		// GREEN_BOOT0
		.Gpio[17].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[18].name				= BSP_Gpio_name_usb_pwr_en_master_1,
		.Gpio[18].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[19].name				= BSP_Gpio_name_wifi_rst_5G,
		.Gpio[19].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[20].name				= BSP_Gpio_name_led_wireless,
		.Gpio[20].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[21].name				= BSP_Gpio_name_voice_rst_zigbee,
		.Gpio[21].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[22].name				= BSP_Gpio_name_wifi_rst_2G4,
		.Gpio[22].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[23].name				= BSP_Gpio_name_buttonDown,
		.Gpio[23].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[24].name				= BSP_Gpio_name_switch_int,
		.Gpio[24].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[25].name				= BSP_Gpio_name_voice_int_fxs_1,
		.Gpio[25].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[26].name				= BSP_Gpio_name_voice_int_fxs_2,
		.Gpio[26].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[27].name				= BSP_Gpio_name_voice_int_zigbee,
		.Gpio[27].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[28].name				= BSP_Gpio_name_button_pairing,
		.Gpio[28].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_soft_intr ),

		.Gpio[29].name				= BSP_Gpio_name_voice_int_dect,
		.Gpio[29].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[30].name				= BSP_Gpio_name_dying_gasp,
		.Gpio[30].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[31].name				= BSP_Gpio_name_button_reset,
		.Gpio[31].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),
	},

	/*----------------------------------------------------------------------------------------------------------------*/
	/** Board "LiveBox 2.8" hardware "v0" (aka "P1" at FT's) */
	.Board.All[BSP_Board_info_lbv28_v0] = {
		.Gpio[0].name				= BSP_Gpio_name_switch_rst,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[1].name				= BSP_Gpio_name_reserved,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[2].name				= BSP_Gpio_name_voice_rst_dect,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[3].name				= BSP_Gpio_name_voice_rst_fxs_1,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[4].name				= BSP_Gpio_name_oled_data_cmd,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_used_as_gpio),

		.Gpio[5].name				= BSP_Gpio_name_spi_cs_fxs_all,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		.Gpio[6].name				= BSP_Gpio_name_reserved,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[7].name				= BSP_Gpio_name_spi_cs_dect,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		.Gpio[8].name				= BSP_Gpio_name_spi_cs_oled,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		.Gpio[9].name				= BSP_Gpio_name_led_screen_red,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio ),

		/* Akka (S.VIE), 2011-10-21: Here is the "wifi" green LED.
		 */
		.Gpio[10].name				= BSP_Gpio_name_led_wireless,
		.Gpio[10].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[11].name				= BSP_Gpio_name_reserved,
		.Gpio[11].flags				= (BSP_Gpio_flags_dir_out),

		/* Akka (S.VIE), 2011-10-21: LEDs "voip", "lan" and "upgrade" are for "LiveBox 2.5" (out of the scope of product "LiveBox 2.8").
		 */
#if 0
		.Gpio[12].name				= BSP_Gpio_name_led_voip,	
		.Gpio[12].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[13].name				= BSP_Gpio_name_led_lan,
		.Gpio[13].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[14].name				= BSP_Gpio_name_led_upgrade,
		.Gpio[14].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio ),
#else
		.Gpio[12].name				= BSP_Gpio_name_reserved,
		.Gpio[12].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[13].name				= BSP_Gpio_name_reserved,
		.Gpio[13].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[14].name				= BSP_Gpio_name_reserved,
		.Gpio[14].flags				= (BSP_Gpio_flags_dir_out),
#endif

		.Gpio[15].name				= BSP_Gpio_name_reserved,
		.Gpio[15].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[16].name				= BSP_Gpio_name_voice_det_fxs1,
		.Gpio[16].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),

		.Gpio[17].name				= BSP_Gpio_name_reserved,		// GREEN_BOOT0
		.Gpio[17].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[18].name				= BSP_Gpio_name_usb_pwr_en_master_1,
		.Gpio[18].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[19].name				= BSP_Gpio_name_wifi_rst_5G,
		.Gpio[19].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		/* Akka (S.VIE), 2011-10-21: The "power" green LED is for "LiveBox 2.5" (out of the scope of product "LiveBox 2.8").
		 */
#if 0
		.Gpio[20].name				= BSP_Gpio_name_led_power,
		.Gpio[20].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
#else
		.Gpio[20].name				= BSP_Gpio_name_reserved,
		.Gpio[20].flags				= (BSP_Gpio_flags_dir_out),
#endif

		.Gpio[21].name				= BSP_Gpio_name_rst_oled,
		.Gpio[21].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[22].name				= BSP_Gpio_name_wifi_rst_2G4,
		.Gpio[22].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[23].name				= BSP_Gpio_name_led_screen_green,
		.Gpio[23].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[24].name				= BSP_Gpio_name_switch_int,
		.Gpio[24].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[25].name				= BSP_Gpio_name_voice_int_fxs_1,
		.Gpio[25].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[26].name				= BSP_Gpio_name_button_wakeup_display,
		.Gpio[26].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),

		.Gpio[27].name				= BSP_Gpio_name_button_pairing,
		.Gpio[27].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),

		.Gpio[28].name				= BSP_Gpio_name_button_wifi_on,
		.Gpio[28].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),

		.Gpio[29].name				= BSP_Gpio_name_voice_int_dect,
		.Gpio[29].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[30].name				= BSP_Gpio_name_dying_gasp,
		.Gpio[30].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[31].name				= BSP_Gpio_name_button_reset,
		.Gpio[31].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),
	},

	/*----------------------------------------------------------------------------------------------------------------*/
	/** Board "LiveBox 2.8" hardware "v1" (aka "P2" at FT's) */
	.Board.All[BSP_Board_info_lbv28_v1] = {
		.Gpio[0].name				= BSP_Gpio_name_switch_rst,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		/* Akka(S.VIE), 2011-11-28: From v0 to v1, the "rst_oled" moved from gpio21 to gpio1. */
		.Gpio[1].name				= BSP_Gpio_name_rst_oled,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[2].name				= BSP_Gpio_name_voice_rst_dect,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[3].name				= BSP_Gpio_name_voice_rst_fxs_1,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),

		.Gpio[4].name				= BSP_Gpio_name_oled_data_cmd,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_used_as_gpio),

		.Gpio[5].name				= BSP_Gpio_name_spi_cs_fxs_all,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		.Gpio[6].name				= BSP_Gpio_name_reserved,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[7].name				= BSP_Gpio_name_spi_cs_dect,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		.Gpio[8].name				= BSP_Gpio_name_spi_cs_oled,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),

		/* Akka(S.VIE), 2011-12-12: From v0 to v1, the polarity of this LED passed from "active_high" to "active_low". */
		.Gpio[9].name				= BSP_Gpio_name_led_screen_red,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[10].name				= BSP_Gpio_name_led_wireless,
		.Gpio[10].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[11].name				= BSP_Gpio_name_reserved,
		.Gpio[11].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[12].name				= BSP_Gpio_name_reserved,
		.Gpio[12].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[13].name				= BSP_Gpio_name_reserved,
		.Gpio[13].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[14].name				= BSP_Gpio_name_reserved,
		.Gpio[14].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[15].name				= BSP_Gpio_name_reserved,
		.Gpio[15].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[16].name				= BSP_Gpio_name_voice_det_fxs1,
		.Gpio[16].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),

		.Gpio[17].name				= BSP_Gpio_name_reserved,		// GREEN_BOOT0
		.Gpio[17].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[18].name				= BSP_Gpio_name_usb_pwr_en_master_1,
		.Gpio[18].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),

		.Gpio[19].name				= BSP_Gpio_name_wifi_rst_5G,
		.Gpio[19].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[20].name				= BSP_Gpio_name_reserved,
		.Gpio[20].flags				= (BSP_Gpio_flags_dir_out),

		/* Akka(S.VIE), 2011-11-28: From v0 to v1, the "rst_oled" moved from gpio21 to gpio1. */
		.Gpio[21].name				= BSP_Gpio_name_reserved,
		.Gpio[21].flags				= (BSP_Gpio_flags_dir_out),

		.Gpio[22].name				= BSP_Gpio_name_wifi_rst_2G4,
		.Gpio[22].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		/* Akka(S.VIE), 2011-12-12: From v0 to v1, the polarity of this LED passed from "active_high" to "active_low". */
		.Gpio[23].name				= BSP_Gpio_name_led_screen_green,
		.Gpio[23].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio ),

		.Gpio[24].name				= BSP_Gpio_name_switch_int,
		.Gpio[24].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[25].name				= BSP_Gpio_name_voice_int_fxs_1,
		.Gpio[25].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[26].name				= BSP_Gpio_name_button_wakeup_display,
		.Gpio[26].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),

		.Gpio[27].name				= BSP_Gpio_name_button_pairing,
		.Gpio[27].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),

		.Gpio[28].name				= BSP_Gpio_name_button_wifi_on,
		.Gpio[28].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),

		.Gpio[29].name				= BSP_Gpio_name_voice_int_dect,
		.Gpio[29].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[30].name				= BSP_Gpio_name_dying_gasp,
		.Gpio[30].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio | BSP_Gpio_flags_intr | BSP_Gpio_flags_edge ),

		.Gpio[31].name				= BSP_Gpio_name_button_reset,
		.Gpio[31].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_soft_intr | BSP_Gpio_flags_used_as_gpio),
	},

};

// vim:ts=5:
