
/*
 * ============================================================================
 *
 * Copyright : (c) 2011 SAGEMCOM - Site Rueil-Malmaison - ATR-URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEMCOM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_gpio_common.c Common Bsp GPIO management.
 *
 * This file contain all Bsp Gpio functions common to all archs.
 * - Timer management for gpio polling when not using interruption
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

/** Get GPIO num */
unsigned int bsp_gpio_get_num(bsp_gpio_name_e _name)
{
	/* Gpio reserved */
	if (_name == BSP_Gpio_name_reserved) {
		/* If bsp */
		HALBSP_DEBUG(	"WARNING -> Reserved GPIO");

		return(-1);
	}

	/* If Gpio name Access null */
	if (bsp_base.Board.Cur.pGpioName[_name] == (pBsp_board_gpio_s)0) {
		/* If bsp */
		HALBSP_DEBUG(	"WARNING -> Unreachable GPIO");

		return(-1);
	}

	/* Out */
	return(bsp_base.Board.Cur.pGpioName[_name] - &(bsp_base.Board.Cur.pInfo->Gpio[0]));
}

#ifdef GPIO_USES_INTERRUPT
/* Interrupt */
irqreturn_t bsp_gpio_irq(	int			_irq,
						void			*_pParam,
		       			struct pt_regs *_pRegs)
{
	bsp_gpio_intr_e	_gpioIntr	= *((bsp_gpio_intr_e *)_pParam);
	int				_indiceGpio, _nameGpio;
	int				_min, _max;
	u16				_flags1, _flags2, _flagsSet1, _flagsSet2, _intSet1, _intSet2;
	u16 				_pol1, _pol2;
	BOARD_BUTTON_NAME		numBut;

	/* Load current Flags Set. */
	_flagsSet1 = bsp_gpio_read_regw(BSP_GPIO_FLAG_SET1);
	_flagsSet2 = bsp_gpio_read_regw(BSP_GPIO_FLAG_SET2);

	/* Load current Flags Set. */
	_flags1 = 0; //bsp_gpio_read_regw(BSP_GPIO_FLAG_CLEAR1);
	_flags2 = 0; //bsp_gpio_read_regw(BSP_GPIO_FLAG_CLEAR2);

	/* Load current int Set. */
	_intSet1 = bsp_gpio_read_regw(BSP_GPIO_INT_SET1);
	_intSet2 = bsp_gpio_read_regw(BSP_GPIO_INT_SET2);

	_pol1 = bsp_gpio_read_regw(BSP_GPIO_POL_SET1);
	_pol2 = bsp_gpio_read_regw(BSP_GPIO_POL_SET2);

	/* Gpio 0 to 15 */
	switch (_gpioIntr) {
		/* Gpio 0 to 15 */
		case BSP_Gpio_intr_0_to_15:
		{
			/* Set min/max */
			_min	= 0;
			_max	= 15;
		}
		break;

		/* Gpio 16 to 31 */
		case BSP_Gpio_intr_16_to_31:
		{
			/* Set min/max */
			_min	= 16;
			_max	= 31;
		}
		break;

		default:
		{
			HALBSP_ERROR("Error -> Type of gpioIntr(%u)",
					_gpioIntr);

			return(IRQ_HANDLED);
		}
		break;
	}

	/* All Gpio. */
	for (_indiceGpio = _min; _indiceGpio <= _max; _indiceGpio++) {
		/* Retrieve name gpio */
		_nameGpio = bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].name;

		/* Check */
		if (_indiceGpio < 16) {
			/* If int and flag */
			if ( (_intSet1 & (1 << _indiceGpio)) && (_flagsSet1 & ( 1 << _indiceGpio)) ) {
				if (bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].flags & BSP_Gpio_flags_edge)
					{
					_flags1 |= 1 << _indiceGpio;

					numBut = bsp_base.Index.But.Name.gpio2user[_nameGpio];

					if ((_pol1 & (1<< _indiceGpio)) != 0)
						{
						/* pol = 1 but is released */
						if (bsp_base.interrupt_cb_p[_indiceGpio] != 0)
							/* callback */
							(*bsp_base.interrupt_cb_p[_indiceGpio])(numBut, kButtonPushed);
						else
							HALBSP_INFO(" interrup1: %s pushed (user)%d", bsp_base.Board.pGpioNameTxt[_nameGpio],numBut);

						_pol1 &= ~(1<< _indiceGpio);
						}
					else
						{
						/* pol = 0 but is pushed */
						if (bsp_base.interrupt_cb_p[_indiceGpio] != 0)
							/* callback */
							(*bsp_base.interrupt_cb_p[_indiceGpio])(numBut, kButtonReleased );
						else
							HALBSP_INFO(" interrup1: %s released (user)%d", bsp_base.Board.pGpioNameTxt[_nameGpio],numBut);

						_pol1 |= 1<< _indiceGpio;
						}
					}
			}
		}
		else {
			/* If int and flag */
			if ( (_intSet2 & (1 << (_indiceGpio - 16))) && (_flagsSet2 & ( 1 << (_indiceGpio - 16))) ) {
				if (bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].flags & BSP_Gpio_flags_edge)
					{
					_flags2 |= 1 << (_indiceGpio - 16);

					numBut = bsp_base.Index.But.Name.gpio2user[_nameGpio];

					if ((_pol2 & (1<< (_indiceGpio - 16))) != 0)
						{
						/* pol = 1 but is released */
						if (bsp_base.interrupt_cb_p[_indiceGpio] != 0)
							/* callback */
							(*bsp_base.interrupt_cb_p[_indiceGpio])(numBut, kButtonPushed);
						else
							HALBSP_INFO(" interrup2: %s pushed (user)%d", bsp_base.Board.pGpioNameTxt[_nameGpio],numBut);

						_pol2 &= ~(1 << (_indiceGpio - 16));
						}
					else
						{
						/* pol = 0 but is pushed */
						if (bsp_base.interrupt_cb_p[_indiceGpio] != 0)
							/* callback */
							(*bsp_base.interrupt_cb_p[_indiceGpio])(numBut, kButtonReleased );
						else
							HALBSP_INFO(" interrup2: %s released (user)%d", bsp_base.Board.pGpioNameTxt[_nameGpio],numBut);

						_pol2 |= 1 << (_indiceGpio - 16);
						}
					}
			}
		}
	}

	bsp_gpio_write_regw (_flags1, BSP_GPIO_FLAG_CLEAR1);
	bsp_gpio_write_regw (_flags2, BSP_GPIO_FLAG_CLEAR2);

	bsp_gpio_write_regw (_pol1,BSP_GPIO_POL_SET1);
	bsp_gpio_write_regw (_pol2,BSP_GPIO_POL_SET2);

	return(IRQ_HANDLED);
}
#endif // GPIO_USES_INTERRUPT

#ifdef GPIO_USES_TIMER

/* TODO add IPC register for interrupt control */
/** kernel timer for polling input GPIO */
struct timer_list 		Gpio_polling_timer;

static void bsp_gpio_timer_initialize ( void)
{
	u32 _input;
	unsigned long     flags;

	/* store current state for soft input GPIO */
	_input = bsp_gpio_getAllValues();

	bsp_base.int_current_state = bsp_base.int_mask & _input;

	bsp_base.int_polling_flag = 0;

	HALBSP_DEBUG ("BSP soft interrupt: mask 0x%.8X init 0x%.8X", bsp_base.int_mask, bsp_base.int_current_state);

	init_timer(&Gpio_polling_timer);

	/* create timer 100 ms for polling input gpio */
	Gpio_polling_timer.expires = (100*HZ)/1000 + jiffies;

	/* init CB */
	Gpio_polling_timer.function = bsp_gpio_timer;
	
	/* param for CB */
	Gpio_polling_timer.data = 0;

	local_irq_save(flags);

	add_timer(&Gpio_polling_timer);

	local_irq_restore(flags);
}

void bsp_gpio_timer(unsigned long name)
{
	int						_indiceGpio, _nameGpio;
	BOARD_BUTTON_NAME		numBut;
	__u32 					input_status;
	unsigned long     flags;

	/* load current state of input */
	input_status = bsp_gpio_getAllValues();

	/* apply interrupt mask */
	input_status &= bsp_base.int_mask;

	/* All Gpio. */
	for (_indiceGpio = 0; _indiceGpio < BSP_GPIO_MAX; _indiceGpio++)
		{
		/* If soft int */
		if (bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].flags & BSP_Gpio_flags_soft_intr)
			{
			/* Retrieve name gpio */
			_nameGpio = bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].name;

			/* check changing state */
			if ( (input_status & (1<<_indiceGpio)) != (bsp_base.int_current_state & (1<<_indiceGpio)))
				{
				//HALBSP_DEBUG("change state  %.8X %.8X", input_status , bsp_base.int_current_state);
				numBut = bsp_base.Index.But.Name.gpio2user[_nameGpio];

				if ((input_status & (1<<_indiceGpio)) != BSP_Gpio_state_set<<_indiceGpio)
						{
						/* pol = 1 but is released */
						if (bsp_base.interrupt_cb_p[_indiceGpio] != 0)
							/* callback */
							(*bsp_base.interrupt_cb_p[_indiceGpio])(numBut, kButtonPushed);
						else
							HALBSP_INFO(" interrup1: %s pushed (user)%d", bsp_base.Board.pGpioNameTxt[_nameGpio],numBut);

						}
				else
						{
						/* pol = 0 but is pushed */
						if (bsp_base.interrupt_cb_p[_indiceGpio] != 0)
							/* callback */
							(*bsp_base.interrupt_cb_p[_indiceGpio])(numBut, kButtonReleased );
						else
							HALBSP_INFO(" interrup1: %s released (user)%d", bsp_base.Board.pGpioNameTxt[_nameGpio],numBut);

						}
				}
			}
		}

	/* store new current state for soft input GPIO */
	bsp_base.int_current_state = input_status;

	/* create timer 100 ms for polling input gpio */						
	local_irq_save(flags);

	Gpio_polling_timer.expires = (100*HZ)/1000 + jiffies;

	/* init CB */
	Gpio_polling_timer.function = bsp_gpio_timer;
	
	/* param for CB */
	Gpio_polling_timer.data = name;
	
	add_timer(&Gpio_polling_timer);	

	local_irq_restore(flags);
}
#endif // GPIO_USES_TIMERS

void bsp_gpio_initialize() {
	int _indiceName, _indiceGpio;

	/* Initialize current reference. */
	bsp_base.Board.Cur.pInfo = &(bsp_base.Board.All[bsp_base.Board.Cur.boardInfo]);

#ifdef BSP_DEBUG
	bsp_base.debug = BSP_FLAGS_DEBUG_GPIO;
#endif

	/* Initialize current gpio name access. */
	{
		/* All Gpio name. */
		for (_indiceName = 0; _indiceName < BSP_Gpio_name_max; _indiceName++) {
			/* Set to NULL */
			bsp_base.Board.Cur.pGpioName[_indiceName] = (pBsp_board_gpio_s)0;
			
			/* Check reserved Gpio */
			if (_indiceName == BSP_Gpio_name_reserved) {

				/* If bsp */
				HALBSP_DEBUG(	"NameAcces: indiceName(%u) -> Gpio reserved",
							_indiceName);
				/* Continue. */
				continue;
			}

			/* All Gpio. */
			for (_indiceGpio = 0; _indiceGpio < BSP_GPIO_MAX; _indiceGpio++) {
				if (bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].name == _indiceName) {
					/* Set */
					bsp_base.Board.Cur.pGpioName[_indiceName] = &(bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio]);

					/* If bsp */
					HALBSP_DEBUG(	"NameAcces: indiceName(%u) indiceGpio(%u) -> pGpio(%p)",
							_indiceName,
							_indiceGpio,
							bsp_base.Board.Cur.pGpioName[_indiceName]);
				}
			}
		}
	}

	bsp_gpio_arch_initialize();
#if GPIO_USES_TIMER
	bsp_gpio_timer_initialize();
#endif
}

// vim:ts=5:
