
/*
 * ============================================================================
 *
 * Copyright : (c) 2011 SAGEMCOM - Site Rueil-Malmaison - ATR-URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEMCOM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_gpio_vx185.c Bsp GPIO management for VOX185 serie (IFE7).
 *
 * This file contain all Bsp Gpio functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

#ifdef CONFIG_FUSIV_VX185

/*
 * Functions
 * ~~~~~~~~~
 */

/** Write word in register. */
static inline void bsp_gpio_write_regw(unsigned long val, unsigned long addr)
{
	/* Lock */
	spin_lock(&bsp_base.Board.Cur.gpio_lock);

	/* Set register */
	*(volatile u32 *)(addr) = val;

	/* Unlock */
	spin_unlock(&bsp_base.Board.Cur.gpio_lock);
}

/** Read Word in register. */
static inline unsigned long bsp_gpio_read_regw(unsigned long addr)
{
	unsigned long _res;

	/* Lock */
	spin_lock(&bsp_base.Board.Cur.gpio_lock);

	/* Read register */
	_res = *(volatile u32 *)(addr);

	/* Unlock */
	spin_unlock(&bsp_base.Board.Cur.gpio_lock);

	/* Out */
	return(_res);
}

/** Initialize gpio. */
void bsp_gpio_arch_initialize(void)
{
   int _indiceGpio;
   u32 _dir1Flgs, _dir2Flgs, _AlterFunctSel, _AlterFunctInit;

   /* Load current flags. */
   _dir1Flgs =		bsp_gpio_read_regw(BSP_GPIO_DIR1);
   _dir2Flgs =		bsp_gpio_read_regw(BSP_GPIO_DIR2);

   /* Load Alternate Function Config. */
   _AlterFunctSel =		bsp_gpio_read_regw(BSP_GPIO_AF_SELECT);
   _AlterFunctInit =		bsp_gpio_read_regw(BSP_GPIO_AF_INIT);
   //HALBSP_DEBUG ("init dir1 %x dir2 %x AFsel %x AFinit %x", _dir1Flgs, _dir2Flgs, _AlterFunctSel, _AlterFunctInit);

   /* If bsp */
   HALBSP_DEBUG(	"InitDir: curDir1(%x) curDir2(%u)",
	    _dir1Flgs,
	    _dir2Flgs);

   /* All Gpio. */
   for ( _indiceGpio = 0 ; _indiceGpio < BSP_GPIO_MAX ; _indiceGpio++) {
	 /* If not reserved */
	 if (bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].name != BSP_Gpio_name_reserved) {
	    /* Direction Part */
	    {
		  /* Standart Output val = bin 01*/
		  if (bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].flags & BSP_Gpio_flags_dir_out) {

			/* If bsp */
			HALBSP_DEBUG(	"InitDir: indiceGpio(%u) indiceName(%u) -> Output",
				 _indiceGpio,
				 bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].name);

			/* Set direction */
			if (_indiceGpio < 16) {
			   _dir1Flgs |= 1 << (_indiceGpio*2);
			}
			else {
			   _dir2Flgs |= 1 << ((_indiceGpio - 16)*2);
			}
		  }

		  /* Open Drain Output val = bin 10 */
		  if (bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].flags & BSP_Gpio_flags_dir_out_opendrain) {

			/* If bsp */
			HALBSP_DEBUG(	"InitDir: indiceGpio(%u) indiceName(%u) -> Output",
				 _indiceGpio,
				 bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].name);

			/* Set direction */
			if (_indiceGpio < 16) {
			   _dir1Flgs |= 2 << (_indiceGpio*2);
			}
			else {
			   _dir2Flgs |= 2 << ((_indiceGpio - 16)*2);
			}
		  }

		  /* Input val = bin 00*/
		  if (bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].flags & BSP_Gpio_flags_dir_in) {

			/* If bsp */
			HALBSP_DEBUG(	"InitDir: indiceGpio(%u) indiceName(%u) -> Input",
				 _indiceGpio,
				 bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].name);

			if (_indiceGpio < 16) {
			   _dir1Flgs &= ~(3 << (_indiceGpio*2));
			}
			else {
			   _dir2Flgs &= ~(3 << ((_indiceGpio - 16)*2));
			}
		  }

		  /* soft interrupt */
		  if (bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].flags & BSP_Gpio_flags_soft_intr) {

			/* If bsp */
			HALBSP_DEBUG(	"Soft int: indiceGpio(%u) indiceName(%u) -> soft interrut",
				 _indiceGpio,
				 bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].name);

			bsp_base.int_mask |= 1<< _indiceGpio;
		  }

	    }

	    /* altenate unction part always 0 for all supported GPIO */
	    _AlterFunctSel &= ~(1 << _indiceGpio);
	    _AlterFunctInit &= ~(1 << _indiceGpio);

	 }
   }

   if ( bsp_base.Board.Cur.boardType == BSP_Board_type_lbv3)
   {
	 // set UART2 RX/TX
	 _dir1Flgs 	 	|= (1 << (11*2)); 	// OUT
	 _AlterFunctSel  |= (1 << 11);  		//TX
	 _AlterFunctInit |= (1 << 11);  		//TX
	 _dir1Flgs 	 	|= (0 << (12*2));  	// IN
	 _AlterFunctSel  |= (1 << 12);  		//RX
	 _AlterFunctInit |= (1 << 12);  		//RX
   }

   HALBSP_DEBUG ("save dir1 %x dir2 %x AFsel %x AFinit %x", _dir1Flgs, _dir2Flgs, _AlterFunctSel, _AlterFunctInit);

   /* Save new dir mode. */
   bsp_gpio_write_regw(	_dir1Flgs,
	    BSP_GPIO_DIR1);
   bsp_gpio_write_regw(	_dir2Flgs,
	    BSP_GPIO_DIR2);

   /* Save new alternate function. */
   bsp_gpio_write_regw(	_AlterFunctSel,
	    BSP_GPIO_AF_SELECT);
   bsp_gpio_write_regw(	_AlterFunctInit,
	    BSP_GPIO_AF_INIT);


   /* 
LCe: flag changed from SA_INTERRUP to IRQF_DISABLED
see in signal.h: SA_INTERRUPT is a no-op, but left due to historical reasons.
#define SA_INTERRUPT	0x20000000	* dummy -- ignored *
    *
    * Gpio 1 irq *
    if (_nbInt1) {
    * Request Irq *
    if (request_irq(	GPIO_INT,
    bsp_gpio_irq,
    IRQF_DISABLED,
    "gpio1",
    (void *)&(bsp_base.Board.GpioIntr.low)) == 0) {
    }
    }

    * Gpio 2 irq *
    if (_nbInt2) {
    * Request Irq *
    if (request_irq(	GPIO2_INT,
    bsp_gpio_irq,
    IRQF_DISABLED,
    "gpio2",
    (void *)&(bsp_base.Board.GpioIntr.high)) == 0) {
    }
    }
    */
   /* If bsp */
   HALBSP_DEBUG(	"InitDir: newDir1(%x) newDir2(%u)",
	    _dir1Flgs,
	    _dir2Flgs);

   /* Ok */
   return;
}

unsigned long bsp_gpio_getAllValues(					void)
{
	return bsp_gpio_read_regw(BSP_GPIO_FLAG_INPUT);
}

/** Set GPIO state */
void  bsp_gpio_set_state_ex(

	bsp_gpio_name_e	_name,
	bsp_gpio_state_e	_state,

	/* Akka(S.VIE), 2011-12-12: New argument "bEnablePolarityInversion".
	 * Setting that argument to 1 will apply the reverted value of argument "_state"
	 * if the underlying GPIO was defined with flag "active_low".
	 */
	int bEnablePolarityInversion)
{
	unsigned int	_indiceGpio	= bsp_gpio_get_num(_name);

	/* Check num */
	if (_indiceGpio == -1) {
		/* Out */
		return;
	}

	{
		int bReversePolarity = 0;

		if (1 == bEnablePolarityInversion)
		{
			if ( BSP_Gpio_flags_active_low ==
			    (BSP_Gpio_flags_active_low & bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].flags))
			{
				bReversePolarity = 1;
			}
		}

		/* Set */
		switch (_state) {
			case BSP_Gpio_state_set:
			{
					/* If bsp */
						HALBSP_DEBUG(	"Set -> nameGpio(%u)indiceGpio(%u)",
								_name,
								_indiceGpio);
	
					/* Set Gpio */
					bsp_gpio_write_regw((u32)(1 << _indiceGpio), (0 == bReversePolarity) ? BSP_GPIO_FLAG_SET : BSP_GPIO_FLAG_RESET);
			}
			break;

			case BSP_Gpio_state_unset:
			{
					/* If bsp */
					HALBSP_DEBUG(	"Unset -> nameGpio(%u) indiceGpio(%u)",
							_name,
							_indiceGpio);

					/* Set Gpio */
					bsp_gpio_write_regw((u32)(1 << _indiceGpio), (0 == bReversePolarity) ? BSP_GPIO_FLAG_RESET : BSP_GPIO_FLAG_SET);
			}
			break;

			default:
			{
				HALBSP_ERROR(	"UNKNOWN -> nameGpio(%u) gpioState(%u)",
						_name,
						_state);
			}
			break;
		}
	}

	return;
}

void  bsp_gpio_set_state(	bsp_gpio_name_e	_name,
						bsp_gpio_state_e	_state)
{
	bsp_gpio_set_state_ex(_name, _state, 0 /* int bEnablePolarityInversion */);
}
EXPORT_SYMBOL(bsp_gpio_set_state);

/** Get GPIO state */
bsp_gpio_state_e bsp_gpio_get_state_ex(

	bsp_gpio_name_e _name,

	/* Akka(S.VIE), 2011-12-12: New argument "bEnablePolarityInversion".
	 * Setting that argument to 1 will make the function return a reverted value
	 * if the underlying GPIO was defined with flag "active_low".
	 */
	int bEnablePolarityInversion)
{
	unsigned char	_gpioState	= BSP_Gpio_state_failure;
	unsigned int	_indiceGpio	= bsp_gpio_get_num(_name);

	/* Check num */
	if (_indiceGpio != -1) {

		int bReversePolarity = 0;

		if (1 == bEnablePolarityInversion)
		{
			if ( BSP_Gpio_flags_active_low ==
			    (BSP_Gpio_flags_active_low & bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].flags))
			{
				bReversePolarity = 1;
			}
		}

		/* Low part */
			/* If bsp */
			//HALBSP_DEBUG(	"nameGpio(%u) indiceGpio(%u)",
			//		_name,
			//		_indiceGpio);

			/* Read */
			if (bsp_gpio_read_regw(BSP_GPIO_FLAG_INPUT) & (1 << _indiceGpio)) {
				/* Set */
				_gpioState = (0 == bReversePolarity) ? BSP_Gpio_state_set : BSP_Gpio_state_unset;
			}
			else {
				/* Unset */
				_gpioState = (0 == bReversePolarity) ? BSP_Gpio_state_unset : BSP_Gpio_state_set;
			}
	}

	/* Error */
	return(_gpioState);
} 

bsp_gpio_state_e bsp_gpio_get_state(bsp_gpio_name_e _name)
{
	return bsp_gpio_get_state_ex(_name, 0 /* int bEnablePolarityInversion */);
}
EXPORT_SYMBOL(bsp_gpio_get_state);

/** sysctl bsp_gpio_maps_proc.
 *
 *   \par Description
 *	<b>This function allow to set connexion tampon.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int bsp_gpio_maps_proc(	SYSCTL_PROC_ARGS)
{
	char	*_pTampon		= (char *)_pTable->data;
	u32	_dir1Flags, _dir2Flags, _int1Set, _int2Set, _polSet1, _polSet2, _edgeSet1, _edgeSet2, _bothSet1, _bothSet2;
	u32 tmp;

	/* Load current state. */
	_dir1Flags = bsp_gpio_read_regw(BSP_GPIO_DIR1);
	_dir2Flags = bsp_gpio_read_regw(BSP_GPIO_DIR2);

	/* Load current Int Set. */
	_int1Set = 0; //bsp_gpio_read_regw(BSP_GPIO_INT_SET1);
	_int2Set = 0; //bsp_gpio_read_regw(BSP_GPIO_INT_SET2);

	/* Load current pol Set. */
	_polSet1 = 0; //bsp_gpio_read_regw(BSP_GPIO_POL_SET1);
	_polSet2 = 0; //bsp_gpio_read_regw(BSP_GPIO_POL_SET2);

	/* Load current edge Set. */
	_edgeSet1 = 0; //bsp_gpio_read_regw(BSP_GPIO_EDGE_SET1);
	_edgeSet2 = 0; //bsp_gpio_read_regw(BSP_GPIO_EDGE_SET2);

	/* Load current edge Set. */
	_bothSet1 = 0; //bsp_gpio_read_regw(BSP_GPIO_BOTH_SET1);
	_bothSet2 = 0; //bsp_gpio_read_regw(BSP_GPIO_BOTH_SET2);


	/* Next */
	_pTampon += strlen(_pTampon);

	/* Retrieve tampon */
	if ((_pTampon = (char *)_pTable->data) != (char *)0) {
		bsp_gpio_name_e	_nameGpio;
		unsigned int		_indiceGpio;

		sprintf(	_pTampon,
				"\n\n");
		_pTampon += strlen(_pTampon);

		sprintf(	_pTampon,"dir1 0x%.8X dir2 0x%.8X\n", _dir1Flags, _dir2Flags);
		_pTampon += strlen(_pTampon);

		tmp = bsp_gpio_read_regw(BSP_GPIO_FLAG_SET);
		sprintf(	_pTampon,"set 0x%.8X ", tmp );
		_pTampon += strlen(_pTampon);

		tmp = bsp_gpio_read_regw(BSP_GPIO_FLAG_RESET);
		sprintf(	_pTampon,"reset 0x%.8X\n", tmp );
		_pTampon += strlen(_pTampon);

		tmp = bsp_gpio_read_regw(BSP_GPIO_FLAG_INPUT);
		sprintf(	_pTampon,"input val 0x%.8X\n", tmp );
		_pTampon += strlen(_pTampon);

		tmp = bsp_gpio_read_regw(BSP_GPIO_AF_SELECT);
		sprintf(	_pTampon,"Alt 0x%.8X ", tmp );
		_pTampon += strlen(_pTampon);

		tmp = bsp_gpio_read_regw(BSP_GPIO_AF_INIT);
		sprintf(	_pTampon,"init 0x%.8X\n\n", tmp );
		_pTampon += strlen(_pTampon);



		sprintf(	_pTampon,

				"| Nb Gpio |          Name          |            Config                       |                Maps                 |\n");

		/* Next */
		_pTampon += strlen(_pTampon);

		sprintf(	_pTampon,
				"|         |                        | Nb Name | Used as | Dir | Active | Intr | Dir | Intr | Pol | Edg | Both | Val |\n");

		/* Next */
		_pTampon += strlen(_pTampon);

		/* All Gpio number */
		for (_indiceGpio = 0; _indiceGpio < BSP_GPIO_MAX; _indiceGpio++) {
			/* Number Gpio */
			sprintf(	_pTampon,
					"|   %2u    |",
					_indiceGpio);

			/* Next */
			_pTampon += strlen(_pTampon);

			/* Retrive name gpio */
			_nameGpio = bsp_base.Board.Cur.pInfo->Gpio[_indiceGpio].name;

			/* Name */
			sprintf(	_pTampon,
					"%s|",
					bsp_base.Board.pGpioNameTxt[_nameGpio]);

			/* Next */
			_pTampon += strlen(_pTampon);

			/* Config */
			{
				/* If gpio does not exist */
				if (bsp_base.Board.Cur.pGpioName[_nameGpio] == (pBsp_board_gpio_s)0) {
					sprintf(	_pTampon,
							"    -    |    -    |  -  |   -    |  -   |  -  |  -   |  -  |  -  |  -   |  -  |\n");

					/* Next */
					_pTampon += strlen(_pTampon);

					/* Continue */
					continue;
				}

				/* Number Name */
				sprintf(	_pTampon,
						"    %02u   |",
						_nameGpio);

				/* Next */
				_pTampon += strlen(_pTampon);

				/* Used as part */
				{
					/* If Gpio */
					if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_used_as_gpio) {
						sprintf(	_pTampon,
								"   Gpio  |");
					}
					/* If config */
					else if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_used_as_config) {
						sprintf(	_pTampon,
								"  Config |");
					}
					/* Spi cs */
					else if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_used_as_spi_cs) {
						sprintf(	_pTampon,
								"  Spi cs |");
					}
					else {
						sprintf(	_pTampon,
								"    -    |");
					}
				}

				/* Next */
				_pTampon += strlen(_pTampon);

				/* Dir part */
				{
					/* If In */
					if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_dir_in) {
						sprintf(	_pTampon,
								" In  |");
					}
					/* If Standart out */
					else if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_dir_out) {
						sprintf(	_pTampon,
								" Out |");
					}
					/* If open drain */
					else if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_dir_out_opendrain) {
						sprintf(	_pTampon,
								" Open|");
					}
					else {
						sprintf(	_pTampon,
								"  -  |");
					}
				}

				/* Next */
				_pTampon += strlen(_pTampon);

				/* Active part */
				{
					/* If Gpio */
					if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_active_low) {
						sprintf(	_pTampon,
								"  Low   |");
					}
					/* If config */
					else if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_active_high) {
						sprintf(	_pTampon,
								"  High  |");
					}
					else {
						sprintf(	_pTampon,
								"   -    |");
					}
				}

				/* Next */
				_pTampon += strlen(_pTampon);

				/* Intr part */
				{
					/* If Gpio */
					if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_intr) {
						sprintf(	_pTampon,
								"  H   |");
					}
					else if (bsp_base.Board.Cur.pGpioName[_nameGpio]->flags & BSP_Gpio_flags_soft_intr) {
						sprintf(	_pTampon,
								"  S   |");
					}
					else
					{
						sprintf(	_pTampon,
								"  -   |");
					}
				}

				/* Next */
				_pTampon += strlen(_pTampon);
			}

			/* Maps */
			{
				int	_numGpio = bsp_gpio_get_num(_nameGpio);
				/* Dir */
				{
					if (_numGpio < 16)
					{
						switch ( (_dir1Flags & (3 << _numGpio*2))>> _numGpio*2 )
						{
						case 0: sprintf(	_pTampon, " In  |"); break;
						case 1: sprintf(	_pTampon, " Out |"); break;
						case 2: sprintf(	_pTampon, " Odr |"); break;
						case 3: sprintf(	_pTampon, " Oso |"); break;
						}
					}
					else
					{
						switch ( (_dir2Flags & (3 << (_numGpio - 16)*2))>> (_numGpio - 16)*2 )
						{
						case 0: sprintf(	_pTampon, " In  |"); break;
						case 1: sprintf(	_pTampon, " Out |"); break;
						case 2: sprintf(	_pTampon, " Odr |"); break;
						case 3: sprintf(	_pTampon, " Oso |"); break;
						}
					}
				}
				/* Next */
				_pTampon += strlen(_pTampon);

				/* Intr */
				{
					/* Set direction */
					if (_numGpio < 16) {
						if (_int1Set & (1 << _numGpio)) {
							sprintf(	_pTampon,
									"  1   |");
						}
						else {
							sprintf(	_pTampon,
									"  0   |");
						}
					}
					else {
						if (_int2Set & (1 << (_numGpio - 16))) {
							sprintf(	_pTampon,
									"  1   |");
						}
						else {
							sprintf(	_pTampon,
									"  0   |");
						}
					}
				}

				/* Next */
				_pTampon += strlen(_pTampon);

				/* Polarity part */
				{
					/* Set direction */
					if (_numGpio < 16) {
						/* If dir out */
						if (_dir1Flags & (1 << _numGpio)) {
							sprintf(	_pTampon,
									"  -  |");
						}
						else {
							if (_polSet1 & (1 << _numGpio)) {
								sprintf(	_pTampon,
										"  1  |");
							}
							else {
								sprintf(	_pTampon,
										"  0  |");
							}
						}
					}
					else {
						/* If dir out */
						if (_dir2Flags & (1 << (_numGpio - 16))) {
							sprintf(	_pTampon,
									"  -  |");
						}
						else {
							if (_polSet2 & (1 << (_numGpio - 16))) {
								sprintf(	_pTampon,
										"  1  |");
							}
							else {
								sprintf(	_pTampon,
										"  0  |");
							}
						}
					}
				}

				/* Next */
				_pTampon += strlen(_pTampon);

				/* Edge part */
				{
					/* Set direction */
					if (_numGpio < 16) {
						/* If dir out */
						if (_dir1Flags & (1 << _numGpio)) {
							sprintf(	_pTampon,
									"  -  |");
						}
						else {
							if (_edgeSet1 & (1 << _numGpio)) {
								sprintf(	_pTampon,
										"  1  |");
							}
							else {
								sprintf(	_pTampon,
										"  0  |");
							}
						}
					}
					else {
						/* If dir out */
						if (_dir2Flags & (1 << (_numGpio - 16))) {
							sprintf(	_pTampon,
									"  -  |");
						}
						else {
							if (_edgeSet2 & (1 << (_numGpio - 16))) {
								sprintf(	_pTampon,
										"  1  |");
							}
							else {
								sprintf(	_pTampon,
										"  0  |");
							}
						}
					}
				}

				/* Next */
				_pTampon += strlen(_pTampon);

				/* Both part */
				{
					/* Set direction */
					if (_numGpio < 16) {
						if (_bothSet1 & (1 << _numGpio)) {
							sprintf(	_pTampon,
									"  1   |");
						}
						else {
							sprintf(	_pTampon,
									"  0   |");
						}
					}
					else {
						if (_bothSet2 & (1 << (_numGpio - 16))) {
							sprintf(	_pTampon,
									"  1   |");
						}
						else {
							sprintf(	_pTampon,
									"  0   |");
						}
					}
				}

				/* Next */
				_pTampon += strlen(_pTampon);

				/* Val */
				{
					/* Get Value */
					switch (bsp_gpio_get_state(_nameGpio)) {
						/* Set */
						case BSP_Gpio_state_set:
						{
							sprintf(	_pTampon,
									"  1  |\n");
						}
						break;

						/* Unset */
						case BSP_Gpio_state_unset:
						{
							sprintf(	_pTampon,
									"  0  |\n");
						}
						break;

						/* Else */
						default:
						{
							sprintf(	_pTampon,
									"  -  |\n");
						}
						break;
					}
				}

				/* Next */
				_pTampon += strlen(_pTampon);
			}
		}
	}
	else {
		/* Print */
		sprintf(	_pTampon,
				"Process Error\n");

		/* Next */
		_pTampon += strlen(_pTampon);
	}

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Call proc_dostring */
	return(SYSCTL_PROC_DOSTRING(	_pTable,
						_write,
						_pFile,
						_pBuffer,
						_pLen,
						_pPpos));
}

#endif

// vim:ts=5:
