/* ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_led.c Bsp Led functions
 *
 * This file contain all Bsp Led functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

/*
 * Functions
 * ~~~~~~~~~
 */

/** Set Led state. */
int kSysLedCtrl(	BOARD_LED_NAME 	_nameUser,
				BOARD_LED_STATE	_stateUser)
{
	/* Check Led name*/
	if ( (_nameUser < 0) || (_nameUser >= kLedEnd) ){
		HALBSP_ERROR(	"WARNING -> Led name >= Max(%u)",
				kLedEnd);

		return(-1);
	}

	/* Check  Led state */
	if ( (_stateUser < 0) || (_stateUser >= kLedStateOn + 1) ){
		HALBSP_ERROR(	"WARNING -> Led state >= Max(%u)",
				kLedStateOn + 1);

		return(-1);
	}

	{
		bsp_gpio_name_e	_nameGpio;
		bsp_gpio_state_e	_stateGpio;

		/* Retrieve nameGpio */
		_nameGpio		= bsp_base.Index.Led.Name.user2gpio[_nameUser];

		/* Retrieve stateGpio */
		_stateGpio	= bsp_base.Index.Led.State.user2gpio[_stateUser];

		/* Set state */
		bsp_gpio_set_state_ex(_nameGpio, _stateGpio, 1 /* bEnablePolarityInversion */);

		/* If bsp */
		HALBSP_DEBUG(	"Led -> nameUser(%u) nameGpio(%u)(%s) stateUser(%u) stateGpio(%u)",
				_nameUser,
				_nameGpio,
				bsp_base.Board.pGpioNameTxt[_nameGpio],
				_stateUser,
				_stateGpio);
	}

	/* Ok */
	return(0);
}
EXPORT_SYMBOL(kSysLedCtrl);

/** Kernel read led state function */
BOARD_LED_STATE kSysGetLedState(BOARD_LED_NAME _nameUser)
{
	BOARD_LED_STATE	_stateUser;

	/* Check name. */
	if ( (_nameUser < 0) || (_nameUser >= kLedEnd) ) {
		HALBSP_ERROR(	"WARNING -> Led >= Max(%u)",
				kLedEnd);

		return(-1);
	}

	/* Retrieve state */
	{
		bsp_gpio_state_e	_stateGpio;
		bsp_gpio_name_e	_nameGpio;

		/* Retrieve nameGpio */
		_nameGpio = bsp_base.Index.Led.Name.user2gpio[_nameUser];

		/* Retrieve stateGpio */
		if (BSP_Gpio_state_failure == (_stateGpio = bsp_gpio_get_state_ex(_nameGpio, 1 /* bEnablePolarityInversion */))) {
			/* Error */
			return(-1);
		}

		/* Retrieve userState */
		_stateUser = bsp_base.Index.Led.State.gpio2user[_stateGpio];

		/* If bsp */
		HALBSP_DEBUG(	"Led -> nameUser(%u) nameGpio(%u) stateUser(%u) stateGpio(%u)",
				_nameUser,
				_nameGpio,
				_stateUser,
				_stateGpio);
	}

	return(_stateUser);
}
EXPORT_SYMBOL(kSysGetLedState);

/** sysctl bsp_led_maps_proc.
 *
 *   \par Description
 *	<b>This function allow to set connexion tampon.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int bsp_led_maps_proc(	SYSCTL_PROC_ARGS)
{
	char				*_pTampon		= (char *)_pTable->data;
	BOARD_LED_STATE	_userState;
	bsp_gpio_name_e	_nameGpio;
	BOARD_LED_NAME  _nameUser;

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Retrieve tampon */
	if ((_pTampon = (char *)_pTable->data) != (char *)0) {

/*		sprintf(	_pTampon,
				"| Power | @ red | @ green | Voip | lan | Wireless | Upgrade |\n");

		* Next *
		_pTampon += strlen(_pTampon);
*/
		for (_nameUser = kLedPower ; _nameUser < kLedEnd ; _nameUser++)
			{
			/* Retrieve nameGpio */
			_nameGpio = bsp_base.Index.Led.Name.user2gpio[_nameUser];

			/* Name */
			sprintf(	_pTampon,
					"%s ",
					bsp_base.Board.pGpioNameTxt[_nameGpio]);

			/* Next */
			_pTampon += strlen(_pTampon);

			/* Retrieve led state */
			_userState = kSysGetLedState(_nameUser);

			switch (_userState) 
				{
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							" ON\n");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							" OFF\n");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							" -\n");
				}
				break;
				}

			/* Next */
			_pTampon += strlen(_pTampon);


			}

		/* Call proc_dostring */
		return(SYSCTL_PROC_DOSTRING(	_pTable,
						_write,
						_pFile,
						_pBuffer,
						_pLen,
						_pPpos));



		/* Adsl part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedPower);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   1   |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   0   |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   -   |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet red part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedInternet_SynchroLine);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"   1   |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"   0   |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"   -   |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet green part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedInternet_DataConnectivity);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1    |");
				}
				break;
	
				{
				case kLedStateOff:
					/* Print */
					sprintf(	_pTampon,
							"    0    |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Wireless part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedVoip1);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"  1   |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"  0   |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"  -   |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Power part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedLan);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"  1  |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"  0  |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"  -  |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Lan part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedWireless);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1     |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"    0     |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -     |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Upgrade part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedUpgrade);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1    |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"    0    |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);
	}
	else {
		/* Print */
		sprintf(	_pTampon,
				"Process Error\n");

		/* Next */
		_pTampon += strlen(_pTampon);
	}

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Call proc_dostring */
	return(SYSCTL_PROC_DOSTRING(	_pTable,
						_write,
						_pFile,
						_pBuffer,
						_pLen,
						_pPpos));
}

#if (defined HALBSP_PERGPIO_PROCENTRY)

/* FIXME: Every call to "printk" should be replaced by a call to "HALBSP_DEBUG".
 */
 
int bsp_led_write_proc_state(

  struct file *filp,
  const char *buffer,
  unsigned long n,
  void *data)
{
  int retVal = 0;

  int brightness;

  bsp_proc_name_e * pProcname = (bsp_proc_name_e *)data;
  BOARD_LED_STATE _stateUser;

  enum _BOARD_LED_NAME _nameLed;

//printk("%s: pProcname=%p\n", __FUNCTION__, pProcname);

  if (NULL == pProcname)
  {
    goto cleanup;
  }

  brightness = simple_strtoul(buffer, NULL, 10);

  if (255 < brightness) brightness = 255;

//printk("%s: n=%d buffer=\"%s\"\n", __FUNCTION__, n, buffer);

  switch (brightness)
  {
    case 0:
      _stateUser = kLedStateOff;
      break;

    default:
      _stateUser = kLedStateOn;
      break;
  }

  for (_nameLed = kLedPower; _nameLed < kLedEnd; _nameLed++)
  {
//  printk("%s: _nameLed=%d mismatches\n", __FUNCTION__, _nameLed);

    if (pProcname != &bsp_base.Index.Led.Name.user2proc[_nameLed])
    {
      continue;
    }

//  printk("%s: _nameLed=%d MATCHES\n", __FUNCTION__, _nameLed);

    kSysLedCtrl(_nameLed, _stateUser);

    /* Success.
     */
    retVal = n;
    goto cleanup;
  }

cleanup:

//printk("%s: retVal=%d\n", __FUNCTION__, retVal);

  return retVal;
}  

static int bsp_led_read_proc_state(

  char *buffer,
  char **debut,
  off_t offset,
  int maxi,
  int *eof,
  void *data)
{
  /* FIXME: Should check whether the buffer is large enough.
   */

  int retVal = 0;

  bsp_proc_name_e * pProcname = (bsp_proc_name_e *)data;

  enum _BOARD_LED_NAME _nameLed;

  if (NULL == pProcname) goto cleanup;

  for (_nameLed = kLedPower; _nameLed < kLedEnd; _nameLed++)
  {
    if (pProcname != &bsp_base.Index.Led.Name.user2proc[_nameLed])
    {
      continue;
    }

    switch (kSysGetLedState(_nameLed))
    {
      case kLedStateOn:
        strcpy(buffer,"255");
        break;

      case kLedStateOff:
        strcpy(buffer,"0");
        break;

      default:
        goto cleanup;
        break;
    }

    /* Success.
     */
    retVal = strlen(buffer);
    goto cleanup;
  }

cleanup:

//printk("%s: retVal=%d\n", __FUNCTION__, retVal);

  return retVal;
}

int bsp_led_proc_initialize(void)
{
  int entries = 0;

  enum _BOARD_LED_NAME _nameLed;
  enum bsp_gpio_name_enu _nameGpio;

  /* Create a "proc entry" for every Led that is bound to both a "proc" name and a "GPIO".
   */
  for (_nameLed = kLedPower; _nameLed < kLedEnd; _nameLed++)
  {
    bsp_proc_name_e * pProcname = &bsp_base.Index.Led.Name.user2proc[_nameLed];

    struct proc_dir_entry * pProcEntry;

    pBsp_board_gpio_s pGpio;

    if (    (BSP_Proc_name_reserved == *pProcname)
         || (BSP_Proc_name_max <= *pProcname)
       )
    {
      /* No "proc name" ID for that resource.
       */
      continue;
    }

    if (NULL == bsp_base.Board.pGpioNameTxt[*pProcname])
    {
      /* No "proc name" text for that resource.
       */
      continue;
    }

    _nameGpio = bsp_base.Index.Led.Name.user2gpio[_nameLed];

    if (BSP_Gpio_name_reserved == _nameGpio)
    {
      /* No "BSP_Gpio_name_led_" is bound to that resource.
       */
      continue;
    }

    if (NULL == (pGpio = bsp_base.Board.Cur.pGpioName[_nameGpio]))
    {
      /* No physical GPIO is bound to that "BSP_Gpio_name_led_".
       */
      continue;
    }

    if (NULL == (pProcEntry = create_proc_entry(

      bsp_base.Board.pProcNameTxt[*pProcname],
      0644,
      NULL)))
    {
      continue;
    }
    
    pProcEntry->nlink = 1;
    pProcEntry->data = pProcname;
    pProcEntry->read_proc = bsp_led_read_proc_state;
    pProcEntry->write_proc = bsp_led_write_proc_state;

    pGpio->pProcEntry = pProcEntry;

    entries++;
  }

#if 0 /* For debug only */

  for (_nameGpio = BSP_Gpio_name_reserved; _nameGpio < BSP_Gpio_name_max; _nameGpio++)
  {
    pBsp_board_gpio_s pGpio = &bsp_base.Board.Cur.pGpioName[_nameGpio];

    printk("%s: pGpioName[%d].pProcEntry=%p\n", __FUNCTION__, _nameGpio, pGpio->pProcEntry);

    if (NULL == pGpio->pProcEntry)
    {
      continue;
    }
  }

#endif /* For debug only */

//printk("%s: Created %d proc entries\n", __FUNCTION__, entries);

  return entries;
}

void bsp_led_proc_release(void)
{
  enum bsp_gpio_name_enu _nameGpio;

  for (_nameGpio = BSP_Gpio_name_reserved; _nameGpio < BSP_Gpio_name_max; _nameGpio++)
  {
    pBsp_board_gpio_s pGpio = &bsp_base.Board.Cur.pGpioName[_nameGpio];

//  printk("%s: pGpioName[%d].pProcEntry=%p\n", __FUNCTION__, _nameGpio, pGpio->pProcEntry);

    if (NULL == pGpio->pProcEntry)
    {
      continue;
    }

    remove_proc_entry(pGpio->pProcEntry, NULL);

    pGpio->pProcEntry = NULL;
  }
}

#endif /* HALBSP_PERGPIO_PROCENTRY */

// vim:ts=5:
