/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_permParam.c Bsp Permanent Parameter format description.
 *
 * This file allow to describe Bsp Permanent Parameter format.
*/

/*
 * Includes
 * ~~~~~~~~
 */
#include "bsp_pri.h"

/*
 * Functions
 * ~~~~~~~~~
 */

/** Get Perm Param All. */
void *bsp_get_permParam_all(						pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam);
}

/** Get Perm Param version. */
void *bsp_get_permParam_version(					pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->pp_version);
}

/** Get Perm Param serial number. */
char *bsp_get_permParam_serialNumber(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->serial_number);
}

/** Get Perm Param Sagem Product Id. */
char *bsp_get_permParam_sagemProductId(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->s_product_id);
}

/** Get Perm Param Sagem Hardware Version. */
char *bsp_get_permParam_sagemHardwareVersion(		pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->s_hardware_version);
}

/** Get Perm Param Sagem Hardware Revision. */
char *bsp_get_permParam_sagemHardwareRevision(		pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->s_hardware_revision);
}

/** Get Perm Param Customer Product Id. */
char *bsp_get_permParam_customerProductId(			pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->c_product_id);
}

/** Get Perm Param Customer Hardware Version. */
char *bsp_get_permParam_customerHardwareVersion(		pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->c_hardware_version);
}

/** Get Perm Param Customer Hardware Revision. */
char *bsp_get_permParam_customerHardwareRevision(		pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->c_hardware_revision);
}

/** Get Perm Param Ram Size. */
__u32 *bsp_get_permParam_ramSize(					pBsp_permParam_s	_pPermParam)
{
	return(&(_pPermParam->ram_size));
}

/** Get Perm Param Flash Size. */
__u32 *bsp_get_permParam_flashSize(				pBsp_permParam_s	_pPermParam)
{
	return(&(_pPermParam->flash_size));
}

/** Get Perm Param Firmware Version. */
char *bsp_get_permParam_firmVersion(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->fmw_version);
}

/** Get Perm Param Lan Base Mac Addr. */
__u8 *bsp_get_permParam_lanBaseMacAddr(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->lan_base_mac_address);
}

/** Get Perm Param Lan Mac Addr Range. */
__u16 *bsp_get_permParam_lanMacAddrRange(			pBsp_permParam_s	_pPermParam)
{
	return(&(_pPermParam->lan_mac_address_range));
}

/** Get Perm Param Wan Base Mac Addr. */
__u8 *bsp_get_permParam_wanBaseMacAddr(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->wan_base_mac_address);
}

/** Get Perm Param Wan Mac Addr Range. */
__u16 *bsp_get_permParam_wanMacAddrRange(			pBsp_permParam_s	_pPermParam)
{
	return(&(_pPermParam->wan_mac_address_range));
}

/** Get Perm Param Usb Mac Addr. */
__u8 *bsp_get_permParam_usbMacAddr(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->usb_mac);
}

/** Get Perm Param Wifi Mac1 Addr. */
__u8 *bsp_get_permParam_wifiMac1Addr(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->wifi_mac1);
}

/** Get Perm Param Wifi ssid1. */
char *bsp_get_permParam_wifiSsid1(					pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->wifi_ssid1);
}

/** Get Perm Param Wifi Mac2 Addr. */
__u8 *bsp_get_permParam_wifiMac2Addr(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->wifi_mac2);
}

/** Get Perm Param Wifi ssid2. */
char *bsp_get_permParam_wifiSsid2(					pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->wifi_ssid2);
}

/** Get Perm Param Manufacturer. */
char *bsp_get_permParam_manufacturer(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->manufacturer);
}

/** Get Perm Param Manufacturer Base Mac Addr. */
__u8 *bsp_get_permParam_manufacturerBaseMacAddr(		pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->manufacturer_oui);
}

/** Get Perm Param Manufacturer Url. */
char *bsp_get_permParam_manufacturerUrl(			pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->manufacturer_url);
}

/** Get Perm Param Commercialization Country. */
char *bsp_get_permParam_comCountry(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->country_code);
}

/** Get Perm Param Wifi Key1. */
char *bsp_get_permParam_wifiKey1(					pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->wifi_key1);
}

/** Get Perm Param Wifi Key2. */
char *bsp_get_permParam_wifiKey2(					pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->wifi_key2);
}

/** Get Perm Param H235 Key. */
char *bsp_get_permParam_h235Key(					pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->h235_key);
}

/** Get Perm Param Public Dsa Syst. */
__u8 *bsp_get_permParam_publicDsaSyst(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->public_dsa_syst);
}

/** Get Perm Param Public Dsa Resc. */
__u8 *bsp_get_permParam_publicDsaResc(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->public_dsa_resc);
}

/** Get Perm Param Private Dsa Key. */
__u8 *bsp_get_permParam_privateDsaKey(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->private_key);
}

/** Get Perm Param Certificate. */
__u8 *bsp_get_permParam_certificate(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->certificate);
}

/** Get Perm Param Certificate. */
__u32 *bsp_get_permParam_dsa(						pBsp_permParam_s	_pPermParam)
{
	return(&(_pPermParam->dsa));
}

/** Get Perm Param Vendor Info1. */
char *bsp_get_permParam_vendorInfo1(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info1);
}

/** Get Perm Param Vendor Info2. */
char *bsp_get_permParam_vendorInfo2(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info2);
}

/** Get Perm Param Vendor Info3. */
char *bsp_get_permParam_vendorInfo3(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info3);
}

/** Get Perm Param Vendor Info4. */
char *bsp_get_permParam_vendorInfo4(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info4);
}

/** Get Perm Param Vendor Info5. */
char *bsp_get_permParam_vendorInfo5(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info5);
}

/** Get Perm Param Vendor Info6. */
char *bsp_get_permParam_vendorInfo6(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info6);
}

/** Get Perm Param Vendor Info7. */
char *bsp_get_permParam_vendorInfo7(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info7);
}

/** Get Perm Param Vendor Info8. */
char *bsp_get_permParam_vendorInfo8(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info8);
}

/** Get Perm Param Vendor Info9. */
char *bsp_get_permParam_vendorInfo9(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info9);
}

/** Get Perm Param Vendor Info10. */
char *bsp_get_permParam_vendorInfo10(				pBsp_permParam_s	_pPermParam)
{
	return(_pPermParam->vendor_info10);
}

/** Get PermParam. */
void *bsp_get_permParam(	enum bsp_get_permParam_enu	_get)
{
   return(bsp_base.Pp.getPpCb[_get](bsp_base.Pp.pFlashAddr));
}
EXPORT_SYMBOL(bsp_get_permParam);

// vim:ts=5:
