/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_permParam.c Bsp Permanent Parameter format description.
 *
 * This file allow to describe Bsp Permanent Parameter format.
*/

#define BSP_NO_ACCESS_TO_FILESYSTEM

/*
 * Includes
 * ~~~~~~~~
 */
#include "bsp_pri.h"
#if (defined BSP_NO_ACCESS_TO_FILESYSTEM)
#include <linux/mtd/ubi.h>
#endif /* BSP_NO_ACCESS_TO_FILESYSTEM */
/*
 * Functions
 * ~~~~~~~~~
 */

#define PERMANENT_VOLNAME "permanent_param"


/** Init PermParam. */
int bsp_init_permParam(void)
{
    /* By default, the permanent parameters are not available */
    int ppOK = -1;
    bsp_permParam_s *permParams;
#if (!defined BSP_NO_ACCESS_TO_FILESYSTEM)
    char *tmpString;
    int i, nbVolume;
#else /* (defined BSP_NO_ACCESS_TO_FILESYSTEM) */
    struct ubi_volume_desc *ppUbiDesc = NULL;
#endif /* BSP_NO_ACCESS_TO_FILESYSTEM */
    HALBSP_DEBUG("enter");
    permParams = (bsp_permParam_s*) kmalloc (sizeof(bsp_permParam_s), GFP_KERNEL);
    if (!permParams) {
	   panic("can't allocate memory to store permanent parameters");
    }
    memset(permParams, 0, sizeof(bsp_permParam_s));

#if (!defined BSP_NO_ACCESS_TO_FILESYSTEM)

    /* We need to retrieve the good mtd number.
	  We are going to use the permparam structure to temporarily store the content of the /proc/mtd file
	  This structure's size is large enough. Thus we can avoid an uneccessary alloc */
    tmpString = (char*) permParams;

    if(!bsp_read_file("/sys/class/ubi/ubi0/volumes_count", tmpString)) {
	   sscanf (tmpString, "%d", &nbVolume); 

	   HALBSP_DEBUG("analyse %d volumes", nbVolume);
	   i = 0;
	   while (i < nbVolume) {
		  sprintf(tmpString, "/sys/class/ubi/ubi0_%d/name", i);

		  HALBSP_DEBUG("analyse file (%s)", tmpString);

		  if(!bsp_read_file(tmpString, tmpString)) {
			 if (!strncmp(tmpString, PERMANENT_VOLNAME, sizeof(PERMANENT_VOLNAME) - 1)) {
				// Whe have found the ubi volume that contains the permanent parameters
				sprintf(tmpString, "/dev/ubi0_%d", i);
				if(!bsp_read_file(tmpString, permParams)) {

#else /* (defined BSP_NO_ACCESS_TO_FILESYSTEM) */

				    ppUbiDesc = ubi_open_volume_nm(0, PERMANENT_VOLNAME, UBI_READONLY);
				    if( 0 > ppUbiDesc ){
					   panic("Unable to open %s\n", PERMANENT_VOLNAME); 
				    }

				    if( 0 != ubi_leb_read(ppUbiDesc, 0, permParams, 0, sizeof(bsp_permParam_s), 0) ){
					   ubi_close_volume(ppUbiDesc);
					   panic("Unable to read %s\n", PERMANENT_VOLNAME);
				    }

				    ubi_close_volume(ppUbiDesc);

#endif /* BSP_NO_ACCESS_TO_FILESYSTEM */

				    /* Read params and check validity (Magic) */
				    if (permParams->magic != BSP_MAGIC_PERMPARAM) {
					   HALBSP_ERROR("Bad Magic(%x-%x) in Permanent Parameter(%p)",
							 permParams->magic,
							 BSP_MAGIC_PERMPARAM,
							 permParams);
					   kfree(permParams);
				    }
				    else {
					   bsp_base.Pp.pFlashAddr = permParams;
					   ppOK = 0;
					   HALBSP_DEBUG ("Permanent parameters found");
					   //
					   // Avoid buffer overflow
					   // Usefull if the PP are wrong !
					   //
					   permParams->pp_version[31] = '\0';
					   permParams->serial_number[63] = '\0';
					   permParams->s_product_id[63] = '\0';
					   permParams->s_hardware_version[63] = '\0';
					   permParams->s_hardware_revision[63] = '\0';
					   permParams->c_product_id[63] = '\0';
					   permParams->c_hardware_version[63] = '\0';
					   permParams->c_hardware_revision[63] = '\0';
					   permParams->fmw_version[31] = '\0';
					   permParams->wifi_ssid1[31] = '\0';
					   permParams->wifi_ssid2[31] = '\0';
					   permParams->manufacturer[63] = '\0';
					   permParams->manufacturer_url[127] = '\0';
					   permParams->country_code[31] = '\0';
					   permParams->wifi_key1[63] = '\0';
					   permParams->wifi_key2[63] = '\0';
					   permParams->h235_key[63] = '\0';
					   permParams->vendor_info1[63] = '\0';
					   permParams->vendor_info2[63] = '\0';
					   permParams->vendor_info3[63] = '\0';
					   permParams->vendor_info4[63] = '\0';
					   permParams->vendor_info5[63] = '\0';
					   permParams->vendor_info6[63] = '\0';
					   permParams->vendor_info7[63] = '\0';
					   permParams->vendor_info8[63] = '\0';
					   permParams->vendor_info9[63] = '\0';
					   permParams->vendor_info10[63] = '\0';
				    }

#if (!defined BSP_NO_ACCESS_TO_FILESYSTEM)

				}
				break;
			 }
		  }
		  i++;
	   }
    }

#endif /* BSP_NO_ACCESS_TO_FILESYSTEM */

    return ppOK;
}

// vim:ts=5:
