/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp.c Bsp functions
 *
 * This file contain all Bsp functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */
#include "bsp_pri.h"


/** Module info. */
MODULE_AUTHOR("Sagem");
/** Module License. */
MODULE_LICENSE("GPL");
//MODULE_LICENSE("Sagem SA");
/********************************************************************
erreur de compilation
FATAL: modpost: GPL-incompatible module gpio_lkm.ko uses GPL-only symbol 'device_destroy'
*********************************************************************/


/** Module description. */
MODULE_DESCRIPTION("Bsp Sagem Module");

/*
 * Functions
 * ~~~~~~~~~
 */

/** Init module. */
static int __init bsp_init(void)
{
	/* By default, the permanent parameters are available and OK */
	int ppOK = 1;

	HALBSP_INFO("Load ...");

	/* Env */
	{
		/* Boot part */
		{
			/* Init Semaphore */
			init_rwsem(&(bsp_base.Env.Boot.Recovery.sem));

			/* Recovery force */
			bsp_base.Env.Boot.Recovery.force	= BSP_Boot_recovery_force_normal;
		}
	}

	/* Board part */
	{
		/* Gpio lock */
		spin_lock_init(&bsp_base.Board.Cur.gpio_lock);
	}

	/* Permanent Parameters */
	{
		/* Check Magic */
		if (bsp_init_permParam() != 0) {
			ppOK = 0;
		}

		/* Check version */
		if (ppOK == 1)
		{
			char *_pVersion;
			char	*_pVersion_0	= "0.";
			char	*_pVersion_1	= "1.";
	
			/* Retrieve version */
			_pVersion = (char *)bsp_get_permParam(BSP_Get_permParam_version);
	
			/* Check 0.x version */
			if (memcmp(	_pVersion,
						_pVersion_0,
						strlen(_pVersion_0)) == 0) {
				/* Set Flag */
				bsp_base.flags |= BSP_FLAGS_BASE_PP_VERSION_0;

				/* Panic */
				panic("Bad type Perm Param version 0.x");
			}
			else {
				/* Check 1.x version */
				if (memcmp(	_pVersion,
							_pVersion_1,
							strlen(_pVersion_1)) == 0) {
					/* Set Flag */
					bsp_base.flags |= BSP_FLAGS_BASE_PP_VERSION_1;
				}
				else {
					/* Panic */
					panic("Bad type Perm Param version ");
				}
			}
		}

		/* Retrieve typeBoard */
		if (ppOK == 1)
		{
			unsigned int	_pp2board	= 0;
			unsigned int	_match	= 0;

			HALBSP_DEBUG ("Search board description (%s, %s, %s) in known PP",
				(char *)bsp_get_permParam(   BSP_Get_permParam_sagem_product_id),
				(char *)bsp_get_permParam(   BSP_Get_permParam_sagem_hardware_version),
				(char *)bsp_get_permParam(   BSP_Get_permParam_sagem_hardware_revision));

			/* All board */
			for (_pp2board = 0; _pp2board < BSP_Pp2board_max; _pp2board++) {
				/* Reset */
				_match = 0;

				/* If sagem product id */
				if (bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_product_id] != (char *)0) {
					char *product_id = bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_product_id];
					/* If doesn't Match */
					if (strncmp(	(char *)bsp_get_permParam(   BSP_Get_permParam_sagem_product_id),
								product_id,
								strlen(product_id)) != 0) {
						/* Continue */
						continue;
					}
				}

				/* If sagem hardware version */
				if (bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_hardware_version] != (char *)0) {
					/* If doesn't Match */
					if (strcmp(	((char *)bsp_get_permParam(	BSP_Get_permParam_sagem_hardware_version)),
								bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_hardware_version]) != 0) {
						/* Continue */
						continue;
					}
				}

				/* If sagem hardware revision */
				if (bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_hardware_revision] != (char *)0) {
					/* If doesn't Match */
					if (strcmp(	((char *)bsp_get_permParam(	BSP_Get_permParam_sagem_hardware_revision)),
								bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_hardware_revision]) != 0) {
						/* Continue */
						continue;
					}
					else {
						/* Found a match */
						_match = 1;
					}
				}

				/* If match */
				if (_match == 1) {
					/* Set Name2board*/
					bsp_base.Board.Cur.pp2board	= _pp2board;

					/* Set Board Info */
					bsp_base.Board.Cur.boardInfo	= bsp_base.Pp.pp2board[_pp2board].boardInfo;
	
					/* Set Board Type */
					bsp_base.Board.Cur.boardType	= bsp_base.Pp.pp2board[_pp2board].boardType;

					/* Set VOX Type */
					bsp_base.Board.Cur.voxType	= bsp_base.Pp.pp2board[_pp2board].voxType;

					/* Out */
					break;
				}
			}
		}

		/* Check */
		if (bsp_base.Board.Cur.pp2board == BSP_Pp2board_unknown) {
#if defined(DEFAULT_BOARD_LBV2)
#warning "Using lbv2 configuration by default - Feature should be used only for developpers !!!"
			/* Set to lbv2 by default */
			{
				bsp_base.Board.Cur.pp2board		= BSP_Pp2board_lb2;
				bsp_base.Board.Cur.boardInfo		= BSP_Board_info_lb2;
				bsp_base.Board.Cur.boardType		= BSP_Board_type_lb2;
			}
#elif defined(DEFAULT_BOARD_LBPROV3)
#warning "Using lbprov3 configuration by default - Feature should be used only for developpers !!!"
			/* Set to lbprov3 by default */
			{

// Modif LCe LBPROV3 devient LBPROV3 version 2
				bsp_base.Board.Cur.pp2board		= BSP_Pp2board_lbv3_v2;
				bsp_base.Board.Cur.boardInfo		= BSP_Board_info_lbv3_v2;
				bsp_base.Board.Cur.boardType		= BSP_Board_type_lbv3_v2;
			}
#elif defined(DEFAULT_BOARD_LB28_V0)                                                                                                                                                  
#warning "Using lb2.8 v0 configuration by default - Feature should be used only for developpers !!!"                                                                                   
			/* Set to lb2.8 v0 by default */                                                                                                                               
			{                                                                                                                                                           
				bsp_base.Board.Cur.pp2board = BSP_Pp2board_lbv28_v0;                                                                                      
				bsp_base.Board.Cur.boardInfo = BSP_Board_info_lbv28_v0;                                                                                    
				bsp_base.Board.Cur.boardType = BSP_Board_type_lbv28_v0;                                                                                    
			}
#elif defined(DEFAULT_BOARD_LB28_V1)                                                                                                                                                  
#warning "Using lb2.8 v1 configuration by default - Feature should be used only for developpers !!!"                                                                                   
			/* Set to lb2.8 v1 by default */                                                                                                                               
			{                                                                                                                                                           
				bsp_base.Board.Cur.pp2board = BSP_Pp2board_lbv28_v1;                                                                                      
				bsp_base.Board.Cur.boardInfo = BSP_Board_info_lbv28_v1;                                                                                    
				bsp_base.Board.Cur.boardType = BSP_Board_type_lbv28_v1;                                                                                    
			}
#elif defined(DEFAULT_BOARD_3764)
#warning "Using 3764 configuration by default - Feature should be used only for developpers !!!"
			/* Set to 3764 by default */
			{
				bsp_base.Board.Cur.pp2board		= BSP_Pp2board_3764;
				bsp_base.Board.Cur.boardInfo		= BSP_Board_info_3764;
				bsp_base.Board.Cur.boardType		= BSP_Board_type_3764;
			}
#elif defined(DEFAULT_BOARD_TDC180)
#warning "Using TDC v180 configuration by default - Feature should be used only for developpers !!!"
			/* Set to TDC180 by default */
			{
				bsp_base.Board.Cur.pp2board		= BSP_Pp2board_TDC_180;
				bsp_base.Board.Cur.boardInfo		= BSP_Board_info_TDC_180;
				bsp_base.Board.Cur.boardType		= BSP_Board_type_TDC_180;
			}
#elif defined(DEFAULT_BOARD_BGC_BBOX2)
#warning "Using BGC Bbox2 configuration by default - Feature should be used only for developpers !!!"
			/* Set to BGC Bbox2 by default */
			{
				bsp_base.Board.Cur.pp2board		= BSP_Pp2board_Bbox;
				bsp_base.Board.Cur.boardInfo		= BSP_Board_info_Bbox;
				bsp_base.Board.Cur.boardType		= BSP_Board_type_Bbox;
			}
#elif defined(DEFAULT_BOARD_TDC183)
#warning "Using TDC HGW3 vx183 configuration by default - Feature should be used only for developpers !!!"
			/* Set to TDC HGW3 vx183 by default */
			{
				bsp_base.Board.Cur.pp2board		= BSP_Pp2board_TDC_183;
				bsp_base.Board.Cur.boardInfo		= BSP_Board_info_TDC_183;
				bsp_base.Board.Cur.boardType		= BSP_Board_type_TDC_183;
			}
#elif defined(DEFAULT_BOARD_X86)
#warning "X86 default to lbprov3 - This Feature should be used only for developpers !!!"
			/* Set to lbprov3 by default */
			{
				bsp_base.Board.Cur.pp2board		= BSP_Pp2board_lbv3;
				bsp_base.Board.Cur.boardInfo		= BSP_Board_info_lbv3;
				bsp_base.Board.Cur.boardType		= BSP_Board_type_lbv3;
			}
#else
			/* Set to unknwon */
			{
				bsp_base.Board.Cur.pp2board		= BSP_Pp2board_unknown;
				bsp_base.Board.Cur.boardInfo		= BSP_Board_info_unknown;
				bsp_base.Board.Cur.boardType		= BSP_Board_type_unknown;
			}
#endif

			if (bsp_base.Board.Cur.pp2board == BSP_Pp2board_unknown) {
				HALBSP_INFO("	Bad Pp -> pp2Board(%u) boardInfo(%u) boardType(%u) => Fix to Unknown board type !",
					bsp_base.Board.Cur.pp2board,
					bsp_base.Board.Cur.boardInfo,
					bsp_base.Board.Cur.boardType);

				/* Fail */
				return(-1);
			}
			else {
				HALBSP_INFO("	Use default Pp -> pp2Board(%u) boardInfo(%u) boardType(%u) => Warning use default value !",
					bsp_base.Board.Cur.pp2board,
					bsp_base.Board.Cur.boardInfo,
					bsp_base.Board.Cur.boardType);
			}
		}
		else {
			const char *firmType;
			switch (bsp_base.Env.Krnopt.typeFirmware) {
				case BSP_Firmware_type_operational:
				{
					firmType = "OPERATIONAL";
				}
				break;
	
				case BSP_Firmware_type_lanrescue:
				{
					firmType = "LANRESCUE";
				}
				break;
	
				case BSP_Firmware_type_rescue:
				{
					firmType = "RESCUE";
				}
				break;
	
				default:
				{
					/* Panic */
					//panic("Bad Type Firmware");
					firmType = "Bad Type Firmware";
				}
				break;
			}

			HALBSP_INFO("	Manufacturer(%s) Serial Number(%s)",
					(char *)bsp_get_permParam(	BSP_Get_permParam_manufacturer),
					(char *)bsp_get_permParam(	BSP_Get_permParam_serial_number));
			HALBSP_INFO("	Sagem Hardware ProductId(%s) Version(%s) Revision(%s)",
					(char *)bsp_get_permParam(	BSP_Get_permParam_sagem_product_id),
					(char *)bsp_get_permParam(	BSP_Get_permParam_sagem_hardware_version),
					(char *)bsp_get_permParam(	BSP_Get_permParam_sagem_hardware_revision));
			HALBSP_INFO("	Customer Hardware ProductId(%s) Version(%s) Revision(%s)",
					(char *)bsp_get_permParam(	BSP_Get_permParam_customer_product_id),
					(char *)bsp_get_permParam(	BSP_Get_permParam_customer_hardware_version),
					(char *)bsp_get_permParam(	BSP_Get_permParam_customer_hardware_revision));

			HALBSP_INFO("	Firmware Version(%s) Type(%s)",
					(char *)bsp_get_permParam(	BSP_Get_permParam_firmware_version),
					firmType);

/*			HALBSP_DEBUG("	Size {Ram -> [Pp(%d) Linux(%ld)] Flash ->[Pp(%d) Linux(%d)]}\n",
					(*(__u32 *)bsp_get_permParam(	BSP_Get_permParam_ram_size)) * 1024,
					totalram_pages << (PAGE_SHIFT-10),
					(*(__u32 *)bsp_get_permParam(	BSP_Get_permParam_flash_size)) * 1024,
					board_flash_size / 1024);
[laurentc@antonieserver linux-2.6.28.8]$ grep -nr "On Board flash device" .
Binary file ./vmlinux.o matches
Binary file ./drivers/mtd/maps/ikan6850.o matches
./drivers/mtd/maps/ikan6850.c:236:      printk(KERN_NOTICE "On Board flash device: 0x%x at 0x%x\n", WINDOW_SIZE, WINDOW_ADDR);
./drivers/mtd/maps/adi6843.c:118:       printk(KERN_NOTICE "On Board flash device: 0x%x at 0x%x\n", WINDOW_SIZE, WINDOW_ADDR);
B
*/
			HALBSP_INFO("	Size {Ram -> [Pp(%d) Linux(%ld)] Flash ->[Pp(%d)]}",
					(*(__u32 *)bsp_get_permParam(	BSP_Get_permParam_ram_size)) * 1024,
					totalram_pages << (PAGE_SHIFT-10),
					(*(__u32 *)bsp_get_permParam(	BSP_Get_permParam_flash_size)) * 1024
					);
		}
	}
	
	/* Gpio */
	{
		/* Initialize Gpio */
	  	bsp_gpio_initialize();
	}

	/* Dev */
	{
		/* Base */
		{
			/* Register chrdev region */
			if (register_chrdev_region(	MKDEV(BSP_DEV_MAJOR, 0),
							BSP_Dev_minor_max,
							BSP_DEV_NAME) != 0) {
				HALBSP_INFO(" KO\n");
	
				/* Error */
				return(-1);
			}

		}

		/* All minor */
		{
			unsigned char _minor;

			for (_minor = 0; _minor < BSP_Dev_minor_max; _minor++) {
				/* Various */
				init_MUTEX(&(bsp_base.Dev[_minor].sem));
				bsp_base.Dev[_minor].nbOpen		= 0;

				/* Cdev_init */
				cdev_init(	&(bsp_base.Dev[_minor].cdev),
							&(bsp_base.Dev[_minor].fops));


				/* Add cdev */
				if (cdev_add(	&(bsp_base.Dev[_minor].cdev),
							MKDEV(BSP_DEV_MAJOR, _minor),
							1) != 0) {
					HALBSP_ERROR(	"FAILURE -> cdev_add.");

					for (_minor = _minor - 1; _minor <= 0; _minor--) {
						/* Cdev_del */
						cdev_del(&(bsp_base.Dev[_minor].cdev));
					}

					/* Unregister chrdev */
					unregister_chrdev_region(	MKDEV(BSP_DEV_MAJOR, _minor),
											BSP_Dev_minor_max);
		
					/* Ko */
					return(-1);
				}

				/* Create Class */
				{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
					if (IS_ERR(bsp_base.Dev[_minor].pClass = class_create(	THIS_MODULE,
																bsp_base.Dev[_minor].name))) {
#else
					if (IS_ERR(bsp_base.Dev[_minor].pClass = class_simple_create(	THIS_MODULE,
																		bsp_base.Dev[_minor].name))) {
#endif
						HALBSP_ERROR(	"FAILURE -> Creating bsp class.");

						for (_minor = _minor; _minor <= 0; _minor--) {
							/* CLASS */
							{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
								/* Destroy */
								class_destroy(bsp_base.Dev[_minor].pClass);
#else
								/* Destroy */
								class_simple_destroy(bsp_base.Dev[_minor].pClass);
#endif
							}

							/* Cdev_del */
							cdev_del(&(bsp_base.Dev[_minor].cdev));
						}
		
						/* Unregister chrdev */
						unregister_chrdev_region(	MKDEV(BSP_DEV_MAJOR, _minor),
												BSP_Dev_minor_max);
		
						/* Ko */
						return(-1);
					}
				}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
				/* Device Create */
				device_create(	bsp_base.Dev[_minor].pClass,
									NULL,
									MKDEV(BSP_DEV_MAJOR, _minor),
									NULL,
									bsp_base.Dev[_minor].name);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
				/* Device Create */
				class_device_create(	bsp_base.Dev[_minor].pClass,
									NULL,
									MKDEV(BSP_DEV_MAJOR, _minor),
									NULL,
									bsp_base.Dev[_minor].name);
#else
				/* Device Add */
				class_simple_device_add(	bsp_base.Dev[_minor].pClass,
									MKDEV(BSP_DEV_MAJOR, _minor),
									NULL,
									bsp_base.Dev[_minor].name);
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
				/* Mk Cdev */
				devfs_mk_cdev(	MKDEV(BSP_DEV_MAJOR, _minor),
							S_IFCHR | S_IRUGO | S_IWUGO,
							bsp_base.Dev[_minor].Dev.name) {
#endif
			}
		}
	}

	/* Register /sys/sagem/ SysCtl */
	{
		/* Register device */
//		if ((bsp_base.SysCtl.head = register_sysctl_table(	bsp_base.SysCtl.sagem,
//													1)) == 0) {
		if ((bsp_base.SysCtl.head = register_sysctl_table(	bsp_base.SysCtl.sagem)) == 0) {
			HALBSP_ERROR(	"Error register_sysctl on /sys/sagem/");
		
			/* Error */
			return(-1);
		}
	}

	// LCe temporaire !!!!!!
	bsp_gpio_set_state (BSP_Gpio_name_switch_rst, BSP_Gpio_state_set);
	bsp_gpio_set_state (BSP_Gpio_name_relay_fxs_1, BSP_Gpio_state_set);
	bsp_gpio_set_state (BSP_Gpio_name_relay_fxs_2, BSP_Gpio_state_set);
	bsp_gpio_set_state (BSP_Gpio_name_usb_pwr_en_master_1, BSP_Gpio_state_set);
	bsp_gpio_set_state (BSP_Gpio_name_wifi_rst_5G, BSP_Gpio_state_set);
	bsp_gpio_set_state (BSP_Gpio_name_wifi_rst_2G4, BSP_Gpio_state_set);

	if ( bsp_base.Board.Cur.boardType == BSP_Board_type_lbv3)
	{
		bsp_gpio_set_state (BSP_Gpio_name_led_power, BSP_Gpio_state_unset);
		bsp_gpio_set_state (BSP_Gpio_name_voice_rst_dect, BSP_Gpio_state_unset);
	}

#if (defined HALBSP_PERGPIO_PROCENTRY)

	/* Akka(S.VIE), 2011-12-01: Per-LED proc entries.
	 */
	bsp_led_proc_initialize();

#endif /* HALBSP_PERGPIO_PROCENTRY */

	HALBSP_INFO("Ok ...");
	
	return(0);
}

static void __exit bsp_exit(void)
{
	HALBSP_INFO(	"BSP(%s): Unload ...",
			(char *)bsp_get_permParam(	BSP_Get_permParam_manufacturer));

	/* If register */
	if (bsp_base.SysCtl.head != (struct ctl_table_header *)0) {
		HALBSP_ERROR("Unregister /sys/sagem/ SysCtl ...");

		/* Unregister sysctl */
		unregister_sysctl_table(bsp_base.SysCtl.head);

		/* Unset */
		bsp_base.SysCtl.head = (struct ctl_table_header *)0;

		HALBSP_INFO("OK");
	}

	/* All minor */
	{
		unsigned char _minor;

		for (_minor = 0; _minor < BSP_Dev_minor_max; _minor++) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
			/* Rem DEVFS */
			devfs_remove(bsp_base.Dev[_minor].cdev.kobj.name);
#endif

			/* CLASS */
			{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
				/* Remove */
				device_destroy(	bsp_base.Dev[_minor].pClass,
									MKDEV(BSP_DEV_MAJOR, _minor));
		
				/* Destroy */
				class_destroy(bsp_base.Dev[_minor].pClass);

#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
				/* Remove */
				class_device_destroy(	bsp_base.Dev[_minor].pClass,
									MKDEV(BSP_DEV_MAJOR, _minor));
		
				/* Destroy */
				class_destroy(bsp_base.Dev[_minor].pClass);
#else
				/* Remove */
				class_simple_device_remove(MKDEV(BSP_DEV_MAJOR, _minor));
		
				/* Destroy */
				class_simple_destroy(bsp_base.Dev[_minor].pClass);
#endif
			}

			/* Del Cdev */
			cdev_del(&(bsp_base.Dev[_minor].cdev));
		}

		/* Unregister chrdev region */
		unregister_chrdev_region(	MKDEV(BSP_DEV_MAJOR, _minor),
								BSP_Dev_minor_max);
	}

#if (defined HALBSP_PERGPIO_PROCENTRY)

	/* Akka(S.VIE), 2011-12-01: Per-LED proc entries.
	 */
  bsp_led_proc_release();

#endif /* HALBSP_PERGPIO_PROCENTRY */

	HALBSP_INFO(	"BSP(%s): Ok ...",
			(char *)bsp_get_permParam(	BSP_Get_permParam_manufacturer));
}

/* Retrieving myfs_start */
static int __init bsp_retrieve_myfs_start(char *_pStr)
{
	/* Get option */
	get_option(	&_pStr,
				&(bsp_base.Env.Krnopt.myfs_start));

	/* Ok */
	return 1;
}

/* Retrieving myfs_size */
static int __init bsp_retrieve_myfs_size(char *_pStr)
{
	/* Get option */
	get_option(	&_pStr,
				&(bsp_base.Env.Krnopt.myfs_size));

	/* Ok */
	return 1;
}

/* Retrieving operational_start_size */
static int __init bsp_retrieve_operational_start(char *_pStr)
{
	/* Get option */
	get_option(	&_pStr,
				&(bsp_base.Env.Krnopt.operational_start));

	/* Ok */
	return 1;
}

/* Retrieving rescue_start_size */
static int __init bsp_retrieve_rescue_start(char *_pStr)
{
	/* Get option */
	get_option(	&_pStr,
				&(bsp_base.Env.Krnopt.rescue_start));

	/* Ok */
	return 1;
}

/** Init module. */
module_init(bsp_init);
/** Exit module. */
module_exit(bsp_exit);

__setup(	"myfs_start=",
		bsp_retrieve_myfs_start);
__setup(	"myfs_size=",
		bsp_retrieve_myfs_size);
__setup(	"operational_start=",
		bsp_retrieve_operational_start);
__setup(	"rescue_start=",
		bsp_retrieve_rescue_start);

// vim:ts=5:
