/*
 * Copyright (c) 2011 SoftAtHome
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/leds.h>
#include "khco_led_api.h"
#include "led_linux.h"

#define LED_AUTONAME_MAX	16

struct sah_linuxled {
	struct led_classdev	cdev;
	BOARD_LED_NAME		sah_led_name;
};

struct sah_linuxled_ext {
	struct sah_linuxled	led;
	char			name[LED_AUTONAME_MAX];
};

struct sah_linuxled_ctx {
	int 			num_leds;
	struct sah_linuxled 	leds[0];
};

struct sah_linuxled_ext_ctx {
	int			num_leds;
	struct sah_linuxled_ext leds[0];
};

static void brightness_set(struct led_classdev *cdev, enum led_brightness value)
{
	struct sah_linuxled *led = container_of(cdev,
					struct sah_linuxled, cdev);
	if (value != LED_OFF)
		kled_hw_set_on(led->sah_led_name);
	else
		kled_hw_set_off(led->sah_led_name);
}

void sah_linuxleds_unregister(void *opaque)
{
	struct sah_linuxled_ext_ctx *ctx = opaque;
	int i;

	for (i = 0; i < ctx->num_leds; i++)
		led_classdev_unregister(&ctx->leds[i].led.cdev);

	kfree(ctx);
}

void *sah_linuxleds_register(int num_leds)
{
	struct sah_linuxled_ext_ctx *ctx;
	int i;

	if ((unsigned)num_leds > SAH_LINUXLEDS_MAXNUM)
		return NULL;

	ctx = kzalloc(sizeof(struct sah_linuxled_ext_ctx) +
			sizeof(struct sah_linuxled_ext) * num_leds,
			GFP_KERNEL);
	if (!ctx)
		return NULL;

	ctx->num_leds = num_leds;

	for (i = 0; i < num_leds; i++) {
		snprintf(ctx->leds[i].name, LED_AUTONAME_MAX,
			"led_%d", i);
		ctx->leds[i].led.cdev.name = ctx->leds[i].name;
		ctx->leds[i].led.sah_led_name = i;
		ctx->leds[i].led.cdev.brightness  = LED_OFF;
		ctx->leds[i].led.cdev.brightness_set = brightness_set;

		if (led_classdev_register(NULL, &ctx->leds[i].led.cdev)) {
			ctx->num_leds = i;
			sah_linuxleds_unregister(ctx);
			ctx = NULL;
			break;
		}
	}

	return ctx;
}

void sah_linuxleds_unregister_desc(void *opaque)
{
	struct sah_linuxled_ctx *ctx = opaque;
	int i;

	for (i = 0; i < ctx->num_leds; i++)
		led_classdev_unregister(&ctx->leds[i].cdev);

	kfree(ctx);
}

void *sah_linuxleds_register_desc(int num_leds, struct sah_linuxled_desc *desc)
{
	struct sah_linuxled_ctx *ctx;
	int i;

	ctx = kzalloc(sizeof(struct sah_linuxled_ctx) +
			sizeof(struct sah_linuxled) * num_leds,
			GFP_KERNEL);
	if (!ctx)
		return NULL;

	ctx->num_leds = num_leds;

	for (i = 0; i < num_leds; i++) {
		ctx->leds[i].sah_led_name = desc[i].sah_led_name;
		ctx->leds[i].cdev.name 	  = desc[i].name;
		ctx->leds[i].cdev.brightness 	 = LED_OFF;
		ctx->leds[i].cdev.brightness_set = brightness_set;

		if (led_classdev_register(NULL, &ctx->leds[i].cdev)) {
			ctx->num_leds = i;
			sah_linuxleds_unregister_desc(ctx);
			ctx = NULL;
			break;
		}
	}

	return ctx;
}
