/*
 * Mapping definitions for the flash on any ADI boards with secure upgrade
 *
 * Author:	CL @ Sagemcom
 * Copyright:	(C) 2011 Sagemcom
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __MTD_SECURE_UPGRADE_H__
#define __MTD_SECURE_UPGRADE_H__

#include <linux/autoconf.h>

// MTD Physical Erase Block size
#define MTD_PEB_SIZE   CONFIG_MTD_SECURE_UPDATE_PEB_SIZE

#if defined(CONFIG_MTD_SECURE_UPDATE_UBI)
// UBI Logical Erase Block size
#define MTD_LEB_SIZE   CONFIG_MTD_SECURE_UPDATE_LEB_SIZE
// Erase block size for UBI volumes
#define MTD_BLOCK_SIZE      MTD_LEB_SIZE
#else
// Erase block size for physical MTD partitions
#define MTD_BLOCK_SIZE      MTD_PEB_SIZE
#endif

/* Partition sizes in Bytes */

// Boot loader
#define MTD_BOOT_SIZE   (CONFIG_MTD_SECURE_UPDATE_BOOT_SIZE * MTD_PEB_SIZE)
// Authentication header
#define MTD_AUTH_SIZE   (CONFIG_MTD_SECURE_UPDATE_AUTH_SIZE * MTD_BLOCK_SIZE)

enum {
  MTD_IPL_PART,
  MTD_UBI_PART,
};

// Configuration partitions
#if defined(CONFIG_MTD_SECURE_UPDATE_CONFIG_PARTITIONS)
#define MTD_CONFIG_SIZE     (CONFIG_MTD_SECURE_UPDATE_CONFIG_SIZE  * MTD_BLOCK_SIZE)
#else
#define MTD_CONFIG_SIZE     0
#endif
// Factory partition
#if defined(CONFIG_MTD_SECURE_UPDATE_FACTORY_PARTITION)
#define MTD_FACTORY_SIZE     (CONFIG_MTD_SECURE_UPDATE_FACTORY_SIZE  * MTD_BLOCK_SIZE)
#else
#define MTD_FACTORY_SIZE     0
#endif
// GUI partition
#if defined(CONFIG_MTD_SECURE_UPDATE_GUI_PARTITION)
#define MTD_GUI_SIZE     (CONFIG_MTD_SECURE_UPDATE_GUI_SIZE  * MTD_BLOCK_SIZE)
#else
#define MTD_GUI_SIZE     0
#endif
// Firmware partition
#define MTD_FIRM_SIZE       (CONFIG_MTD_SECURE_UPDATE_OPERATIONAL_SIZE  * MTD_BLOCK_SIZE)
#if defined(CONFIG_MTD_SECURE_UPDATE_DUAL_ASYMMETRIC)
// Rescue firmware partition
#define MTD_RESCUE_SIZE     (CONFIG_MTD_SECURE_UPDATE_RESCUE_SIZE  * MTD_BLOCK_SIZE)
#endif
#if defined(CONFIG_MTD_SECURE_UPDATE_SINGLE_EXTENDED)
// Root filesystem extension partition
#define MTD_EXTENSION_SIZE  (CONFIG_MTD_SECURE_UPDATE_EXTENSION_SIZE  * MTD_BLOCK_SIZE)
#endif
#if defined(CONFIG_MTD_SECURE_UPDATE_NO_FILESYSTEM)
#define MTD_ALL_FS_SIZE     0
#else
// First journalized filesystem partition
#define MTD_FS1_SIZE        (CONFIG_MTD_SECURE_UPDATE_FILESYSTEM1_SIZE * MTD_BLOCK_SIZE)
#if defined(CONFIG_MTD_SECURE_UPDATE_TWO_FILESYSTEM)
// Second journalized filesystem partition
#define MTD_FS2_SIZE        (CONFIG_MTD_SECURE_UPDATE_FILESYSTEM2_SIZE * MTD_BLOCK_SIZE)
#define MTD_ALL_FS_SIZE     (MTD_FS1_SIZE + MTD_FS2_SIZE)
#else
#define MTD_ALL_FS_SIZE     MTD_FS1_SIZE
#endif
#endif

#define MTD_FIRM_OFFSET     (MTD_BOOT_SIZE + MTD_FACTORY_SIZE + (2 * MTD_CONFIG_SIZE))

#if defined(CONFIG_MTD_SECURE_UPDATE_DUAL_ASYMMETRIC)
#define MTD_ALL_FIRM_SIZE   (MTD_RESCUE_SIZE + MTD_FIRM_SIZE)
#elif defined(CONFIG_MTD_SECURE_UPDATE_DUAL_SYMMETRIC)
#define MTD_ALL_FIRM_SIZE   (2 * MTD_FIRM_SIZE)
#elif defined(CONFIG_MTD_SECURE_UPDATE_SINGLE_EXTENDED)
#define MTD_ALL_FIRM_SIZE   (MTD_FIRM_SIZE + MTD_EXTENSION_SIZE)
#endif

#if (MTD_FIRM_OFFSET + MTD_ALL_FIRM_SIZE + MTD_ALL_FS_SIZE) > (CONFIG_MTD_SECURE_UPDATE_FLASH_SIZE * 1024 * 1024)
#error "Sum of MTD partitions is greater than flash size."
#endif

enum {
  MTD_BOOT_PART,      // Boot loader
  //MTD_AUTH_PART,      // Authentication header (dynamicaly defined)
  MTD_KERNEL_PART,    // Current kernel (dynamicaly defined)
  MTD_ROOTFS_PART,    // Current root filesystem (dynamicaly defined)
#if defined(CONFIG_MTD_SECURE_UPDATE_DUAL_ASYMMETRIC)
  MTD_RESCUE_PART,    // Rescue firmware
  MTD_FIRM_PART,      // Operational firmware
#elif defined(CONFIG_MTD_SECURE_UPDATE_DUAL_SYMMETRIC)
  MTD_FIRM1_PART,     // First firmware
  MTD_FIRM2_PART,     // Second firmware
#elif defined(CONFIG_MTD_SECURE_UPDATE_SINGLE_EXTENDED)
  // Order: firmware + extension
  MTD_FIRM1_PART,     // Minimal firmware
  MTD_EXTEND1_PART,   // Root filesystem extension
  // Order: extension + firmware
  MTD_FIRM2_PART,     // Minimal firmware
  MTD_EXTEND2_PART,   // Root filesystem extension
#endif
#if defined(CONFIG_MTD_SECURE_UPDATE_FACTORY_PARTITION)
  MTD_FACTORY_PART,
#endif
#if defined(CONFIG_MTD_SECURE_UPDATE_GUI_PARTITION)
  MTD_GUI_PART,
#endif
#if defined(CONFIG_MTD_SECURE_UPDATE_CONFIG_PARTITIONS)
  MTD_CONFIG1_PART,
  MTD_CONFIG2_PART,
#endif
#if !defined(CONFIG_MTD_SECURE_UPDATE_NO_FILESYSTEM)
  MTD_FS1_PART,
#if defined(CONFIG_MTD_SECURE_UPDATE_TWO_FILESYSTEM)
  MTD_FS2_PART,
#endif
#endif
};

#endif // __MTD_SECURE_UPGRADE_H__

