/*
 * Handle mapping of the flash on any boards with secure upgrade
 *
 * Author:	CL @ Sagemcom
 * Copyright:	(C) 2011 Sagemcom
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __MTD_MAPS_COMMON_SAGEMCOM_H__
#define __MTD_MAPS_COMMON_SAGEMCOM_H__

#include <linux/mtd/maps/mtd-secure-upgrade.h>
#include "ubi-partitions.h"


static struct mtd_partition ubi_partition_info[]={
    { .name   = "ipl", // Initial program loader
      .offset = 0,
      .size   = MTD_BOOT_SIZE }
   ,{ .name   = "ubi", // Partion for UBI device
      .offset = MTDPART_OFS_APPEND,
      .size   = MTDPART_SIZ_FULL /* Sice end of the device */ }
};

static struct mtd_partition ubi_image_partition_info[]={
    { .name   = "auth", // Authentication header
      .offset = 0,
      .size   = MTD_AUTH_SIZE }
   ,{ .name   = "kernel", // Current kernel
      .offset = /*MTDPART_OFS_APPEND*/0,
      .size   = 0 /* Dynamic definition */ }
   ,{ .name   = "rootfs", // Current root filesystem
      .offset = 0 /* Dynamic definition */,
      .size   = 0 /* Dynamic definition */ }
};

static struct mtd_partition ubi_gui_partition_info[]={
    { .name   = "auth", // Authentication header
      .offset = 0,
      .size   = MTD_AUTH_SIZE }
   ,{ .name   = "guifs", // GUI filesystem
      .offset = /*MTDPART_OFS_APPEND*/0,
      .size   = MTDPART_SIZ_FULL /* Sice end of the device */ }
};

static struct mtd_partition secure_partition_info[]={
    { .name   = "boot", // Boot program loader
      .offset = 0,
      .size   = MTD_BOOT_SIZE }
   ,{ .name   = "kernel", // Current kernel
      .offset = 0 /* Dynamic definition */,
      .size   = 0 /* Dynamic definition */ }
   ,{ .name   = "rootfs", // Current root filesystem
      .offset = 0 /* Dynamic definition */,
      .size   = 0 /* Dynamic definition */ }
   ,{ .name   = "firmwares", // All system firmwares
      .offset = MTD_FIRM_OFFSET,
      .size   = MTD_ALL_FIRM_SIZE }
#if defined(CONFIG_MTD_SECURE_UPDATE_DUAL_ASYMMETRIC)
   ,{ .name   = "rescue", // Rescue firmxare
      .offset = MTD_FIRM_OFFSET,
      .size   = MTD_RESCUE_SIZE }
   ,{ .name   = "operational", // Operational firmware
      .offset = (MTD_FIRM_OFFSET + MTD_RESCUE_SIZE),
      .size   = MTD_FIRM_SIZE }
#elif defined(CONFIG_MTD_SECURE_UPDATE_DUAL_SYMMETRIC)
   ,{ .name   = "firmware1", // First firmware partition
      .offset = MTD_FIRM_OFFSET,
      .size   = MTD_FIRM_SIZE }
   ,{ .name   = "firmware2", // Second firmware partition
      .offset = (MTD_FIRM_OFFSET + MTD_FIRM_SIZE),
      .size   = MTD_FIRM_SIZE }
#elif defined(CONFIG_MTD_SECURE_UPDATE_SINGLE_EXTENDED)
   ,{ .name   = "firmware1", // Firmware partition for "firmware + extension" order
      .offset = MTD_FIRM_OFFSET,
      .size   = MTD_FIRM_SIZE }
   ,{ .name   = "extension1", // Extension partition for "firmware + extension" order
      .offset = (MTD_FIRM_OFFSET + MTD_FIRM_SIZE),
      .size   = MTD_EXTENSION_SIZE }
   ,{ .name   = "firmware2", // Firmware partition for "extension + firmware" order
      .offset = (MTD_FIRM_OFFSET + MTD_EXTENSION_SIZE),
      .size   = MTD_FIRM_SIZE }
   ,{ .name   = "extension2", // Extension partition for "extension + firmware" order
      .offset = MTD_FIRM_OFFSET,
      .size   = MTD_EXTENSION_SIZE }
#endif
#if defined(CONFIG_MTD_SECURE_UPDATE_FACTORY_PARTITION)
   ,{ .name   = "factory", // Factory informations
      .offset = MTD_BOOT_SIZE,
      .size   = MTD_FACTORY_SIZE }
#endif
#if defined(CONFIG_MTD_SECURE_UPDATE_CONFIG_PARTITIONS)
   ,{ .name   = "config1", // First configuration partition
      .offset = (MTD_BOOT_SIZE + MTD_FACTORY_SIZE),
      .size   = MTD_CONFIG_SIZE }
   ,{ .name   = "config2", // Second configuration partition
      .offset = (MTD_BOOT_SIZE + MTD_FACTORY_SIZE + MTD_CONFIG_SIZE),
      .size   = MTD_CONFIG_SIZE }
#endif
#if defined(CONFIG_MTD_SECURE_UPDATE_GUI_PARTITION)
   ,{ .name   = "gui", // Externalized GUI firmware
      .offset = (MTD_FIRM_OFFSET + MTD_ALL_FIRM_SIZE),
      .size   = MTD_GUI_SIZE }
#endif
#if !defined(CONFIG_MTD_SECURE_UPDATE_NO_FILESYSTEM)
   ,{ .name   = "filesystem1", // First journalized filesystem
      .offset = (MTD_FIRM_OFFSET + MTD_ALL_FIRM_SIZE + MTD_GUI_SIZE),
      .size   = MTD_FS1_SIZE }
#if defined(CONFIG_MTD_SECURE_UPDATE_TWO_FILESYSTEM)
   ,{ .name   = "filesystem2", // Second journalized filesystem
      .offset = (MTD_FIRM_OFFSET + MTD_ALL_FIRM_SIZE + MTD_GUI_SIZE + MTD_FS1_SIZE),
      .size   = MTD_FS2_SIZE }
#endif
#endif
};

static unsigned int rootfs_start = 0;
static unsigned int rootfs_offset = 0;
static unsigned int rootfs_size = 0;
static unsigned int image_addr = 0;
static char image_ubivol[UBI_VOL_NAME_MAX + 1] = "";

static int __init set_image_addr(char *str)
{
	get_option(&str, &image_addr);
	return 1;
}

static int __init set_image_ubivol(char *str)
{
  strncpy(image_ubivol, str, UBI_VOL_NAME_MAX);
  image_ubivol[UBI_VOL_NAME_MAX] = '\0';
  printk("UBI image volume: \"%s\"\n", image_ubivol);
	return 1;
}

static int __init set_rootfs_start(char *str)
{
	get_option(&str, &rootfs_start);
	return 1;
}

static int __init set_rootfs_offset(char *str)
{
	get_option(&str, &rootfs_offset);
	return 1;
}

static int __init set_rootfs_size(char *str)
{
	get_option(&str, &rootfs_size);
	return 1;
}

__setup("image_addr=", set_image_addr);
__setup("image_ubivol=", set_image_ubivol);
__setup("rootfs_start=", set_rootfs_start);
__setup("rootfs_offset=", set_rootfs_offset);
__setup("rootfs_size=", set_rootfs_size);

void mtd_add_volume_partition(struct mtd_info *volmtd, struct ubi_volume *vol)
{
  printk("Add UBI volume partitions: name=%s\n", vol->name);
  if (strcmp(vol->name, image_ubivol) == 0) {
    ubi_image_partition_info[1].size = rootfs_offset;
    ubi_image_partition_info[2].offset = rootfs_offset;
    ubi_image_partition_info[2].size = rootfs_size;
    add_mtd_partitions(volmtd, ubi_image_partition_info,
        sizeof(ubi_image_partition_info) / sizeof(ubi_image_partition_info[0]));
  } else if (strcmp(vol->name, "gui") == 0) {
    add_mtd_partitions(volmtd, ubi_gui_partition_info,
        sizeof(ubi_gui_partition_info) / sizeof(ubi_gui_partition_info[0]));
  }
}

#endif // __MTD_MAPS_COMMON_SAGEMCOM_H__

