include $(STAGINGDIR)/components.config


ifeq ($(notdir $(CC)),gcc)
	UPNP_FULL_CONFIGURATION:= --prefix=/ --disable-samples --enable-shared --without-documentation --disable-notification_reordering
else
	UPNP_FULL_CONFIGURATION:= --prefix=/ --disable-samples --enable-shared --without-documentation --disable-notification_reordering --host=mips-linux --build=i686
endif

ifeq ($(CONFIG_SAH_LIB_UPNP_ENABLE_DEBUG),y)
	CFLAGS= -ggdb3 -O0
else
	CFLAGS= -g -O2
endif

ifeq ($(CONFIG_SAH_LIB_UPNP_ENABLE_IPV6),y)
  UPNP_FULL_CONFIGURATION:=$(UPNP_FULL_CONFIGURATION) --enable-ipv6
endif

ifeq ($(CONFIG_SAH_LIB_UPNP_ENABLE_LOGGING),y)
	CFLAGS+=-DDEBUG
	UPNP_FULL_CONFIGURATION += --enable-debug
endif

CFLAGS+=-pthread

UPNP_FULL_CONFIGURATION:=$(UPNP_FULL_CONFIGURATION) PTHREAD_LIBS=-lpthread CFLAGS="$(CFLAGS)"

LIBUPNP-VERSION := libupnp-1.6.13

all: libupnp-all

clean:
	- [ -f $(LIBUPNP-VERSION)/Makefile ] && \
	  $(MAKE) -C $(LIBUPNP-VERSION) clean
	- find $(LIBUPNP-VERSION) -name .deps -exec rm -rf '{}' \;
	find $(LIBUPNP-VERSION) -name .dirstamp -exec rm '{}' \;
	find $(LIBUPNP-VERSION) -name "stamp-h?" -exec rm '{}' \;
	find $(LIBUPNP-VERSION) -name "Makefile" -exec rm '{}' \;
	rm -f $(LIBUPNP-VERSION)/autoconfig.h
	rm -f $(LIBUPNP-VERSION)/config.log
	rm -f $(LIBUPNP-VERSION)/config.status
	rm -f $(LIBUPNP-VERSION)/upnp/inc/upnpconfig.h
	rm -f $(LIBUPNP-VERSION)/*.pc
	rm -rf $(LIBUPNP-VERSION)/autom4te.cache

install:
	$(MAKE) -C $(LIBUPNP-VERSION) install DESTDIR=$(STAGINGDIR)
	$(INSTALL) -m 0644 $(LIBUPNP-VERSION)/upnp/inc/* $(STAGINGDIR)/include/upnp/
	$(INSTALL) -m 0644 $(LIBUPNP-VERSION)/upnp/src/inc/* $(STAGINGDIR)/include/upnp/
	$(INSTALL) -d $(PKG_CONFIG_LIBDIR)
	$(INSTALL) -m 0755 libupnp.pc $(PKG_CONFIG_LIBDIR)/
	# flags archives for library reduction script (see bug 13956)
	#ln -sf libupnp.a $(STAGINGDIR)/lib/libupnp_pic.a # see bug 15138
	#ln -sf libixml.a $(STAGINGDIR)/lib/libixml_pic.a # see bug 15138
	#ln -sf libthreadutil.a $(STAGINGDIR)/lib/libthreadutil_pic.a # see bug 15138

release:
	$(INSTALL) -d $(ROOT_FS_DIR)/lib
	$(INSTALL) -m 0644 $(STAGINGDIR)/lib/libupnp.so.6 $(ROOT_FS_DIR)/lib
ifneq ($(CONFIG_SAH_LIB_UPNP_ENABLE_DEBUG_NO_STRIPPING),y)
	$(STRIP) $(ROOT_FS_DIR)/lib/libupnp.so.6
endif
	ln -sf libupnp.so.6 $(ROOT_FS_DIR)/lib/libupnp.so
	$(INSTALL) -m 0644 $(STAGINGDIR)/lib/libthreadutil.so.6 $(ROOT_FS_DIR)/lib/
ifneq ($(CONFIG_SAH_LIB_UPNP_ENABLE_DEBUG_NO_STRIPPING),y)
	$(STRIP) $(ROOT_FS_DIR)/lib/libthreadutil.so.6
endif
	ln -sf libthreadutil.so.6 $(ROOT_FS_DIR)/lib/libthreadutil.so
	$(INSTALL) -m 0644 $(STAGINGDIR)/lib/libixml.so.2 $(ROOT_FS_DIR)/lib/
ifneq ($(CONFIG_SAH_LIB_UPNP_ENABLE_DEBUG_NO_STRIPPING),y)
	$(STRIP) $(ROOT_FS_DIR)/lib/libixml.so.2
endif
	ln -sf libixml.so.2 $(ROOT_FS_DIR)/lib/libixml.so

$(LIBUPNP-VERSION)/Makefile:
	cd $(LIBUPNP-VERSION); \
	./configure $(UPNP_FULL_CONFIGURATION)

libupnp-all: $(LIBUPNP-VERSION)/Makefile
	make -C $(LIBUPNP-VERSION) all

.PHONY: all clean install release
