ifeq ($(notdir $(CC)),gcc)
	# When building for x86 target, the option --disable-dbus can be removed if you install package libdbus-1-dev
	CFG:=--host=i386-linux --enable-shared --with-gnu-ld --disable-mono --disable-monodoc --disable-dbus
	PKG_CONFIG_PATH := $(PKG_CONFIG_LIBDIR):/usr/lib/pkgconfig:/usr/share/pkgconfig
else
	CFG:=--build=i386-linux --host=mips-linux --disable-dbus 
	PKG_CONFIG_PATH := $(PKG_CONFIG_LIBDIR)
endif

all: Makefile avahi-daemon/avahi-daemon

Makefile:
	LDFLAGS="-L$(STAGINGDIR)/lib -ldaemon -Wl,--gc-sections" \
	CFLAGS="-I${STAGINGDIR}/include/expat/ -I${STAGINGDIR}/include/ -ffunction-sections -fdata-sections -Os" \
	LIBDAEMON_CFLAGS=-I${STAGINGDIR}/include/libdaemon \
	LIBDAEMON_LIBS=-L${STAGINGDIR}/lib \
	PKG_CONFIG_PATH=$(PKG_CONFIG_PATH) \
	./configure $(CFG) --prefix="" --with-distro=none --disable-glib --disable-gobject --disable-qt3 --disable-qt4 --disable-gtk --with-xml=expat --disable-gtk3 --disable-gdbm --disable-xmltoman --disable-manpages --disable-autoipd --disable-python --disable-pygtk --disable-python-dbus --disable-stack-protector --disable-doxygen-doc --with-avahi-user=avahi --with-avahi-group=avahi 

avahi-daemon/avahi-daemon:
	make 

clean:
	@echo " ##   Clean avahi"
	-make distclean

install:
	@echo " ##   Installing avahi"
	make DESTDIR=${STAGINGDIR} install
	$(INSTALL) -d ${STAGINGDIR}/sbin/
	$(INSTALL) -m 755 avahi-daemon/.libs/avahi-daemon ${STAGINGDIR}/sbin
	$(INSTALL) -m 755 avahi-dnsconfd/.libs/avahi-dnsconfd ${STAGINGDIR}/sbin
	$(INSTALL) -d ${STAGINGDIR}/lib/
	$(INSTALL) -m 755 avahi-common/.libs/libavahi-common.so.3.5.2 $(STAGINGDIR)/lib
	cd $(STAGINGDIR)/lib ; ln -sf libavahi-common.so.3.5.2 libavahi-common.so.3
	cd $(STAGINGDIR)/lib ; ln -sf libavahi-common.so.3.5.2 libavahi-common.so
	$(INSTALL) -m 755 avahi-core/.libs/libavahi-core.so.7.0.0 $(STAGINGDIR)/lib
	cd $(STAGINGDIR)/lib ; ln -fs libavahi-core.so.7.0.0 libavahi-core.so.7
	cd $(STAGINGDIR)/lib ; ln -fs libavahi-core.so.7.0.0 libavahi-core.so	
	$(INSTALL) -d ${STAGINGDIR}/share/avahi/
	$(INSTALL) -m 755 avahi-daemon/avahi-service.dtd $(STAGINGDIR)/share/avahi
	$(INSTALL) -d $(STAGINGDIR)/etc/avahi
	$(INSTALL) -m 755 avahi-daemon/avahi-daemon.conf $(STAGINGDIR)/etc/avahi
	$(INSTALL) -m 755 avahi-daemon/hosts $(STAGINGDIR)/etc/avahi
	$(INSTALL) -d $(STAGINGDIR)/etc_ro_fs/init.d
	$(INSTALL) -d $(STAGINGDIR)/etc/init.d
	$(INSTALL) -m 755 scripts/avahi.sh $(STAGINGDIR)/etc/init.d/avahi
	$(INSTALL) -d $(STAGINGDIR)/etc/avahi/services
	$(INSTALL) -m 755 scripts/http.service $(STAGINGDIR)/etc/avahi/services
	@echo " ##   Installing /etc/rc1.d/S98avahi"
	$(INSTALL) -d $(STAGINGDIR)/etc/rc1.d
	ln -sf ../init.d/avahi $(STAGINGDIR)/etc/rc1.d/S98avahi
	# fix for bug 13956
	$(INSTALL) -m 755 avahi-common/.libs/libavahi-common.a $(STAGINGDIR)/lib
	$(INSTALL) -m 755 avahi-core/.libs/libavahi-core.a $(STAGINGDIR)/lib
	#ln -sf libavahi-common.a $(STAGINGDIR)/lib/libavahi-common_pic.a # see bug 15138
	#ln -sf libavahi-core.a $(STAGINGDIR)/lib/libavahi-core_pic.a # see bug 15138
	

release:
	@echo " ##   Releasing avahi"
	$(INSTALL) -d $(ROOT_FS_DIR)/lib
	$(INSTALL) -m 755 ${STAGINGDIR}/lib/libavahi-common.so.3.5.2 $(ROOT_FS_DIR)/lib
	$(STRIP) $(ROOT_FS_DIR)/lib/libavahi-common.so.3.5.2
	cd $(ROOT_FS_DIR)/lib ; ln -sf libavahi-common.so.3.5.2 libavahi-common.so.3
	cd $(ROOT_FS_DIR)/lib ; ln -sf libavahi-common.so.3.5.2 libavahi-common.so
	$(INSTALL) -m 755 ${STAGINGDIR}/lib/libavahi-core.so.7.0.0 $(ROOT_FS_DIR)/lib
	$(STRIP) $(ROOT_FS_DIR)/lib/libavahi-core.so.7.0.0
	cd $(ROOT_FS_DIR)/lib ;	ln -sf libavahi-core.so.7.0.0 libavahi-core.so.7
	cd $(ROOT_FS_DIR)/lib ;	ln -sf libavahi-core.so.7.0.0 libavahi-core.so
	$(INSTALL) -d $(ROOT_FS_DIR)/sbin
	$(INSTALL) -m 755 ${STAGINGDIR}/sbin/avahi-daemon $(ROOT_FS_DIR)/sbin
	$(STRIP) $(ROOT_FS_DIR)/sbin/avahi-daemon
	$(INSTALL) -m 755 ${STAGINGDIR}/sbin/avahi-dnsconfd $(ROOT_FS_DIR)/sbin
	$(STRIP) $(ROOT_FS_DIR)/sbin/avahi-dnsconfd
	@echo " ##   Releasing /etc_ro_fs/rc1.d/S98avahi"
	$(INSTALL) -d $(ROOT_FS_DIR)/etc_ro_fs/init.d	
	$(INSTALL) -m 755 $(STAGINGDIR)/etc/init.d/avahi $(ROOT_FS_DIR)/etc_ro_fs/init.d/avahi
	$(INSTALL) -d $(ROOT_FS_DIR)/etc_ro_fs/avahi/services
	$(INSTALL) -m 755 $(STAGINGDIR)/etc/avahi/services/http.service $(ROOT_FS_DIR)/etc_ro_fs/avahi/services
	$(INSTALL) -d $(ROOT_FS_DIR)/etc_ro_fs/rc1.d
	ln -sf ../init.d/avahi $(ROOT_FS_DIR)/etc_ro_fs/rc1.d/S98avahi

doc:

.PHONY: all install release clean doc 

