#if !defined(PCB_CIRCULAR_BUFFER_H)
#define PCB_CIRCULAR_BUFFER_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <time.h>

/**
@file
@brief
Header file with circular buffer type definitions and circular buffer functions
*/

/**
@ingroup pcb_utils
@defgroup pcb_utils_circbuf CircularBuffer
@{

@brief
CircularBuffer handling functions.

@details
A circular buffer allows one to read an write to an in memory buffer, acting as a stream
and growing as needed. It expect reads and writes to be interlaced so that it will not 
expand indefinitely
*/

typedef struct _circbuf {
    FILE *stream;
    char *buffer;
    size_t size;
    char *read;
    size_t toRead;
    char *write;
    size_t toWrite;
    char seekMode;
} circbuf_t;

extern bool circbuf_initialize(circbuf_t *circbuf, size_t size);
extern void circbuf_cleanup(circbuf_t *circbuf);
extern void circbuf_reset(circbuf_t *circbuf);
extern FILE *circbuf_stream(circbuf_t *circbuf);
extern ssize_t circbuf_size(circbuf_t *circbuf);
extern ssize_t circbuf_availableForRead(circbuf_t *circbuf);
extern void *circbuf_readBuffer(circbuf_t *circbuf);
extern ssize_t circbuf_markAsRead(circbuf_t *circbuf, size_t size);
extern FILE *circbuf_fopen(circbuf_t *circbuf, const char *opentype);

#ifdef __cplusplus
}
#endif

#endif // PCB_CIRCULAR_BUFFER_H
