
#define PS_BUFFER_SIZE 1024*512       // the size of the circular buffer, if it has an overrun, the print job fails

#define PS_PRINT_JOB_RESET_TIMEOUT 95  // This is the ultimate timeout, if all else fails, this timeout will cause the print_stream to re-initialize
#define PS_HTTP_INTERVAL           15  // After this interval, a new http packet is fetched and processes, avoiding TCP/HTTP/IPP timeouts
#define PS_PRINT_ERROR_TIMEOUT     50  // If we could not write to the printer for PS_PRINT_ERROR_TIMEOUT seconds, there must be something wrong




#include "circular_buffer.h"
enum print_status {
  PRINTER_IDLE,     // printer is not opened
  PRINTER_OPENED,   // printer is opened
  PRINTER_PRINTING, // printer is printing
  PRINTER_ERROR     // a printer error has occurred
};

enum print_stream_status {
  STREAM_IDLE,      // no stream available
  STREAM_STARTING,  // stream is starting
  STREAM_STREAMING, // stream available and streaming
  STREAM_DONE       // stream part done, but still data in the buffer
};

typedef struct _print_stream {
  int  writeFD;      // file descriptor to the printer
  int  readFD;       // file descriptor to the incoming http stream
  circbuf_t *cb;      // circular buffer used as intermediate storage
  FILE *buffer;      // circular buffer used as intermediate storage
  int  lastRead;     // timestamp of last succesfull read from the incoming stream
  int  lastWrite;    // timestamp of last succesfull write to the printer
  enum print_status        printer_status; // the status of the print job
  enum print_stream_status stream_status;  // the status of the incoming http stream
} print_stream;



print_stream* ps_print_stream_init();
void          ps_print_stream_cleanup();
void          ps_print_stream_status();

int ps_printer_ready(print_stream* myStream);
int ps_printer_open(print_stream* myStream);
int ps_printer_close(print_stream* myStream);
int ps_printer_start(print_stream* myStream);
int ps_printer_stop(print_stream* myStream);
int ps_printer_error(print_stream* myStream);
int ps_printer_is_printing(print_stream* myStream);
int ps_printer_fd(print_stream* myStream);

int ps_stream_prepare(print_stream* myStream);
int ps_stream_bind(print_stream* myStream, int streamFD);
int ps_stream_release(print_stream* myStream);
int ps_stream_reset(print_stream* myStream);

int ps_stream_mask(print_stream* myStream, int fd);
int ps_stream_is_starting(print_stream* myStream);
int ps_stream_started(print_stream* myStream, int streamFD);
int ps_stream_overflow(print_stream* myStream);

int ps_print_job_data_available(print_stream* myStream);

int ps_print_job_start(print_stream* myStream);
int ps_print_job_get_data(print_stream* myStream, char *data, int bytes);
int ps_print_job_put_data(print_stream* myStream);
int ps_print_job_finished(print_stream* myStream);
int ps_print_job_finish(print_stream* myStream);
int ps_print_job_error(print_stream* myStream);
int ps_print_job_reset(print_stream* myStream);
int ps_print_job_interupted(print_stream* myStream);



