#!/bin/bash

tmpfile_start()
{
  tmpfile=`tempfile --suffix=.c` || return 1
  trap "/bin/rm -f -- '$tmpfile'" EXIT
  return 0
}

tmpfile_stop()
{
  /bin/rm -f -- "$tmpfile"
  trap - EXIT
  unset tmpfile
}

fail()
{
  echo "$1" >&2
  exit 1
}

[ -z "$CC" ] && fail "error: gcc not specified in CC environment variable"

tmpfile_start || exit 1
#cat > $tmpfile << "EOF"
printf "#define _BSD_SOURCE\n#include <endian.h>\n__BYTE_ORDER\n__LITTLE_ENDIAN\n__BIG_ENDIAN\n" | \
  $CC -E -c - | grep '^[14]' >$tmpfile

exec 3<> $tmpfile
read order <&3
read little <&3
read big <&3
exec 3>&-
tmpfile_stop

[ -z "$order" ] && fail "failed to determine endianness"
if [ "$order" == "$little" ]; then
  echo "little"
elif [ "$order" == "$big" ]; then
  echo "big"
else
  fail "endianness unknown"
fi

exit 0
