#ifndef __G_MACROS_H__
#define __G_MACROS_H__

/* GLIB - Library of useful routines for C programming
 * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GLib Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GLib at ftp://ftp.gtk.org/pub/gtk/.
 */

/*
 * This file contains all the defines/functions/macros needed to get
 * the ghash routines up and running. The code was extraced from different
 * source files in glib v2.20.4.
 */


#include "gtypes.h"

/* the following macro's & functions are OK*/

typedef unsigned int gsize;

#define offsetof(p_type,field) ((size_t)&(((p_type *)0)->field))
#define MAX(a, b)  (((a) > (b)) ? (a) : (b))
#define G_UNLIKELY(expr) (expr)

#ifndef	FALSE
#define	FALSE	(0)
#endif

#ifndef	TRUE
#define	TRUE	(!FALSE)
#endif

#define g_return_if_fail(operation) \
   if(!(operation)) return; 


#define g_return_val_if_fail(operation,value)  \
   if(!(operation)) return value;

#define GPOINTER_TO_UINT(v) \
    (unsigned int)(v)

typedef struct _GString		GString;

struct _GString
{
  gchar  *str;
  gsize len;    
  gsize allocated_len;
};

/* the following macro's/functions might cause problems, check this */

#include <stdlib.h>

#define g_new0(struct_type, n_structs)		\
    ((struct_type *) calloc ((gsize) (n_structs),(gsize) sizeof (struct_type)))

#define g_free	free

#define  g_slice_new(type)      \
    ((type*) calloc (1, sizeof (type)))

#define g_slice_free(type,struct)		\
    ((void) free (struct))


gint g_atomic_int_exchange_and_add (volatile gint *atomic, gint val);
void g_atomic_int_add (volatile gint *atomic, gint val);

/* Hash Functions
 */
gboolean g_str_equal (gconstpointer  v1,
                      gconstpointer  v2);
guint    g_str_hash  (gconstpointer  v);


gboolean g_int_equal (gconstpointer  v1,
                      gconstpointer  v2);
guint    g_int_hash  (gconstpointer  v);

/* This "hash" function will just return the key's address as an
 * unsigned integer. Useful for hashing on plain addresses or
 * simple integer values.
 * Passing NULL into g_hash_table_new() as GHashFunc has the
 * same effect as passing g_direct_hash().
 */
guint    g_direct_hash  (gconstpointer  v);
gboolean g_direct_equal (gconstpointer  v1,
                         gconstpointer  v2);


#endif /* __G_MACROS_H__ */


