#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "status.h"
#include "l2tp.h"

// keep up to date with "wacd_common.h"
#define WACD_PIPE "/var/run/wacd.pipe"
#define WACD_UPDATES "/var/run/wacd.updates"
#define BUF_LEN 128

void setStatus(enum L2TPStatus status, const char *msg) {
	int fd, res;
	char buf[BUF_LEN] = {};
	
	strcat(buf, "[L2TP;INTERFACE;");

	if (status == L2TP_UP) {
		strcat(buf, "UP");
	}
	else if (status == L2TP_DOWN) {
		strcat(buf, "DOWN");
	}
	else if (status == L2TP_INIT) {
		strcat(buf, "INIT");
	}
	else if (status == L2TP_STARTING) {
		strcat(buf, "STARTING");
	}
	else /* (status == PStatus_Error) */ {
		strcat(buf, "ERROR");
	}
	if (msg) {
		strcat(buf, "(");
		strcat(buf, msg);
		strcat(buf, ")");
	}
	strcat(buf, "]");

	fd = open(WACD_PIPE, O_WRONLY);
	if (fd < 0){
		log(LOG_DEBUG,"open(%s) failed, status update failed(%s)", WACD_PIPE, buf);
		return;
	}

	log(LOG_DEBUG,"sending status to wacdplugin(%s)", buf);
	/* Write to the pipe */
	res = write(fd, buf, strlen(buf));
	close(fd);

	/* update the l2tp status to wacd */
	fd = open(WACD_UPDATES, O_WRONLY);
	if (fd < 0){
		log(LOG_DEBUG,"open(%s) failed, wacd status update failed(%s)", WACD_PIPE, buf);
		return;
	}
	log(LOG_DEBUG,"sending status to wacd(%s)", buf);
	/* Write to the pipe */
	res = write(fd, buf, strlen(buf));
	close(fd);
}

void setPPPStatus(enum PPPStatus status, const char *msg) {
	int fd, res;
	char buf[BUF_LEN] = {};
	
	strcat(buf, "[PPP;INTERFACE;");

	if (status == PPP_UP) {
		strcat(buf, "UP]");
	}
	else if (status == PPP_DOWN) {
		strcat(buf, "DOWN]");
	}
	else if (status == PPP_INIT) {
		strcat(buf, "INIT]");
	}
	else if (status == PPP_STARTING) {
		strcat(buf, "STARTING]");
	}
	else /* (status == PStatus_Error) */
	{
		strcat(buf, "ERROR");
		if (msg) {
			strcat(buf, "(");
			strcat(buf, msg);
			strcat(buf, ")");
		}
		strcat(buf, "]");
	}

	fd = open(WACD_PIPE, O_WRONLY);
	if (fd < 0){
		log(LOG_DEBUG,"open(%s) failed, status update failed(%s)", WACD_PIPE, buf);
		return;
	}
	log(LOG_DEBUG,"sending ppp status to wacdplugin(%s)", buf);
	/* Write to the pipe */
	res = write(fd, buf, strlen(buf));
	close(fd);
}
